package com.winhc.bigdata.calc;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import org.apache.commons.lang.StringUtils;

/**
 * @description: 货币换算工具类
 * @author: yujie
 * @date 2019/7/1 18:48
 */
public class MoneyConversion {

    public static void main(String[] args) {
        double i = convert("500万元人民币",MoneyUnit.元);
        double divValue = NumberUtil.div(i, 14*10000, 2);
        System.out.println(divValue);


    }

    /**
     * 精确单位
     */
    public enum MoneyUnit {
        元, 万元;
    }
    /**
     * 格式换算，例：100万元美元
     * @param money
     * @return
     */
    public static double convert(String money, MoneyUnit unit) {
        if (StringUtils.isBlank(money)) {
            return 0;
        }
        String number = getNumber(money);
        if (StringUtils.isBlank(number)) {
            return 0;
        }
        Number num = NumberUtil.parseNumber(number);
        if (money.contains("亿")) {
            num = NumberUtil.mul(num, 10000);
        } else if (money.contains("千万")) {
            num = NumberUtil.mul(num, 1000);
        } else if (money.contains("百万")) {
            num = NumberUtil.mul(num, 100);
        } else if (money.contains("万")) {
            // do nothing
        } else if (money.contains("元")) {
            num = NumberUtil.div(num,10000,2);
        }

        if (unit == MoneyUnit.元) {
            num = NumberUtil.mul(num,10000);
        }

        if (money.contains("港元")) {
            num = NumberUtil.mul(num,0.8840);
        } else if (money.contains("澳元")) {
            num = NumberUtil.mul(num, 4.7277);
        } else if (money.contains("台币")) {
            num = NumberUtil.mul(num, 0.2205);
        } else if (money.contains("欧元")) {
            num = NumberUtil.mul(num, 7.6202);
        } else if (money.contains("美元")) {
            num = NumberUtil.mul(num, 6.8698);
        } else if (money.contains("英镑")) {
            num = NumberUtil.mul(num, 8.8835);
        } else if (money.contains("韩元")) {
            num = NumberUtil.mul(num, 0.0058);
        } else if (money.contains("日元")) {
            num = NumberUtil.mul(num, 0.0630);
        } else if (money.contains("卢布")) {
            num = NumberUtil.mul(num, 0.1044);
        } else if (money.contains("铢")) {
            num = NumberUtil.mul(num, 0.2129);
        }
        num = NumberUtil.roundDown(num,2);
        return num.doubleValue();
    }

    /**
     * 获取字符串中的数字，必须是连续的，默认返回0
     * @param numberStr
     * @return
     */
    public static String getNumber(String numberStr) {
        if (StringUtils.isBlank(numberStr)) {
            return "0";
        }
        StringBuilder numberBuilder = new StringBuilder();
        for (int index = 0; index < numberStr.length(); index++) {
            if (Character.isDigit(numberStr.charAt(index))) {
                numberBuilder.append(numberStr.charAt(index));
            } else if (numberStr.charAt(index) == '.'
                    && StringUtils.isNotBlank(numberBuilder.toString())
                    && !numberBuilder.toString().contains(".")) {
                numberBuilder.append(".");
            } else if (numberStr.charAt(index) == '-') {
                numberBuilder.append(numberStr.charAt(index));
            }
        }
        if (StrUtil.isNotBlank(numberBuilder.toString())) {
            return numberBuilder.toString();
        } else {
            return "0";
        }
    }


}
