package com.winhc.bigdata.spark.implicits

import org.json4s.{Formats, NoTypeHints}
import org.json4s.jackson.Serialization
import org.json4s.jackson.Serialization.read

/**
 * @Author: XuJiakai
 * @Date: 2020/7/14 08:45
 * @Description:
 */
object BaseHelper {

  implicit class StringEnhancer(str: String) extends Serializable {
    private val pattern = "^([0-9]*\\.?[0-9]+)[^0-9\\.]*".r

    import com.winhc.bigdata.spark.implicits.RegexUtils._


    def isRegCapital(): Boolean = pattern matches str


    def getOrNull(): String = if (str == null) "\"\"" else "\"" + str + "\""


    def toRegCapital(): Double = str match {
      case pattern(v) => {
        v.toDouble
      }
      case _ => {
        -1
      }
    }


    def toAnyMap(): Map[String, Any] = {
      implicit val formats: AnyRef with Formats = Serialization.formats(NoTypeHints)
      if (str == null) {
        null
      } else
        read[Map[String, Any]](str)
    }
  }


  implicit class SeqEnhancer(arr: Seq[String]) extends Serializable {
    def mkStringOrNull(): String = {
      if (arr.isEmpty) null else arr.mkString(",")
    }

    def mkStringOrNull(sep: String): String = arr.mkStringOrNull("", sep, "")

    def mkStringOrNull(start: String, sep: String, end: String): String = {
      if (arr.isEmpty) null else arr.mkString(start, sep, end)
    }
  }
}
