package com.winhc.bigdata.spark.jobs

import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.{LoggingUtils, SparkUtils}
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @Author: XuJiakai
 * @Date: 2020/6/5 14:28
 * @Description:
 */
object CompanyIndexSave2Es {

  case class CompanyIndexSave2Es_all_inc(s: SparkSession, project: String) extends LoggingUtils with BaseFunc {
    @(transient@getter) val spark: SparkSession = s

    val tmp_table = "xjk_company_test_0721_step2"

    def calc(): Unit = {
      import com.winhc.bigdata.spark.implicits.CompanyIndexSave2EsHelper._
      val all_company_max_ds = getLastPartitionsOrElse(s"${project}.ads_company", "0")
      val code = code2Name()

      println(
        s"""
           |DROP TABLE IF EXISTS winhc_eci_dev.$tmp_table
           |""".stripMargin)
      println(
        s"""
           |CREATE TABLE IF NOT EXISTS winhc_eci_dev.$tmp_table AS
           |SELECT  ${companyIndexFields.map(f => if (f.eq("estiblish_time")) "date_format(tmp.estiblish_time,'yyyy-MM-dd') estiblish_time" else "tmp." + f).mkString(",")}
           |FROM    (
           |            SELECT  ${companyIndexFields.mkString(",")},update_time
           |                    ,ROW_NUMBER() OVER(PARTITION BY cid ORDER BY update_time DESC ) AS num
           |            FROM    (
           |                        SELECT  ${companyIndexFields.mkString(",")}
           |                                ,update_time
           |                        FROM    winhc_eci_dev.ads_company
           |                        WHERE   ds = $all_company_max_ds
           |                        UNION ALL
           |                        SELECT  ${companyIndexFields.mkString(",")}
           |                                ,update_time
           |                        FROM    winhc_eci_dev.inc_ads_company
           |                        WHERE   ds > $all_company_max_ds
           |                    )
           |        ) AS tmp
           |WHERE   tmp.num = 1
           |""".stripMargin)
      sql(
        s"""
           |SELECT  *
           |FROM    winhc_eci_dev.$tmp_table
           |""".stripMargin)
        .companyIndexSave2Es(code._1, code._2)
    }
  }

  def main(args: Array[String]): Unit = {
    val project = "winhc_eci_dev"
    val map = EsConfig.getEsConfigMap ++ mutable.Map(
      "spark.hadoop.odps.project.name" -> project,
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "3800"
    )

    val spark = SparkUtils.InitEnv("CompanyIndexSave2Es", map)

    CompanyIndexSave2Es_all_inc(spark, project).calc

    spark.stop()
  }
}
