package com.winhc.bigdata.spark.jobs

import com.winhc.bigdata.spark.config.HBaseConfig
import com.winhc.bigdata.spark.utils.{CompanyNameMappingUtil, SparkUtils}
import org.apache.hadoop.hbase.TableName
import org.apache.hadoop.hbase.spark.HBaseContext
import org.apache.hadoop.hbase.spark.HBaseRDDFunctions._
import org.apache.hadoop.hbase.util.Bytes
import org.apache.spark.internal.Logging
import org.apache.spark.sql.Row
import org.apache.spark.sql.types.{StringType, StructField, StructType}

import scala.collection.mutable

/**
 * @Author: XuJiakai
 * @Date: 2020/5/26 10:41
 * @Description: 公司名称映射关系计算
 */
@deprecated
object CompanyNameMappingPro extends Logging {
  val f_bytes: Array[Byte] = Bytes.toBytes("f")
  val cname_bytes: Array[Byte] = Bytes.toBytes("name")
  val current_cid_bytes: Array[Byte] = Bytes.toBytes("current_cid")

  def main(args: Array[String]): Unit = {
    val map = mutable.Map[String, String](
      "spark.hadoop.odps.spark.local.partition.amt" -> "10"
    )
//    import com.winhc.bigdata.spark.utils.BaseUtil.getExecutorConfigOrExit
//    val map = getExecutorConfigOrExit(args)

    val hbaseKVTable = "company_name_kv"
    val inputTable = "ods_company"
    val resultTable = "company_name_mapping_pro_v2"
    val tmpResultTable = "company_name_mapping_pro_tmp_2"


    val spark = SparkUtils.InitEnv("CompanyNameMapping",map)
    import spark._
    /**
     * 写hbase,供查询
     */
    val jobConf = HBaseConfig.HBaseOutputJobConf(hbaseKVTable)

    /*  val df = sql(s"select cid,name,current_cid from $inputTable where cid is not null")
      df.rdd.map(row => {
        val id = row(0).asInstanceOf[Long].toString
        val name = row(1).asInstanceOf[String]
        val current_cid = row(2).asInstanceOf[Long].toString
        val put = new Put(Bytes.toBytes(id))
        if (name != null) {
          put.addColumn(f_bytes, cname_bytes, Bytes.toBytes(name))
        }
        if (!"0".equals(current_cid)) {
          put.addColumn(f_bytes, current_cid_bytes, Bytes.toBytes(current_cid))
        }
        (new ImmutableBytesWritable, put)
      }).saveAsHadoopDataset(jobConf)*/

    logInfo("save hbase success!")


    val df_old = sql(
      s"""
         |SELECT  cid
         |        ,name
         |        ,current_cid
         |FROM    $inputTable
         |WHERE   current_cid IS NOT NULL AND cid IS NOT NULL
         |""".stripMargin).repartition(500)

    val hbaseContext = new HBaseContext(spark.sparkContext, jobConf)
    /**
     * 查hbase,找到最新的公司id
     */
    val tmp_rdd = df_old.rdd.hbaseMapPartitions(hbaseContext, (f, con) => {
      val table = con.getTable(TableName.valueOf(hbaseKVTable))
      val rdd_par = f.map(record => {
        try {
          val cid = record.getLong(0).toString
          val name = record.getString(1)
          val current_cid = record.getAs[Long](2).toString
          val (res_cid, res_name) = CompanyNameMappingUtil.getCurrentIdAndName(table, current_cid)
          Row(cid, name, current_cid, res_cid, res_name)
        } catch {
          case e: Exception => {
            logWarning(record.toString())
            logError(e.getMessage, e)
          }
            null
        }
      })
//      table.close()
      rdd_par
    }).filter(_ != null)

    val schema = StructType(Array(
      StructField("cid", StringType),
      StructField("cname", StringType),
      StructField("current_cid", StringType),
      StructField("new_cid", StringType),
      StructField("new_cname", StringType)))

    val tmp_df = spark.createDataFrame(tmp_rdd, schema)

    tmp_df.write.mode("overwrite").insertInto(tmpResultTable)
//        tmp_df.createTempView(tmpResultTable) //注册临时表

    logInfo("new_cid add success")

    /**
     * 生成id的曾（现）用名列表
     */
    import spark.implicits._
    val id_other = sql(
      s"""
         |SELECT  new_cid
         |        ,concat_ws(
         |            ';'
         |            ,collect_set(concat_ws(':', cid, cname))
         |        ) other_id_name
         |FROM    $tmpResultTable
         |GROUP BY new_cid
         |""".stripMargin)
      .flatMap(r => {
        try {
          val new_cid = r.getAs[String]("new_cid")
          val other_id_name = r.getAs[String]("other_id_name")
          val other_id_name_map = other_id_name.split(";").map(str => (str.split(":")(0)) -> str.split(":")(1)).toMap

          other_id_name
            .split(";")
            .map(str => (str.split(":")(0), other_id_name_map)).:+((new_cid, other_id_name_map)).toSeq
        } catch {
          case e: Exception => {
            logError(e.getMessage)
          }
            logError(r.getAs[String]("new_cid"))
            logError(r.getAs[String]("other_id_name"))
            null
        }
      }).filter(_ != null).toDF("cid", "other_id_name")

    id_other.printSchema()
    id_other.show()

    /**
     * 补全数据，加入未发生过名称变更的公司
     */
    var tmp_df2 = sql(
      s"""
         |SELECT  cid
         |        ,name as cname
         |        ,current_cid
         |FROM    $inputTable
         |WHERE    current_cid IS NULL AND cid IS NOT NULL
         |""".stripMargin)
      .toDF("cid", "cname", "current_cid")
    tmp_df2 = tmp_df2.withColumn("new_cid", tmp_df2.col("cid"))
    tmp_df2 = tmp_df2.withColumn("new_cname", tmp_df2.col("cname"))

    val tmp_df3 = tmp_df.union(tmp_df2)

    /**
     * 生成结果表
     */
    val resultDf =
      tmp_df3.join(id_other, Seq("cid"), "left").select(
        "cid"
        , "cname"
        , "other_id_name"
        , "new_cid"
        , "new_cname"
      )
    resultDf.printSchema()
    resultDf.write.mode("append").insertInto(resultTable)

    logInfo("CompanyNameMapping success")
    spark.stop()
  }
}
