package com.winhc.bigdata.spark.jobs

import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{LoggingUtils, SparkUtils}
import org.apache.spark.internal.Logging
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @Author: XuJiakai
 * @Date: 2020/5/21 11:18
 * @Description: 单表写入摘要
 */
object CompanySummaryBySingle extends Logging {


  case class CompanySummaryBySingleUtil(s: SparkSession,
                                        project: String
                                       ) extends LoggingUtils with Logging with BaseFunc {
    @(transient@getter) val spark: SparkSession = s

    def all(tableName: String, out: String = null): Unit = {
      val lastDs = getLastPartitionsOrElse(s"$project.ads_$tableName", "0")

      if (out != null) {
        sql(
          s"""
             |CREATE TABLE IF NOT EXISTS winhc_eci_dev.$out
             |(
             |    cid  STRING COMMENT 'FIELD'
             |    ,$tableName BIGINT COMMENT 'FIELD'
             |)
             |COMMENT '临时摘要'
             |""".stripMargin)
      }

      val cols = Seq("rowkey", "ds")
      val df =

        sql(
          s"""
             |SELECT  split(rowkey,'_')[0] AS cid
             |        ,COUNT(1) as ${tableName}
             |FROM    (
             |            SELECT  t1.*
             |                    ,ROW_NUMBER() OVER(PARTITION BY rowkey ORDER BY ds DESC) AS num
             |            FROM    (
             |                        SELECT  ${cols.mkString(",")}
             |                        FROM    winhc_eci_dev.ads_${tableName}
             |                        WHERE   ds = ${lastDs}
             |                        ${
            spark.catalog.tableExists(s"winhc_eci_dev.inc_ads_${tableName}") match {
              case true => {
                s"""
                   |                        UNION ALL
                   |                        SELECT  ${cols.mkString(",")}
                   |                        FROM    winhc_eci_dev.inc_ads_${tableName}
                   |                        WHERE   ds > $lastDs
                   |""".stripMargin
              }
              case _ => ""
            }
          }
             |                    ) AS t1
             |        ) AS t2
             |WHERE   t2.num = 1
             |GROUP BY split(rowkey,'_')[0]
             |""".stripMargin)

      if (out != null) {
        df.createTempView("xjk_tmp_summary_test")
        sql(
          s"""
             |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE winhc_eci_dev.$out
             |SELECT cid,$tableName
             |FROM
             |    xjk_tmp_summary_test
             |""".stripMargin)
      } else {
        import com.winhc.bigdata.spark.implicits.DataFrame2HBaseHelper._;
        df.save2HBase("TEST_COMPANY_SUMMARY", "CID", Seq(tableName.toUpperCase()))
      }
    }
  }


  def main(args: Array[String]): Unit = {

    val Array(project, tableName) = args

    println(
      s"""
         |project: $project
         |tableName: $tableName
         |""".stripMargin)

    val config = EsConfig.getEsConfigMap ++ mutable.Map(
      "spark.hadoop.odps.project.name" -> project,
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "100"
    )

    val spark = SparkUtils.InitEnv("CompanySummaryBySingle", config)

    CompanySummaryBySingleUtil(spark, "winhc_eci_dev").all(tableName, out = "xjk_tmp_summ")

    spark.stop()
  }
}
