package com.winhc.bigdata.spark.jobs

import com.winhc.bigdata.spark.udf.CourtRank
import com.winhc.bigdata.spark.utils.{BaseUtil, SparkUtils}
import scala.collection.mutable
import com.winhc.bigdata.spark.udf.{BaseFunc, CompanyMapping}
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.LoggingUtils
import org.apache.spark.sql.SparkSession



object JudicialCaseRelationPreCourtRegister {
  def main(args: Array[String]): Unit = {
    val project = "winhc_eci_dev"
    println(
      s"""
         |project: $project
         |""".stripMargin)

    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> s"$project",
      "spark.hadoop.odps.spark.local.partition.amt" -> "10000"
    )
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)

    val r = JudicialCaseRelationPre2(spark, project)
    r.precalc()
    //r.calc()
    spark.stop()
  }
}

case class JudicialCaseRelationPre2(s: SparkSession, project: String
                                   ) extends LoggingUtils with CompanyMapping with BaseFunc with CourtRank {
  override protected val spark: SparkSession = s


  private def getStrToMap(cols: Seq[String]): String = {
    val set = cols.toSet
    val str = set.map(e => {
      s"concat_ws('\001','$e',cast($e as string))"
    }).mkString(",")
    s"str_to_map(concat_ws('\002',$str),'\002','\001')"
  }

  def precalc(): Unit = {
    prepareFunctions(spark)
    val t1 = s"$project.inc_ads_company_court_register"
    val t1_ds = BaseUtil.getPartion(t1, spark)

    //立案信息预处理
    sql(
      s"""
         |insert ${if (isWindows) "INTO" else "OVERWRITE"} table $project.ads_judicial_case_relation_pre partition(ds='$t1_ds',tn='court_register')
         |select
         |  judicase_id
         |  ,flag
         |  ,title
         |  ,case_type
         |  ,case_reason
         |  ,case_no
         |  ,court_name
         |  ,case_stage
         |  ,yg_name
         |  ,bg_name
         |  ,date
         |  ,detail_id
         |  ,case_amt
         |from (
         |     select
         |     *,row_number() over(partition by detail_id order by date desc) num
         |     from (
         |           select
         |            md5(cleanup(case_no)) as judicase_id
         |            ,"8" as flag
         |            ,concat_ws('',plaintiff,'与',defendant,case_reason) as title
         |            ,concat_ws('',case_type(case_no)) as case_type
         |            ,case_reason
         |            ,case_no
         |            ,court as court_name
         |            ,concat_ws('',case_stage(case_no)) as case_stage
         |            ,plaintiff as yg_name
         |            ,defendant as bg_name
         |            ,filing_date as date
         |            ,rowkey as detail_id
         |            ,0.0 as case_amt
         |      from $project.inc_ads_company_court_register
         |      where length(case_no) > 0 and ds > '0'
         |      union all
         |      select
         |            md5(cleanup(case_no)) as judicase_id
         |            ,"8" as flag
         |            ,concat_ws('',plaintiff,'与',defendant,case_reason) as title
         |            ,concat_ws('',case_type(case_no)) as case_type
         |            ,case_reason
         |            ,case_no
         |            ,court as court_name
         |            ,concat_ws('',case_stage(case_no)) as case_stage
         |            ,plaintiff as yg_name
         |            ,defendant as bg_name
         |            ,filing_date as date
         |            ,rowkey as detail_id
         |            ,0.0 as case_amt
         |      from $project.ads_company_court_register
         |      where length(case_no) > 0 and ds > '0'
         |     )
         |   )
         |where num = 1
         |""".stripMargin).show(10, false)
  }



}