package com.winhc.bigdata.spark.jobs.chance

import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{LoggingUtils, SparkUtils}
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @Author: XuJiakai
 * @Date: 2020/7/13 11:14
 * @Description: 全量文书债权表补充数据
 */
object creditor_info_add_other {

  case class add_cols_other(s: SparkSession, project: String) extends LoggingUtils with BaseFunc {
    @(transient@getter) val spark: SparkSession = s

    def prefix(ds: String, targetTable: String): Unit = {
      company_split()
      area_code()
      tyc_split()
      code2Name()

      sql(
        s"""
           |DROP TABLE IF EXISTS winhc_eci_dev.$targetTable
           |""".stripMargin)

      sql(
        s"""
           |CREATE TABLE IF NOT EXISTS `winhc_eci_dev`.`$targetTable` (
           |  `id` BIGINT,
           |  `case_id` BIGINT,
           |  `case_no` STRING,
           |  `case_type` STRING,
           |  `case_reason` STRING,
           |  `case_stage` STRING,
           |  `case_amt` DOUBLE,
           |  `ys_yg` STRING,
           |  `ys_bg` STRING,
           |  `judge_date` DATETIME,
           |  `zhixing_date` STRING,
           |  `zhixing_result` STRING,
           |  `curr_stage` STRING,
           |  `curr_date` STRING,
           |  `curr_result` STRING,
           |  `ys_yg_cid` STRING COMMENT '一审原告cid',
           |  `ys_bg_cid` STRING COMMENT '一审被告cid',
           |  `yg_reg_status` STRING,
           |  `yg_province_code` STRING,
           |  `yg_province_name` STRING,
           |  `yg_city_code` STRING,
           |  `yg_city_name` STRING,
           |  `yg_county_code` STRING,
           |  `yg_county_name` STRING,
           |  `yg_reg_location` STRING,
           |  `yg_estiblish_time` STRING,
           |  `yg_category_code` STRING,
           |  `yg_category_first` STRING,
           |  `yg_category_second` STRING,
           |  `yg_category_third` STRING,
           |  `yg_reg_capital` STRING,
           |  `yg_phones` STRING,
           |  `yg_emails` STRING,
           |  `bg_reg_status` STRING,
           |  `bg_province_code` STRING,
           |  `bg_province_name` STRING,
           |  `bg_city_code` STRING,
           |  `bg_city_name` STRING,
           |  `bg_county_code` STRING,
           |  `bg_county_name` STRING,
           |  `bg_reg_location` STRING,
           |  `bg_estiblish_time` STRING,
           |  `bg_category_code` STRING,
           |  `bg_category_first` STRING,
           |  `bg_category_second` STRING,
           |  `bg_category_third` STRING,
           |  `bg_reg_capital` STRING,
           |  `bg_phones` STRING,
           |  `bg_emails` STRING,
           |  `deleted` BIGINT
           |  )
           |""".stripMargin)
      sql(
        s"""
           |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE winhc_eci_dev.$targetTable
           |SELECT  id
           |        ,case_id
           |        ,case_no
           |        ,case_type
           |        ,case_reason
           |        ,case_stage
           |        ,case_amt
           |        ,ys_yg_xjk AS ys_yg
           |        ,ys_bg_xjk AS ys_bg
           |        ,judge_date
           |        ,zhixing_date
           |        ,zhixing_result
           |        ,curr_stage
           |        ,curr_date
           |        ,curr_result
           |        ,'' AS ys_yg_cid
           |        ,'' AS ys_bg_cid
           |        ,'' AS yg_reg_status
           |        ,'' AS yg_province_code
           |        ,'' AS yg_province_name
           |        ,'' AS yg_city_code
           |        ,'' AS yg_city_name
           |        ,'' AS yg_county_code
           |        ,'' AS yg_county_name
           |        ,'' AS yg_reg_location
           |        ,'' AS yg_estiblish_time
           |        ,'' AS yg_category_code
           |        ,'' AS yg_category_first
           |        ,'' AS yg_category_second
           |        ,'' AS yg_category_third
           |        ,'' AS yg_reg_capital
           |        ,'' AS yg_phones
           |        ,'' AS yg_emails
           |        ,'' AS bg_reg_status
           |        ,'' AS bg_province_code
           |        ,'' AS bg_province_name
           |        ,'' AS bg_city_code
           |        ,'' AS bg_city_name
           |        ,'' AS bg_county_code
           |        ,'' AS bg_county_name
           |        ,'' AS bg_reg_location
           |        ,'' AS bg_estiblish_time
           |        ,'' AS bg_category_code
           |        ,'' AS bg_category_first
           |        ,'' AS bg_category_second
           |        ,'' AS bg_category_third
           |        ,'' AS bg_reg_capital
           |        ,'' AS bg_phones
           |        ,'' AS bg_emails
           |        ,CASE (zhixing_result = 2 OR( zhixing_result IS NULL AND curr_result = '胜')) WHEN TRUE THEN 0 ELSE 1 END AS deleted
           |FROM    winhc_eci_dev.ods_creditor_info
           |LATERAL VIEW explode(company_split(ys_bg)) a AS ys_bg_xjk
           |LATERAL VIEW explode(company_split(ys_yg)) b AS ys_yg_xjk
           |WHERE   ds = '$ds'
           |AND     yg_type = '企业'
           |AND     bg_type = '企业'
           |AND     LENGTH(ys_yg_xjk) > 4
           |AND     LENGTH(ys_bg_xjk) > 4
           |""".stripMargin)
    }

    def suffix(targetTable: String): Unit = {
      val cols = getColumns(s"winhc_eci_dev.$targetTable").map("tmp."+_).mkString(",")
      sql(
        s"""
           |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE winhc_eci_dev.$targetTable
           |SELECT  $cols
           |FROM    (
           |            SELECT  a.*
           |                    ,row_number() OVER (PARTITION BY a.id,a.ys_yg_cid,a.ys_bg_cid ORDER BY id DESC) c
           |            FROM    (
           |                        SELECT  *
           |                        FROM    winhc_eci_dev.$targetTable
           |                        WHERE   ys_yg_cid IS NOT NULL
           |                        AND     ys_bg_cid IS NOT NULL
           |                        AND     ys_yg_cid <> ''
           |                        AND     ys_bg_cid <> ''
           |                    ) AS a
           |        ) AS tmp
           |WHERE   tmp.c = 1
           |""".stripMargin)

    }

    def calc(pre: String, table: String): Unit = {
      val fields = Seq(
        "reg_status", "province_code", "city_code", "county_code", "province_name", "city_name", "county_name", "reg_location", "estiblish_time", "category_code",
        "category_first", "category_second", "category_third", "reg_capital", "phones", "emails"
      )

      var org_name = s"ys_${pre}"
      org_name = org_name.substring(0, org_name.length - 1)

      val cid = "ys_" + pre + "cid"
      val fs = fields.map(pre + _).toSet ++ Set(cid)

      val str = getColumns(s"winhc_eci_dev.$table").map(f => {
        if (fs.contains(f)) {
          if(f.endsWith("cid")){
            s"tt2.${f.substring(6)} as $f"
          }else{
            s"tt2.${f.substring(3)} as $f"
          }
        } else {
          s"tt1.$f"
        }
      }).mkString(",")

      sql(
        s"""
           |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE winhc_eci_dev.$table
           |SELECT  $str
           |FROM    (
           |            SELECT  *
           |            FROM    winhc_eci_dev.$table
           |            WHERE   $org_name IS NOT NULL
           |        ) AS tt1
           |LEFT JOIN (
           |              SELECT  cast(tmp.cid as string) cid
           |                      ,tmp.name
           |                      ,tmp.history_names
           |                      ,cast(tmp.current_cid as string) current_cid
           |                      ,cast(tmp.company_type as string) company_type
           |                      ,get_province_code(tmp.area_code) as province_code
           |                      ,get_province_name(tmp.area_code) as province_name
           |                      ,get_city_code(tmp.area_code) as city_code
           |                      ,get_city_name(tmp.area_code) as city_name
           |                      ,get_county_code(tmp.area_code) as county_code
           |                      ,get_county_name(tmp.area_code) as county_name
           |                      ,tmp.credit_code
           |                      ,cast(tmp.reg_status as string) as reg_status
           |                      ,tmp.reg_location
           |                      ,date_format(tmp.estiblish_time,'yyyy-MM-dd') estiblish_time
           |                      ,cast(tmp.lat as string) lat
           |                      ,cast(tmp.lng as String) lng
           |                      ,cast(tmp.category_code as string) category_code
           |                      ,get_category_first(cast(tmp.category_code as string)) category_first
           |                      ,get_category_second(cast(tmp.category_code as string)) category_second
           |                      ,get_category_third(cast(tmp.category_code as string)) category_third
           |                      ,tmp.reg_capital
           |                      ,cast(tmp.reg_capital_amount as string) reg_capital_amount
           |                      ,CONCAT_WS(',',tyc_split(tmp.phones)) phones
           |                      ,CONCAT_WS(',',tyc_split(tmp.emails)) emails
           |              FROM    (
           |                          SELECT  cid
           |                                  ,name
           |                                  ,history_names
           |                                  ,current_cid
           |                                  ,company_type
           |                                  ,credit_code
           |                                  ,reg_status
           |                                  ,area_code
           |                                  ,reg_location
           |                                  ,estiblish_time
           |                                  ,lat
           |                                  ,lng
           |                                  ,category_code
           |                                  ,reg_capital
           |                                  ,reg_capital_amount
           |                                  ,phones
           |                                  ,emails
           |                                  ,update_time
           |                                  ,ROW_NUMBER() OVER(PARTITION BY cid ORDER BY update_time DESC ) AS num
           |                          FROM    (
           |                                      SELECT  cid
           |                                              ,name
           |                                              ,history_names
           |                                              ,current_cid
           |                                              ,company_type
           |                                              ,credit_code
           |                                              ,reg_status
           |                                              ,area_code
           |                                              ,reg_location
           |                                              ,estiblish_time
           |                                              ,lat
           |                                              ,lng
           |                                              ,category_code
           |                                              ,reg_capital
           |                                              ,reg_capital_amount
           |                                              ,phones
           |                                              ,emails
           |                                              ,update_time
           |                                      FROM    winhc_eci_dev.ads_company
           |                                      WHERE   ds = 20200604
           |                                      UNION ALL
           |                                      SELECT  cid
           |                                              ,name
           |                                              ,history_names
           |                                              ,current_cid
           |                                              ,company_type
           |                                              ,credit_code
           |                                              ,reg_status
           |                                              ,area_code
           |                                              ,reg_location
           |                                              ,estiblish_time
           |                                              ,lat
           |                                              ,lng
           |                                              ,category_code
           |                                              ,reg_capital
           |                                              ,reg_capital_amount
           |                                              ,phones
           |                                              ,emails
           |                                              ,update_time
           |                                      FROM    winhc_eci_dev.inc_ads_company
           |                                      WHERE   ds > 20200604
           |                                  )
           |                      ) AS tmp
           |              WHERE   tmp.num = 1
           |          ) AS tt2
           |ON      tt1.$org_name = tt2.name
           |""".stripMargin)

    }
  }


  def main(args: Array[String]): Unit = {
    val project = "winhc_eci_dev"
    val map = EsConfig.getEsConfigMap ++ mutable.Map(
      "spark.hadoop.odps.project.name" -> project,
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "3800"
    )
    val Array(ds) = args
    //    val ds = "20200721"
    val spark = SparkUtils.InitEnv("add_cols_other", map)

    val table = "xjk_tmp_ads_cre_info_test_v3"

    val v = add_cols_other(spark, project)
    v.prefix(ds, table)

    for (pre <- Seq("yg_", "bg_")) {
      v.calc(pre, table)
    }

    v.suffix(table)
    spark.stop()
  }
}
