package com.winhc.bigdata.spark.jobs.chance

import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.udf.{BaseFunc, CaseChanceFunc}
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, SparkUtils}
import org.apache.spark.internal.Logging
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @Author: XuJiakai
 * @Date: 2020/7/15 17:25
 * @Description: 利好消息
 */
object eci_good_news {

  case class eci_good_news_handle(s: SparkSession, ds: String) extends LoggingUtils with BaseFunc with CaseChanceFunc with Logging {
    @(transient@getter) val spark: SparkSession = s

    val source_ads_change_extract = "ads_change_extract"
    val change_tn_list = Seq(
      "company_tm"
      , "company_patent_list"
      , "company_copyright_works_list"
      , "company_copyright_reg_list"
      , "company_land_publicity"
      , "company_land_announcement"
      , "company_bid_list"
      , "company_land_transfer"
      , "company_employment" //招聘
      , "company_certificate" //资质证书
    )

    val update_tn_list = Seq(
      "company_zxr_restrict"
      , "company_zxr_list"
      , "company_dishonest_info"
    )

    val target_ads_case_chance = "ads_case_chance_good_news"
    val target_ads_case_chance_element = "ads_case_chance_element_good_news"

    private val env = "prod"

    def company_ip(): Unit = {
      cleanup()
      json_utils()
      json_add_kv()
      map_2_json()
      chance_dynamic_type()
      val now_time = BaseUtil.atMonthsBefore(0)

      val eci_debtor_rel_ds = getLastPartitionsOrElse("winhc_eci.ads_eci_debtor_relation_v2", "0")

      val relation_cols = getColumns("winhc_eci.ads_eci_debtor_relation_v2").filter(!_.equals("ds"))

      val good_news_cols = getColumns(s"winhc_eci_dev.$source_ads_change_extract").filter(!_.equals("ds"))

      val df = sql(
        s"""
           |SELECT  *
           |FROM    (
           |            SELECT  ${relation_cols.map(n => s"$n as rel_$n").mkString(",")}
           |            FROM    winhc_eci.ads_eci_debtor_relation_v2
           |            WHERE   ds = '$eci_debtor_rel_ds'
           |            and deleted = 0
           |        ) AS t1
           |JOIN (
           |              SELECT  ${good_news_cols.map(n => s"$n as detail_$n").mkString(",")}
           |              FROM    winhc_eci_dev.$source_ads_change_extract
           |              WHERE   ds >= '${ds}'
           |              AND     tn in (${change_tn_list.map("'" + _ + "'").mkString(",")})
           |              AND     TYPE = 'insert'
           |              AND     months_between('$now_time',to_date(biz_date)) < 3
           |              UNION ALL
           |              SELECT  ${good_news_cols.map(n => s"$n as detail_$n").mkString(",")}
           |              FROM    winhc_eci_dev.$source_ads_change_extract
           |              WHERE   ds >= '${ds}'
           |              AND     tn in (${update_tn_list.map("'" + _ + "'").mkString(",")})
           |              AND     TYPE = 'update'
           |              AND     months_between('$now_time',to_date(biz_date)) < 3
           |              UNION ALL
           |              SELECT  ${good_news_cols.map(n => s"$n as detail_$n").mkString(",")}
           |              FROM    winhc_eci_dev.$source_ads_change_extract
           |              WHERE   ds >= '${ds}'
           |              AND     tn in ('company_court_open_announcement_list','wenshu_detail_combine')
           |              AND     TYPE = 'insert'
           |              AND     data['case_no'] like concat('%','恢','%')
           |              AND     months_between('$now_time',to_date(biz_date)) < 3
           |              UNION ALL
           |              SELECT  ${good_news_cols.map(n => s"$n as detail_$n").mkString(",")}
           |              FROM    winhc_eci_dev.$source_ads_change_extract
           |              WHERE   ds >= '${ds}'
           |              AND     tn in ('company_holder')
           |              AND     TYPE = 'insert'
           |              AND     months_between('$now_time',to_date(biz_date)) < 3
           |              UNION ALL
           |              SELECT  ${good_news_cols.map(n => s"$n as detail_$n").mkString(",")}
           |              FROM    winhc_eci_dev.$source_ads_change_extract
           |              WHERE   ds >= '${ds}'
           |              AND     tn in ('company_equity_info_list')
           |              AND     TYPE = 'insert'
           |              AND     data['type'] = '1'
           |              AND     months_between('$now_time',to_date(biz_date)) < 3
           |          ) AS t2
           |ON      t1.rel_bg_cid = t2.detail_cid
           |WHERE t1.rel_bg_name not like concat('%','银行','%') AND t1.rel_bg_name not like concat('%','保险','%')
           |""".stripMargin)
      df
        .cache()
        .createOrReplaceTempView("good_news_debtor_relation_view")


      sql(
        s"""
           |INSERT  OVERWRITE TABLE ${if (env.equals("dev")) "winhc_eci_dev" else "winhc_eci"}.$target_ads_case_chance_element PARTITION(ds='$ds')
           |SELECT  md5(cleanup(CONCAT_WS('',case_chance_id,case_chance_type,type,province,city,dynamic_time))) AS id
           |        ,CASE_CHANCE_ID
           |        ,TYPE
           |        ,province
           |        ,city
           |        ,county
           |        ,dynamic_time
           |        ,public_date
           |        ,CASE_CHANCE_TYPE
           |FROM    (
           |            SELECT  *
           |                    ,ROW_NUMBER() OVER(PARTITION BY CASE_CHANCE_ID,TYPE,PROVINCE,city ORDER BY CASE_CHANCE_ID) AS num
           |            FROM    (
           |                        SELECT  detail_rowkey AS CASE_CHANCE_ID
           |                                ,0 AS TYPE
           |                                ,get_table_type(detail_table_name) AS CASE_CHANCE_TYPE
           |                                ,rel_yg_province_code AS PROVINCE
           |                                ,rel_yg_city_code AS city
           |                                ,rel_yg_county_code AS county
           |                                ,detail_biz_date AS dynamic_time
           |                                ,detail_update_time AS public_date
           |                        FROM    good_news_debtor_relation_view
           |                        UNION ALL
           |                        SELECT  detail_rowkey AS CASE_CHANCE_ID
           |                                ,1 AS TYPE
           |                                ,get_table_type(detail_table_name) AS CASE_CHANCE_TYPE
           |                                ,rel_bg_province_code AS PROVINCE
           |                                ,rel_bg_city_code AS city
           |                                ,rel_bg_county_code AS county
           |                                ,detail_biz_date AS dynamic_time
           |                                ,detail_update_time AS public_date
           |                        FROM    good_news_debtor_relation_view
           |                    )
           |        ) AS t
           |WHERE   t.num = 1
           |""".stripMargin)

      sql(
        s"""
           |
           |INSERT OVERWRITE TABLE ${if (env.equals("dev")) "winhc_eci_dev" else "winhc_eci"}.$target_ads_case_chance PARTITION(ds='$ds')
           |SELECT  detail_rowkey AS case_chance_id
           |        ,detail_title AS title
           |        ,null AS plaintiff
           |        ,null AS defendant
           |        ,rel_bg_name AS company_name
           |        ,detail_cid AS cid
           |        ,json_add_str(detail_label,CONCAT_WS(',',get_json_kv('reg_capital',rel_bg_reg_capital),get_json_kv('province_code',rel_bg_province_code),get_json_kv('city_code',rel_bg_city_code),get_json_kv('county_code',rel_bg_county_code),get_json_kv('estiblish_time',rel_bg_estiblish_time),get_json_kv('category_code',rel_bg_category_code)  ,get_json_kv('province_name',rel_bg_province_name),get_json_kv('city_name',rel_bg_city_name),get_json_kv('county_name',rel_bg_county_name),get_json_kv('category_first',rel_bg_category_first),get_json_kv('category_second',rel_bg_category_second),get_json_kv('category_third',rel_bg_category_third)  )) AS tags
           |        ,detail_rowkey AS biz_id
           |        ,get_table_type(detail_table_name) AS type
           |        ,get_chance_dynamic_type(detail_table_name) AS dynamic_type
           |        ,map_2_json(detail_data) AS dynamic_content
           |        ,detail_biz_date AS dynamic_time
           |        ,detail_update_time AS public_date
           |FROM    (
           |            SELECT  *
           |                    ,ROW_NUMBER() OVER(PARTITION BY detail_rowkey ORDER BY detail_rowkey) AS num
           |            FROM    good_news_debtor_relation_view
           |        ) AS t
           |WHERE   t.num = 1
           |""".stripMargin)

    }
  }


  def main(args: Array[String]): Unit = {
    val Array(ds) = args

    val config = EsConfig.getEsConfigMap ++ mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_eci_dev",
      "spark.hadoop.odps.spark.local.partition.amt" -> "1000"
    )
    val spark = SparkUtils.InitEnv("eci_good_news", config)
    eci_good_news_handle(spark, ds).company_ip()
    spark.stop()
  }

}
