package com.winhc.bigdata.spark.jobs

import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils._
import org.apache.spark.internal.Logging
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable


/**
 * @Author: XuJiakai
 * @Date: 2020/7/29 11:18
 * @Description: 股权出质增量数据外理
 */
object inc_company_equity_info {


  case class IncCompanyEquityInfoUtils(s: SparkSession,
                                       project: String, //表所在工程名
                                       ds: String //此维度主键
                                      ) extends LoggingUtils with Logging with BaseFunc {
    @(transient@getter) val spark: SparkSession = s


    private val dedup = Seq("reg_number", "pledgor", "pledgee")

    def calc(): Unit = {
      cleanup()
      val cols = getColumns(s"winhc_eci_dev.ads_company_equity_info").diff(Seq("ds", "rowkey", "id"))

      var startPart = getLastPartitionsOrElse(s"winhc_eci_dev.inc_ads_company_equity_info", "0")
      val endPart = getLastPartitionsOrElse(s"winhc_eci_dev.inc_ods_company_equity_info", BaseUtil.getYesterday())

      if (startPart.equals(endPart)) {
        println("start partition = end partition!")
        startPart = getSecondLastPartitionOrElse(s"winhc_eci_dev.inc_ads_company_equity_info", "0")
      }

      sql(
        s"""
           |select  t1.id
           |        ,t1.cid
           |        ,t2.cname
           |        ,t1.base
           |        ,t1.reg_number
           |        ,t1.pledgor
           |        ,t1.certif_number_l
           |        ,t1.equity_amount
           |        ,t1.pledgee
           |        ,t1.certif_number_r
           |        ,t1.reg_date
           |        ,t1.state
           |        ,t1.pub_date
           |        ,t1.change_situation
           |        ,t1.cancel_date
           |        ,t1.cancel_reason
           |        ,t1.pledgor_type
           |        ,t1.pledgor_id
           |        ,t1.pledgee_type
           |        ,t1.pledgee_id
           |        ,t1.create_time
           |        ,t1.update_time
           |        ,t1.deleted
           |        from (
           |SELECT  md5(cleanup(CONCAT_WS('',${dedup.map("tmp." + _).mkString(",")}))) as id
           |        ,tmp.cid
           |        ,null as cname
           |        ,tmp.base
           |        ,tmp.reg_number
           |        ,tmp.pledgor
           |        ,tmp.certif_number_l
           |        ,tmp.equity_amount
           |        ,tmp.pledgee
           |        ,tmp.certif_number_r
           |        ,tmp.reg_date
           |        ,tmp.state
           |        ,tmp.pub_date
           |        ,tmp.change_situation
           |        ,tmp.cancel_date
           |        ,tmp.cancel_reason
           |        ,tmp.pledgor_type
           |        ,tmp.pledgor_id
           |        ,tmp.pledgee_type
           |        ,tmp.pledgee_id
           |        ,tmp.create_time
           |        ,tmp.update_time
           |        ,tmp.deleted
           |FROM    (
           |            SELECT  a.*
           |                    ,row_number() OVER (PARTITION BY cleanup(CONCAT_WS('',${dedup.map("a." + _).mkString(",")})) ORDER BY update_time DESC) c
           |            FROM    (
           |                        SELECT  *
           |                        FROM    winhc_eci_dev.inc_ods_company_equity_info
           |                        WHERE   ds > $startPart
           |                    ) as a
           |        ) AS tmp
           |WHERE   tmp.c = 1
           |) as t1
           |left join
           |(
           |SELECT  *
           |              FROM    winhc_eci_dev.base_company_mapping
           |              WHERE   ds = '${getLastPartitionsOrElse("winhc_eci_dev.base_company_mapping", "0")}'
           |) as t2
           |on t1.cid = t2.cid
           |""".stripMargin).createOrReplaceTempView("tmp_company_equity_info_all")


      sql(
        s"""
           |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE winhc_eci_dev.inc_ads_company_equity_info PARTITION(ds='$endPart')
           |SELECT  id
           |        ,${cols.mkString(",")}
           |FROM    tmp_company_equity_info_all
           |""".stripMargin)


      sql(
        s"""
           |SELECT  cid,current_cid as new_cid
           |FROM    winhc_eci_dev.inc_ods_company
           |WHERE   ds >= ${startPart}
           |AND     cid IS NOT NULL
           |AND     current_cid IS NOT NULL
           |GROUP BY cid,current_cid
           |""".stripMargin).createOrReplaceTempView("mapping")

      sql(
        s"""
           |SELECT   CONCAT_WS('_',t2.new_cid,split(t1.rowkey,'_')[1]) as rowkey
           |         ,t2.new_cid as cid
           |         ,${getColumns("winhc_eci_dev.ads_company_equity_info_list").diff(Seq("ds", "cid", "rowkey")).map("t1." + _).mkString(",")}
           |FROM    (
           |            SELECT  ${getColumns("winhc_eci_dev.ads_company_equity_info_list").diff(Seq("ds")).map("tmp." + _).mkString(",")}
           |            FROM    (
           |                        SELECT  t.*
           |                                ,ROW_NUMBER() OVER(PARTITION BY rowkey ORDER BY rowkey) AS c
           |                        FROM    (
           |                                    SELECT  *
           |                                    FROM    winhc_eci_dev.ads_company_equity_info_list
           |                                    WHERE   ds = ${getLastPartitionsOrElse("winhc_eci_dev.ads_company_equity_info_list", "0")}
           |                                    UNION ALL
           |                                    SELECT  *
           |                                    FROM    winhc_eci_dev.inc_ads_company_equity_info_list
           |                                    WHERE   ds > ${getLastPartitionsOrElse("winhc_eci_dev.ads_company_equity_info_list", "0")}
           |                                ) AS t
           |                    ) AS tmp
           |            WHERE   tmp.c = 1
           |        ) AS t1
           |JOIN    mapping as t2
           |ON      t1.cid = t2.cid
           |""".stripMargin).createOrReplaceTempView("replace_cid_tab")

      sql(
        s"""
           |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE winhc_eci_dev.inc_ads_company_equity_info_list PARTITION(ds='$endPart')
           |SELECT  CONCAT_WS('_',pledgee_id,id) as rowkey
           |        ,pledgee_id AS cid
           |        ,id AS main_id
           |        ,pledgee AS cname
           |        ,2 AS type
           |        ,cname AS target
           |        ,cid as target_cid
           |        ,pledgee
           |        ,pledgee_type
           |        ,pledgee_id
           |        ,certif_number_l
           |        ,pledgor
           |        ,pledgor_type
           |        ,pledgor_id
           |        ,certif_number_r
           |        ,equity_amount
           |        ,reg_date
           |        ,reg_number
           |        ,pub_date
           |        ,state
           |        ,deleted
           |FROM    tmp_company_equity_info_all
           |WHERE   pledgee_id IS NOT NULL
           |AND     pledgee_type == 2
           |
           |UNION ALL
           |
           |SELECT  CONCAT_WS('_',pledgor_id,id) as rowkey
           |        ,pledgor_id AS cid
           |        ,id AS main_id
           |        ,pledgor AS cname
           |        ,1 AS type
           |        ,cname AS target
           |        ,cid as target_cid
           |        ,pledgee
           |        ,pledgee_type
           |        ,pledgee_id
           |        ,certif_number_l
           |        ,pledgor
           |        ,pledgor_type
           |        ,pledgor_id
           |        ,certif_number_r
           |        ,equity_amount
           |        ,reg_date
           |        ,reg_number
           |        ,pub_date
           |        ,state
           |        ,deleted
           |FROM    tmp_company_equity_info_all
           |WHERE   pledgor_id IS NOT NULL
           |AND     pledgor_type == 2
           |
           |UNION ALL
           |
           |SELECT  CONCAT_WS('_',cid,id) as rowkey
           |        ,cid AS cid
           |        ,id AS main_id
           |        ,null AS cname
           |        ,0 AS type
           |        ,cname AS target
           |        ,cid as target_cid
           |        ,pledgee
           |        ,pledgee_type
           |        ,pledgee_id
           |        ,certif_number_l
           |        ,pledgor
           |        ,pledgor_type
           |        ,pledgor_id
           |        ,certif_number_r
           |        ,equity_amount
           |        ,reg_date
           |        ,reg_number
           |        ,pub_date
           |        ,state
           |        ,deleted
           |FROM    tmp_company_equity_info_all
           |WHERE   cid IS NOT NULL
           |
           |UNION ALL
           |
           |select * from replace_cid_tab
           |""".stripMargin)

      val outFields = getColumns("winhc_eci_dev.inc_ads_company_equity_info").map(_.toUpperCase)
      import com.winhc.bigdata.spark.implicits.DataFrame2HBaseHelper._
      sql(
        s"""
           |SELECT  *
           |FROM    winhc_eci_dev.inc_ads_company_equity_info
           |WHERE   ds = '$endPart'
           |""".stripMargin)
        .save2HBase("COMPANY_EQUITY_INFO", "id", outFields)
      val writeCols = getColumns("winhc_eci_dev.inc_ads_company_equity_info_list").diff(Seq("ds", "rowkey"))

      MaxComputer2Phoenix(spark
        , writeCols
        , "inc_ads_company_equity_info_list"
        , "COMPANY_EQUITY_INFO_LIST"
        , endPart
        , "CONCAT_WS('_',cid,main_id)").syn()

      CompanySummaryPro(s = spark
        , project = "winhc_eci_dev"
        , tableName = "company_equity_info_list"
        , cidField = "split(rowkey,'_')[0]"
        , groupByInfo = GroupByInfo(field = "type", value_alias = Seq(
          ("0", "company_equity_info_list_0")
          , ("1", "company_equity_info_list_1")
          , ("2", "company_equity_info_list_2")
        ))
      )
        .calc()

    }
  }


  def main(args: Array[String]): Unit = {
    val Array(project, ds) = args

    println(
      s"""
         |project: $project
         |ds: $ds
         |""".stripMargin)

    val config = EsConfig.getEsConfigMap ++ mutable.Map(
      "spark.hadoop.odps.project.name" -> project,
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "100"
    )
    val spark = SparkUtils.InitEnv("inc_company_equity_info", config)
    IncCompanyEquityInfoUtils(spark, project, ds).calc()
    spark.stop()
  }
}
