package com.winhc.bigdata.spark.jobs.judicial

import com.winhc.bigdata.spark.udf.{BaseFunc, CompanyMapping}
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, SparkUtils}
import org.apache.spark.sql.SparkSession

import scala.collection.mutable


object JudicialCaseRelationPre10 {
  def main(args: Array[String]): Unit = {
    val project = "winhc_eci_dev"
    println(
      s"""
         |project: $project
         |""".stripMargin)

    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> s"$project",
      "spark.hadoop.odps.spark.local.partition.amt" -> "1000"
    )
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)
    JudicialCaseRelationPre10(spark, project).precalc()
    spark.stop()
  }
}


//司法案件-被执行人数据预处理
case class JudicialCaseRelationPre10(s: SparkSession, project: String
                                   ) extends LoggingUtils with CompanyMapping with BaseFunc {
  override protected val spark: SparkSession = s

  def precalc(): Unit = {
    prepareFunctions(spark)
    case_no_trim_udf()
    //被执行人(企业)
    var lastDsIncAds = BaseUtil.getPartion(s"$project.inc_ads_company_zxr", spark)
    spark.sparkContext.setJobDescription(s"处理zxr($lastDsIncAds)")
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $project.ads_judicial_case_relation_pre PARTITION(ds='$lastDsIncAds',tn='zxr')
         |SELECT
         |        judicase_id
         |        ,flag
         |        ,title
         |        ,case_type
         |        ,case_reason
         |        ,case_no
         |        ,court_name
         |        ,case_stage
         |        ,yg_name
         |        ,bg_name
         |        ,DATE
         |        ,detail_id
         |        ,case_amt
         |FROM    (
         |        SELECT
         |                A.*
         |                ,COALESCE(C.judicase_id,md5(cleanup(case_no))) AS judicase_id
         |                ,row_number() OVER(PARTITION BY detail_id,A.case_no ORDER BY DATE DESC ) num
         |        FROM    (
         |                SELECT
         |                        "7" AS flag
         |                        ,concat_ws('',cname,'被执行人') AS title
         |                        ,concat_ws('',case_type(case_no)) AS case_type
         |                        ,NULL AS case_reason
         |                        ,case_no
         |                        ,court AS court_name
         |                        ,concat_ws('',case_stage(case_no)) AS case_stage
         |                        ,NULL AS yg_name
         |                        ,cname AS bg_name
         |                        ,case_create_time AS DATE
         |                        ,rowkey AS detail_id
         |                        ,exec_money AS case_amt
         |                FROM    $project.inc_ads_company_zxr
         |                WHERE   length(case_no) >0 AND ds> '0'
         |                UNION ALL
         |                SELECT
         |                        "7" AS flag
         |                        ,concat_ws('',cname,'被执行人') AS title
         |                        ,concat_ws('',case_type(case_no)) AS case_type
         |                        ,NULL AS case_reason
         |                        ,case_no
         |                        ,court AS court_name
         |                        ,concat_ws('',case_stage(case_no)) AS case_stage
         |                        ,NULL AS yg_name
         |                        ,cname AS bg_name
         |                        ,case_create_time AS DATE
         |                        ,rowkey AS detail_id
         |                        ,exec_money AS case_amt
         |                FROM    $project.ads_company_zxr
         |                WHERE   length(case_no) >0 AND ds> '0'
         |                UNION ALL
         |                SELECT
         |                        "-1" AS flag
         |                        ,concat_ws('',cname,'被执行人') AS title
         |                        ,concat_ws('',case_type(case_no)) AS case_type
         |                        ,NULL AS case_reason
         |                        ,gist_id as case_no
         |                        ,court AS court_name
         |                        ,concat_ws('',case_stage(case_no)) AS case_stage
         |                        ,NULL AS yg_name
         |                        ,cname AS bg_name
         |                        ,case_create_time AS DATE
         |                        ,rowkey AS detail_id
         |                        ,exec_money AS case_amt
         |                FROM    $project.inc_ads_company_zxr
         |                WHERE   length(gist_id) >0 AND ds> '0'
         |                UNION ALL
         |                SELECT
         |                        "-1" AS flag
         |                        ,concat_ws('',cname,'被执行人') AS title
         |                        ,concat_ws('',case_type(case_no)) AS case_type
         |                        ,NULL AS case_reason
         |                        ,gist_id as case_no
         |                        ,court AS court_name
         |                        ,concat_ws('',case_stage(case_no)) AS case_stage
         |                        ,NULL AS yg_name
         |                        ,cname AS bg_name
         |                        ,case_create_time AS DATE
         |                        ,rowkey AS detail_id
         |                        ,exec_money AS case_amt
         |                FROM    $project.ads_company_zxr
         |                WHERE   length(gist_id) >0 AND ds> '0'
         |                ) A
         |        LEFT JOIN (
         |                SELECT  rowkey,max(judicase_id) judicase_id
         |                FROM    $project.ads_judicial_case_relation_graph
         |                WHERE   flag = 'company_zxr'
         |                GROUP BY rowkey
         |                  ) C
         |        ON      A.detail_id = C.rowkey
         |        )
         |WHERE   num =1
         |""".stripMargin).show(10, false)


    //被执行人预处理(个人）
    val columns: Seq[String] = spark.table(s"$project.inc_ads_company_zxr_person").schema.map(_.name).filter(_!="flag")
    lastDsIncAds = BaseUtil.getPartion(s"$project.inc_ads_company_zxr_person", spark)
    //身份证补全表最新分区
    val personIdDs = BaseUtil.getPartion("ads_person_idcard_cloze", spark)
    is_id_card_udf()
    spark.sparkContext.setJobDescription(s"处理zxr_person($lastDsIncAds)")
    sql(
      s"""
         |insert ${if (isWindows) "INTO" else "OVERWRITE"} table $project.ads_judicial_case_relation_pre partition(ds='$lastDsIncAds',tn='zxr_person')
         |select
         |  judicase_id
         |  ,flag
         |  ,title
         |  ,case_type
         |  ,case_reason
         |  ,case_no
         |  ,court
         |  ,case_stage
         |  ,yg_name
         |  ,bg_name
         |  ,date
         |  ,detail_id
         |  ,case_amt
         |from (
         |      select
         |      COALESCE(C.judicase_id,md5(cleanup(A.case_no))) as judicase_id
         |      ,flag
         |      ,concat_ws('',A.cname,'被执行人') AS title
         |      ,concat_ws('',case_type(A.case_no)) as case_type
         |      ,NULL AS case_reason
         |      ,A.case_no
         |      ,court
         |      ,concat_ws('',case_stage(A.case_no)) as case_stage
         |      ,NULL as yg_name
         |      ,A.cname as bg_name
         |      ,case_create_time as date--目前天眼查数据没有执行日期，先以此代替
         |      ,A.rowkey as detail_id
         |      ,exec_money as case_amt
         |      ,row_number() over(partition by A.rowkey,A.case_no order by update_time desc) num
         |      from (
         |        select case_no,court,cname,case_create_time,rowkey,update_time,card,exec_money,"10" as flag
         |        from $project.ads_company_zxr_person
         |        where length(case_no) > 0 and ds>'0'
         |        union all
         |        select case_no,court,cname,case_create_time,rowkey,update_time,card,exec_money,"10" as flag
         |        from $project.inc_ads_company_zxr_person
         |        where length(case_no) > 0 and ds>'0'
         |        union all
         |        select gist_id as case_no,court,cname,case_create_time,rowkey,update_time,card,exec_money,"-1" as flag
         |        from $project.ads_company_zxr_person
         |        where length(gist_id) > 0 and ds>'0'
         |        union all
         |        select gist_id as case_no,court,cname,case_create_time,rowkey,update_time,card,exec_money,"-1" as flag
         |        from $project.inc_ads_company_zxr_person
         |        where length(gist_id) > 0 and ds>'0'
         |      ) A
         |      left join(
         |        select name, identity_num, case_no
         |        from $project.ads_person_idcard_cloze
         |        where ds=${personIdDs}
         |      ) B
         |      on A.cname=B.name AND A.case_no=B.case_no
         |      left join(
         |        SELECT  rowkey,max(judicase_id) judicase_id
         |        FROM $project.ads_judicial_case_relation_graph
         |        WHERE flag = 'company_zxr_person'
         |        GROUP BY rowkey
         |      ) C
         |      on A.rowkey=C.rowkey
         |      where is_id_card(A.card) OR is_id_card(B.identity_num)
         |   )
         |where num = 1
         |""".stripMargin).show(10, false)

  }
}