package com.winhc.bigdata.spark.jobs.judicial

import com.winhc.bigdata.spark.udf.{BaseFunc, CompanyMapping}
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, SparkUtils}
import org.apache.spark.sql.SparkSession

import scala.collection.mutable


object JudicialCaseRelationPre12 {
  def main(args: Array[String]): Unit = {
    val project = "winhc_eci_dev"
    println(
      s"""
         |project: $project
         |""".stripMargin)

    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> s"$project",
      "spark.hadoop.odps.spark.local.partition.amt" -> "1000"
    )
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)
    JudicialCaseRelationPre12(spark, project).precalc()
    spark.stop()
  }
}


//司法案件，终本数据预处理
case class JudicialCaseRelationPre12(s: SparkSession, project: String
                                   ) extends LoggingUtils with CompanyMapping with BaseFunc {
  override protected val spark: SparkSession = s

  def precalc(): Unit = {
    prepareFunctions(spark)
    case_no_trim_udf()
    //终本案件-企业
    var lastDsIncAds = BaseUtil.getPartion(s"$project.inc_ads_company_zxr_final_case", spark)
    spark.sparkContext.setJobDescription(s"处理company_zxr_final_case($lastDsIncAds)")
    sql(
      s"""
         |insert ${if (isWindows) "INTO" else "OVERWRITE"} table $project.ads_judicial_case_relation_pre partition(ds='$lastDsIncAds',tn='zxr_final_case')
         |select
         |  judicase_id
         |  ,flag
         |  ,title
         |  ,case_type
         |  ,case_reason
         |  ,case_no
         |  ,court_name
         |  ,case_stage
         |  ,yg_name
         |  ,bg_name
         |  ,date
         |  ,detail_id
         |  ,case_amt
         |from (
         |   select
         |      * ,
         |      row_number() over(partition by detail_id order by date desc) num
         |   from (
         |      select
         |         md5(cleanup(case_no)) as judicase_id
         |         ,"6" as flag
         |         ,concat_ws('',name,'终本案件') as title
         |         ,concat_ws('',case_type(case_no)) as case_type
         |         ,NULL as case_reason
         |         ,case_no
         |         ,court_name
         |         ,concat_ws('',case_stage(case_no)) as case_stage
         |         ,NULL as yg_name
         |         ,name as bg_name
         |         ,case_create_time as date
         |         ,rowkey as detail_id
         |         ,exec_amount as case_amt
         |      from $project.inc_ads_company_zxr_final_case
         |      where length(case_no) > 0 and ds > '0'
         |      union all
         |      select
         |         md5(cleanup(case_no)) as judicase_id
         |         ,"6" as flag
         |         ,concat_ws('',name,'终本案件') as title
         |         ,concat_ws('',case_type(case_no)) as case_type
         |         ,NULL as case_reason
         |         ,case_no
         |         ,court_name
         |         ,concat_ws('',case_stage(case_no)) as case_stage
         |         ,NULL as yg_name
         |         ,name as bg_name
         |         ,case_create_time as date
         |         ,rowkey as detail_id
         |         ,exec_amount as case_amt
         |      from $project.ads_company_zxr_final_case
         |      where length(case_no) > 0 and ds > '0'
         |     )
         |   )
         |where num = 1
         |""".stripMargin).show(10, false)


    //终本案件预处理(个人）
    val columns: Seq[String] = spark.table(s"$project.inc_ads_company_zxr_final_case_person").schema.map(_.name).filter(_!="flag")
    lastDsIncAds = BaseUtil.getPartion(s"$project.inc_ads_company_zxr_final_case_person", spark)
    //身份证补全表最新分区
    val personIdDs = BaseUtil.getPartion("ads_person_idcard_cloze", spark)
    is_id_card_udf()
    spark.sparkContext.setJobDescription(s"处理zxr_final_case_person($lastDsIncAds)")
    sql(
      s"""
         |insert ${if (isWindows) "INTO" else "OVERWRITE"} table $project.ads_judicial_case_relation_pre partition(ds='$lastDsIncAds',tn='zxr_final_case_person')
         |select
         |  judicase_id
         |  ,flag
         |  ,title
         |  ,case_type
         |  ,case_reason
         |  ,case_no
         |  ,court_name
         |  ,case_stage
         |  ,yg_name
         |  ,bg_name
         |  ,date
         |  ,detail_id
         |  ,case_amt
         |from (
         |      select
         |      md5(cleanup(A.case_no)) as judicase_id
         |      ,"12" as flag
         |      ,concat_ws('',A.name,'终本案件') AS title
         |      ,concat_ws('',case_type(A.case_no)) as case_type
         |      ,NULL AS case_reason
         |      ,A.case_no
         |      ,court_name
         |      ,concat_ws('',case_stage(A.case_no)) as case_stage
         |      ,NULL as yg_name
         |      ,A.name as bg_name
         |      ,case_create_time as date--目前天眼查数据没有执行日期，先以此代替
         |      ,rowkey as detail_id
         |      ,exec_amount as case_amt
         |      ,row_number() over(partition by rowkey order by update_time desc) num
         |      from (
         |        select case_no,court_name,name,case_create_time,rowkey,update_time,identity_num,exec_amount
         |        from $project.ads_company_zxr_final_case_person
         |        where length(case_no) > 0 and ds>'0'
         |        union all
         |        select case_no,court_name,name,case_create_time,rowkey,update_time,identity_num,exec_amount
         |        from $project.inc_ads_company_zxr_final_case_person
         |        where length(case_no) > 0 and ds>'0'
         |      ) A
         |      left join(
         |        select name, identity_num, case_no
         |        from $project.ads_person_idcard_cloze
         |        where ds=${personIdDs}
         |      ) B
         |      on A.name=B.name AND A.case_no=B.case_no
         |      where is_id_card(A.identity_num) OR is_id_card(B.identity_num)
         |   )
         |where num = 1
         |""".stripMargin).show(10, false)

  }
}