package com.winhc.bigdata.spark.jobs.judicial

import com.winhc.bigdata.spark.udf.{BaseFunc, CompanyMapping}
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{LoggingUtils, SparkUtils}
import org.apache.spark.internal.Logging
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @Author: XuJiakai
 * @Date: 2020/9/27 11:41
 * @Description: 其它几个维度司法案件前置处理
 */
case class JudicialCaseRelationPre39(s: SparkSession,
                                     project: String //表所在工程名
                                    ) extends LoggingUtils with Logging with BaseFunc with CompanyMapping {
  @(transient@getter) val spark: SparkSession = s

  private val target_table = "ads_judicial_case_relation_pre"

  val flag_map = Map("company_dishonest_info" -> "3"
    , "company_dishonest_info_person" -> "9")


  def company_dishonest_info(is_inc: Boolean = true): Unit = {
    prepareFunctions(spark)
    case_no_trim_udf()


    def all(table_name: String): Unit = {
      val ads_ds = getLastPartitionsOrElse(s"$project.ads_$table_name", "0")
      val inc_last_ds = getLastPartitionsOrElse(s"$project.inc_ads_$table_name", "0")

      val intersectCols = getColumns(s"$project.ads_$table_name").toSet & getColumns(s"$project.inc_ads_$table_name").toSet


      sql(
        s"""
           |SELECT  rowkey,max(judicase_id) judicase_id
           |FROM    winhc_eci_dev.ads_judicial_case_relation_graph
           |WHERE   flag = '$table_name'
           |group by rowkey
           |""".stripMargin)
        .createOrReplaceTempView("all_judicial_mapping_tmp")

      sql(
        s"""
           |SELECT  md5(CONCAT(rowkey,'$table_name')) as judicase_id
           |        ,${flag_map(table_name)} as flag
           |        ,CONCAT('关于',name,'的失信信息') as title
           |        ,case_type(gist_dd) as case_type
           |        ,null as case_reason
           |        ,case_no_trim(gist_dd) as case_no
           |        ,court as court_name
           |        ,'执行' as case_stage
           |        ,null as yg_name
           |        ,name as bg_name
           |        ,pub_date as date
           |        ,rowkey as detail_id
           |        ,null as case_amt
           |        ,case_no_trim(case_no) as xjk_case_no
           |FROM    (
           |            SELECT  *
           |                    ,ROW_NUMBER() OVER(PARTITION BY rowkey ORDER BY ds DESC ) AS num
           |            FROM    (
           |                        SELECT  ${intersectCols.mkString(",")}
           |                        FROM    winhc_eci_dev.ads_$table_name
           |                        WHERE   ds = '$ads_ds'
           |                        AND     case_no_trim(gist_dd) is not null
           |                        UNION ALL
           |                        SELECT  ${intersectCols.mkString(",")}
           |                        FROM    winhc_eci_dev.inc_ads_$table_name
           |                        WHERE   ds > '$ads_ds'
           |                        AND     case_no_trim(gist_dd) is not null
           |                    ) AS t1
           |        ) AS t2
           |WHERE   t2.num = 1
           |""".stripMargin)
        .createOrReplaceTempView(s"all_${table_name}_tmp")

      sql(
        s"""
           |SELECT  coalesce(t2.judicase_id,t1.judicase_id) AS judicase_id
           |        ,t1.flag
           |        ,t1.title
           |        ,t1.case_type
           |        ,t1.case_reason
           |        ,t1.case_no
           |        ,t1.court_name
           |        ,t1.case_stage
           |        ,t1.yg_name
           |        ,t1.bg_name
           |        ,t1.date
           |        ,t1.detail_id
           |        ,t1.case_amt
           |        ,t1.xjk_case_no
           |FROM    (
           |            SELECT  *
           |            FROM    all_${table_name}_tmp
           |        ) AS t1
           |LEFT JOIN (
           |              SELECT  *
           |              FROM    all_judicial_mapping_tmp
           |          ) AS t2
           |ON      t1.detail_id = t2.rowkey
           |""".stripMargin)
        .createOrReplaceTempView(s"ok_table_$table_name")



      //fixme  替换司法案件id
      sql(
        s"""
           |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $project.ads_judicial_case_relation_pre PARTITION(ds='$inc_last_ds',tn='$table_name')
           |SELECT  judicase_id
           |        ,"-1" as flag
           |        ,title
           |        ,case_type
           |        ,case_reason
           |        ,case_no
           |        ,court_name
           |        ,case_stage
           |        ,yg_name
           |        ,bg_name
           |        ,date
           |        ,detail_id
           |        ,case_amt
           |FROM    ok_table_$table_name
           |UNION ALL
           |SELECT  judicase_id
           |        ,flag
           |        ,title
           |        ,case_type
           |        ,case_reason
           |        ,xjk_case_no AS case_no
           |        ,court_name
           |        ,case_stage
           |        ,yg_name
           |        ,bg_name
           |        ,date
           |        ,detail_id
           |        ,case_amt
           |FROM    ok_table_$table_name
           |WHERE   xjk_case_no IS NOT NULL
           |""".stripMargin)
    }

    def inc(table_name: String): Unit = {

      val last_ds = sql(s"show partitions $project.$target_table")
        .collect()
        .map(r => r.getString(0))
        .filter(str => str.contains(table_name))
        .flatMap(str => str.split("/"))
        .filter(str => str.contains("ds"))
        .map(str => str.split("=")(1))
        .max

      val inc_last_ds = getLastPartitionsOrElse(s"$project.inc_ads_$table_name", "0")

      sql(
        s"""
           |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $project.ads_judicial_case_relation_pre PARTITION(ds='$inc_last_ds',tn='$table_name')
           |SELECT  judicase_id
           |        ,flag
           |        ,title
           |        ,case_type
           |        ,case_reason
           |        ,case_no
           |        ,court_name
           |        ,case_stage
           |        ,yg_name
           |        ,bg_name
           |        ,date
           |        ,detail_id
           |        ,case_amt
           |FROM    (
           |            SELECT  *
           |                    ,ROW_NUMBER() OVER(PARTITION BY judicase_id ORDER BY xjk_f DESC ) AS num
           |            FROM    (
           |                        SELECT  judicase_id
           |                                ,flag
           |                                ,title
           |                                ,case_type
           |                                ,case_reason
           |                                ,case_no
           |                                ,court_name
           |                                ,case_stage
           |                                ,yg_name
           |                                ,bg_name
           |                                ,date
           |                                ,detail_id
           |                                ,case_amt
           |                                ,0 AS xjk_f
           |                        FROM    winhc_eci_dev.ads_judicial_case_relation_pre
           |                        WHERE   ds = '$last_ds'
           |                        AND     tn = '$table_name'
           |                        UNION ALL
           |                        SELECT  md5(CONCAT(rowkey,'$table_name')) as judicase_id
           |                                ,${flag_map(table_name)} as flag
           |                                ,CONCAT('关于',name,'的失信信息') as title
           |                                ,case_type(gist_dd) as case_type
           |                                ,null as case_reason
           |                                ,case_no_trim(gist_dd) as case_no
           |                                ,court as court_name
           |                                ,'执行' as case_stage
           |                                ,null as yg_name
           |                                ,name as bg_name
           |                                ,pub_date as date
           |                                ,rowkey as detail_id
           |                                ,null as case_amt
           |                                ,1 AS xjk_f
           |                        FROM    winhc_eci_dev.inc_ads_$table_name
           |                        WHERE   ds > '$last_ds'
           |                        AND     case_no_trim(gist_dd) is not null
           |                    ) AS t1
           |        ) AS t2
           |WHERE   t2.num = 1
           |""".stripMargin)

    }

    for (t <- Seq("company_dishonest_info", "company_dishonest_info_person"))
      if (is_inc)
        inc(t)
      else
        all(t)
  }
}

object JudicialCaseRelationPre39 {
  def main(args: Array[String]): Unit = {
    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_eci_dev",
      "spark.hadoop.odps.spark.local.partition.amt" -> "1000"
    )
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)

    //直接每天计算全量、不受前一天数据影响
    JudicialCaseRelationPre39(s = spark, project = "winhc_eci_dev").company_dishonest_info(false)
    spark.stop()
  }
}
