package com.winhc.bigdata.spark.jobs.judicial

import com.winhc.bigdata.spark.udf.{BaseFunc, CompanyMapping}
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, SparkUtils}
import org.apache.spark.sql.SparkSession

import scala.collection.mutable

/**
 * @Description:司法案件预处理
 * @author Yan Yongnian
 * @date 2020/9/27
 */
object JudicialCaseRelationPre456 {
  def main(args: Array[String]): Unit = {
    val project = "winhc_eci_dev"
    println(
      s"""
         |project: $project
         |""".stripMargin)

    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> s"$project",
      "spark.hadoop.odps.spark.local.partition.amt" -> "1000"
    )
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)
    JudicialCaseRelationPre456(spark, project).precalc()
    spark.stop()
  }
}
case class JudicialCaseRelationPre456(s: SparkSession, project: String
                                   ) extends LoggingUtils with CompanyMapping with BaseFunc {
  override protected val spark: SparkSession = s

  def precalc(): Unit = {
    prepareFunctions(spark)
    case_no_trim_udf()
    //送达公告预处理
    var lastDsIncAds = BaseUtil.getPartion(s"$project.inc_ads_company_send_announcement", spark)
    spark.sparkContext.setJobDescription(s"处理send_announcement($lastDsIncAds)")
    sql(
      s"""
         |insert ${if (isWindows) "INTO" else "OVERWRITE"} table $project.ads_judicial_case_relation_pre partition(ds='$lastDsIncAds',tn='send_announcement')
         |select
         |  judicase_id
         |  ,flag
         |  ,title
         |  ,case_type
         |  ,case_reason
         |  ,case_no
         |  ,court_name
         |  ,case_stage
         |  ,yg_name
         |  ,bg_name
         |  ,date
         |  ,detail_id
         |  ,case_amt
         |from (
         |      select
         |      md5(cleanup(case_no)) as judicase_id
         |      ,"4" as flag
         |      ,title(plaintiff,defendant,'送达公告') as title
         |      ,concat_ws('',case_type(case_no)) as case_type
         |      ,case_reason
         |      ,case_no
         |      ,court as court_name
         |      ,concat_ws('',case_stage(case_no)) as case_stage
         |      ,plaintiff as yg_name
         |      ,defendant as bg_name
         |      ,start_date as date
         |      ,rowkey as detail_id
         |      ,0.0 as case_amt
         |      ,row_number() over(partition by rowkey order by update_time desc) num
         |      from (
         |        select case_no,plaintiff,defendant,case_reason,court,start_date,rowkey,update_time
         |        from $project.ads_company_send_announcement
         |        where length(case_no) > 0 and ds > '0'
         |        union all
         |        select case_no,plaintiff,defendant,case_reason,court,start_date,rowkey,update_time
         |        from $project.inc_ads_company_send_announcement
         |        where length(case_no) > 0 and ds > '0'
         |      )
         |   )
         |where num = 1
         |""".stripMargin).show(10, false)

    //限制高消费预处理(企业）
    lastDsIncAds = BaseUtil.getPartion(s"$project.inc_ads_company_zxr_restrict", spark)
    spark.sparkContext.setJobDescription(s"处理zxr_restrict($lastDsIncAds)")
    sql(
      s"""
         |insert ${if (isWindows) "INTO" else "OVERWRITE"} table $project.ads_judicial_case_relation_pre partition(ds='$lastDsIncAds',tn='zxr_restrict')
         |select
         |  judicase_id
         |  ,flag
         |  ,title
         |  ,case_type
         |  ,case_reason
         |  ,case_no
         |  ,court_name
         |  ,case_stage
         |  ,yg_name
         |  ,bg_name
         |  ,date
         |  ,detail_id
         |  ,case_amt
         |from (
         |      select
         |      md5(cleanup(case_no)) as judicase_id
         |      ,"5" as flag
         |      ,if(length(company_name)>3 OR length(company_info)>3, concat_ws('',coalesce(company_name,company_info),'及',name,'被采取限制消费措施'),concat_ws('',name,'被采取限制消费措施')) AS title
         |      ,concat_ws('',case_type(case_no)) as case_type
         |      ,NULL AS case_reason
         |      ,case_no
         |      ,court_name
         |      ,concat_ws('',case_stage(case_no)) as case_stage
         |      ,NULL as yg_name
         |      ,if(length(company_name)>3 OR length(company_info)>3, coalesce(company_name,company_info),name) as bg_name
         |      ,case_create_time as date--目前天眼查数据没有执行日期，先以此代替
         |      ,rowkey as detail_id
         |      ,0.0 as case_amt
         |      ,row_number() over(partition by rowkey order by update_time desc) num
         |      from (
         |        select case_no,company_name,company_info,name,court_name,case_create_time,rowkey,update_time
         |        from $project.ads_company_zxr_restrict
         |        where length(case_no) > 0 and ds > '0'
         |        union all
         |        select case_no,company_name,company_info,name,court_name,case_create_time,rowkey,update_time
         |        from $project.inc_ads_company_zxr_restrict
         |        where length(case_no) > 0 and ds > '0'
         |      )
         |   )
         |where num = 1
         |""".stripMargin).show(10, false)
    //限制高消费预处理(个人）
    lastDsIncAds = BaseUtil.getPartion(s"$project.inc_ads_company_zxr_restrict_person", spark)
    //身份证补全表最新分区
    val personIdDs = BaseUtil.getPartion("ads_person_idcard_cloze", spark)
    is_id_card_udf()
    spark.sparkContext.setJobDescription(s"处理zxr_restrict_person($lastDsIncAds)")
    sql(
      s"""
         |insert ${if (isWindows) "INTO" else "OVERWRITE"} table $project.ads_judicial_case_relation_pre partition(ds='$lastDsIncAds',tn='zxr_restrict_person')
         |select
         |  judicase_id
         |  ,flag
         |  ,title
         |  ,case_type
         |  ,case_reason
         |  ,case_no
         |  ,court_name
         |  ,case_stage
         |  ,yg_name
         |  ,bg_name
         |  ,date
         |  ,detail_id
         |  ,case_amt
         |from (
         |      select
         |      md5(cleanup(A.case_no)) as judicase_id
         |      ,"11" as flag
         |      ,concat_ws('',A.name,'被采取限制消费措施') AS title
         |      ,concat_ws('',case_type(A.case_no)) as case_type
         |      ,NULL AS case_reason
         |      ,A.case_no
         |      ,court_name
         |      ,concat_ws('',case_stage(A.case_no)) as case_stage
         |      ,NULL as yg_name
         |      ,A.name as bg_name
         |      ,case_create_time as date--目前天眼查数据没有执行日期，先以此代替
         |      ,rowkey as detail_id
         |      ,0.0 as case_amt
         |      ,row_number() over(partition by rowkey order by update_time desc) num
         |      from (
         |        select case_no,court_name,name,case_create_time,rowkey,update_time,identity_num
         |        from $project.ads_company_zxr_restrict_person
         |        where length(case_no) > 0 and ds>'0'
         |        union all
         |        select case_no,court_name,name,case_create_time,rowkey,update_time,identity_num
         |        from $project.inc_ads_company_zxr_restrict_person
         |        where length(case_no) > 0 and ds>'0'
         |      ) A
         |      left join(
         |        select name, identity_num, case_no
         |        from $project.ads_person_idcard_cloze
         |        where ds=${personIdDs}
         |      ) B
         |      on A.name=B.name AND A.case_no=B.case_no
         |      where is_id_card(A.identity_num) OR is_id_card(B.identity_num)
         |   )
         |where num = 1
         |""".stripMargin).show(10, false)

  }
}