package com.winhc.bigdata.spark.jobs.judicial

import com.winhc.bigdata.spark.implicits.RegexUtils.RichRegex
import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{AsyncExtract, LoggingUtils, SparkUtils, case_connect_utils}
import org.apache.spark.internal.Logging
import org.apache.spark.sql.functions.monotonically_increasing_id
import org.apache.spark.sql.types._
import org.apache.spark.sql.{Row, SparkSession}

import scala.annotation.meta.getter
import scala.collection.mutable
import scala.collection.mutable.ArrayBuffer

/**
 * @Author: XuJiakai
 * @Date: 2020/8/28 16:52
 * @Description:
 */
case class JudicialCaseRelation_CaseAgg(s: SparkSession,
                                        project: String //表所在工程名
                                       ) extends LoggingUtils with Logging with BaseFunc {
  @(transient@getter) val spark: SparkSession = s
  private val table_id_map = Map("wenshu_detail" -> "case_id")
  private val pat = ".*\\d+.*".r

  private val separation = "@@"

  import spark.implicits._

  case_no_trim_udf()
  is_id_card_udf()

  def etl(ds: String): Unit = {
    AsyncExtract.startAndWait(spark, Seq(
      ("dishonest etl...", () => {
        etl_company_dishonest_info(ds)
        true
      })
      ,
      ("wenshu etl...", () => {
        etl_wenshu(ds)
        true
      })
      ,
      ("company_zxr etl...", () => {
        etl_company_zxf(ds)
        true
      })
    ))

        etc_dwd_judicial_case(ds)
    relationByGroup()
  }

  private def etc_dwd_judicial_case(ds: String): Unit = {

    val rdd = sql(
      s"""
         |SELECT  main_case_no
         |        ,case_no
         |        ,rowkey
         |        ,case_attribute
         |        ,ds
         |        ,tn
         |FROM    winhc_eci_dev.dwd_judicial_case
         |WHERE   ds = '$ds'
         |""".stripMargin)
      .withColumn("id", monotonically_increasing_id)
      .rdd.map(r =>
      (s"${r.getAs[String]("rowkey")}_${r.getAs[String]("tn")}", r)
    ).groupByKey(500).flatMap(r => {
      val li = r._2
      val id = li.last.getAs[Long]("id")
      li.map(r => {
        val main_case_no = r.getAs[Long]("main_case_no")
        val case_no = r.getAs[String]("case_no")
        val rowkey = r.getAs[String]("rowkey")
        val case_attribute = r.getAs[Map[String, String]]("case_attribute")
        val ds = r.getAs[String]("ds")
        val tn = r.getAs[String]("tn")
        Row(id, main_case_no, case_no, rowkey, case_attribute, ds, tn)
      })
    })
    val schema = StructType(Array(
      StructField("id", LongType),
      StructField("main_case_no", LongType),
      StructField("case_no", StringType),
      StructField("rowkey", StringType),
      StructField("case_attribute", MapType(StringType, StringType)),
      StructField("ds", StringType),
      StructField("tn", StringType)
    ))

    spark.createDataFrame(rdd, schema).createTempView("all_tmp_xjk")


    sql(
      s"""
         |INSERT OVERWRITE TABLE winhc_eci_dev.dwd_judicial_case_2 PARTITION(ds='$ds')
         |select id
         |       ,main_case_no
         |       ,case_no
         |       ,rowkey
         |       ,case_attribute
         |       ,tn
         |from   all_tmp_xjk
         |""".stripMargin)
  }

  private def etl_company_zxf(ds: String): Unit = {
    val view =
      s"""
         | rowkey as id
         | ,court as court_name
         | ,case_no
         | ,court as connect_court_name
         | ,gist_id as connect_case_no
         | ,null as yg_name
         | ,cname as bg_name
         | ,ds
         |""".stripMargin

    /*val companyCidAndNameUtils = utils.CompanyCidAndNameUtils(spark)
      def detail_zxr(tn: String): Unit = {
      val org_ads_tab = s"$project.ads_$tn"
      val org_inc_ads_tab = s"$project.inc_ads_$tn"
      val ads_last_ds = getLastPartitionsOrElse(org_ads_tab, "0")

      val intersect_cols = getColumns(org_ads_tab).intersect(getColumns(org_inc_ads_tab))

      val verify = tn.contains("person") match {
        case true => s"AND     is_id_card(t2.card)"
        case false => ""
      }
      sql(
        s"""
           |SELECT  *
           |FROM    (
           |            SELECT  *
           |                    ,ROW_NUMBER() OVER(PARTITION BY rowkey ORDER BY ds DESC ) AS num
           |            FROM    (
           |                        SELECT  ${intersect_cols.mkString(",")}
           |                        FROM    $org_ads_tab
           |                        WHERE   ds = '$ads_last_ds'
           |                        UNION ALL
           |                        SELECT  ${intersect_cols.mkString(",")}
           |                        FROM    $org_inc_ads_tab
           |                        WHERE   ds > '$ads_last_ds'
           |                    ) AS t1
           |        ) AS t2
           |WHERE   t2.num = 1
           |$verify
           |""".stripMargin).createTempView("xjk_tmp")

      var other_cols: Seq[String] = null
      var result_tab: String = null

      if (!tn.contains("person")) {
        val res_tab = companyCidAndNameUtils.replaceNewNameByCid("xjk_tmp", "new_cid", "cname")
        result_tab = s"${res_tab}_tmp"
        sql(
          s"""
             |SELECT  $view
             |FROM    $res_tab
             |""".stripMargin)
          .cache()
          .createTempView(result_tab)
      } else {
        result_tab = s"xjk_tmp_xjk_tmp"
        sql(
          s"""
             |SELECT  $view
             |FROM    xjk_tmp
             |""".stripMargin)
          .createTempView(result_tab)
      }
      other_cols = getColumns(result_tab)

      sql(
        s"""
           |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE winhc_eci_dev.dwd_judicial_case  PARTITION(ds='$ds',tn='$tn')
           |SELECT  id
           |        , 1 as main_case_no
           |        ,case_no
           |        ,${getStrToMap(other_cols)} as case_attribute
           |FROM    $result_tab
           |UNION ALL
           |SELECT  id
           |        , 0 as main_case_no
           |        ,connect_case_no as case_no
           |        ,${getStrToMap(other_cols)} as case_attribute
           |FROM    $result_tab
           |WHERE   connect_case_no is not null
           |""".stripMargin)

    }

    AsyncExtract.startAndWait(spark, Seq(
      ("company_zxr", () => {
        detail_zxr("company_zxr_list")
        true
      })
      , ("company_zxr_person", () => {
        detail_zxr("company_zxr_person")
        true
      })
    ))*/

    AsyncExtract.startAndWait(spark, Seq(
      ("company_zxr", () => {
        detail_etl(ds, "company_zxr", view)
        true
      })
      , ("company_zxr_person", () => {
        detail_etl(ds, "company_zxr_person", view)
        true
      })
    ))
  }

  private def detail_etl(ds: String, tn: String, view: String): Unit = {
    val tmp_tab = s"all_etl_${tn}_tmp"

    val org_ads_tab = s"$project.ads_$tn"
    val org_inc_ads_tab = s"$project.inc_ads_$tn"
    val ads_last_ds = getLastPartitionsOrElse(org_ads_tab, "0")

    val intersect_cols = getColumns(org_ads_tab).intersect(getColumns(org_inc_ads_tab))

    sql(
      s"""
         |SELECT $view
         |FROM    (
         |            SELECT  *
         |                    ,ROW_NUMBER() OVER(PARTITION BY rowkey ORDER BY ds DESC ) AS num
         |            FROM    (
         |                        SELECT  ${intersect_cols.mkString(",")}
         |                        FROM    $org_ads_tab
         |                        WHERE   ds = '$ads_last_ds'
         |                        UNION ALL
         |                        SELECT  ${intersect_cols.mkString(",")}
         |                        FROM    $org_inc_ads_tab
         |                        WHERE   ds > '$ads_last_ds'
         |                    ) AS t1
         |        ) AS t2
         |WHERE   t2.num = 1
         |""".stripMargin)
      .createTempView(tmp_tab)
    val other_cols = getColumns(tmp_tab)

    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE winhc_eci_dev.dwd_judicial_case  PARTITION(ds='$ds',tn='$tn')
         |SELECT  null as id
         |        , 1 as main_case_no
         |        ,case_no_trim(case_no) as case_no
         |        ,id as rowkey
         |        ,${getStrToMap(other_cols)} as case_attribute
         |FROM    $tmp_tab
         |WHERE   case_no_trim(case_no) is  not null
         |UNION ALL
         |SELECT  null as id
         |        , 0 as main_case_no
         |        ,case_no_trim(connect_case_no) as case_no
         |        ,id as rowkey
         |        ,${getStrToMap(other_cols)} as case_attribute
         |FROM    $tmp_tab
         |WHERE   connect_case_no is not null
         |AND     case_no_trim(connect_case_no) is not null
         |""".stripMargin)
  }


  private def etl_company_dishonest_info(ds: String): Unit = {
    val view =
      s"""
         | rowkey as id
         | ,court as court_name
         | ,case_no
         | ,gist_unit as connect_court_name
         | ,gist_dd as connect_case_no
         | ,null as yg_name
         | ,name as bg_name
         | ,ds
         |""".stripMargin


    AsyncExtract.startAndWait(spark, Seq(
      ("company_dishonest_info", () => {
        detail_etl(ds, "company_dishonest_info", view)
        true
      })
      , ("company_dishonest_info_person", () => {
        detail_etl(ds, "company_dishonest_info_person", view)
        true
      })
    ))
  }

  private def etl_wenshu(ds: String): Unit = {
    val tableName = "wenshu_detail"

    val org_tab = s"winhc_eci.ods_$tableName"

    val table_id = table_id_map(tableName)
    val other_cols = Seq("yg_name", "court_name", "case_no", "bg_name") ++ Seq(table_id, "ds", "connect_case_no")

    val ods_end_ds = getLastPartitionsOrElse(org_tab, "0")
    val tmp_tab = s"all_${tableName}_tmp_$ods_end_ds"

    sql(
      s"""
         |SELECT  *
         |FROM    (
         |            SELECT  *
         |                    ,ROW_NUMBER() OVER(PARTITION BY $table_id ORDER BY ds DESC ) AS num
         |            FROM    (
         |                        SELECT  ${other_cols.mkString(",")}
         |                        FROM    $org_tab
         |                        WHERE   ds > 0
         |                    ) AS t1
         |        ) AS t2
         |WHERE   t2.num = 1
         |""".stripMargin)
      .createTempView(tmp_tab)


    sql(
      s"""
         |SELECT  *
         |FROM    $tmp_tab lateral view OUTER explode(split(connect_case_no,'\\n')) t as single_connect_case_no
         |""".stripMargin)
      //      .cache()
      .createTempView(s"explode_$tmp_tab")

    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE winhc_eci_dev.dwd_judicial_case  PARTITION(ds='$ds',tn='$tableName')
         |SELECT  null as id
         |        , 1 as main_case_no
         |        ,case_no_trim(case_no) as case_no
         |        ,$table_id as rowkey
         |        ,${getStrToMap(other_cols)} as case_attribute
         |FROM    explode_$tmp_tab
         |WHERE   case_no_trim(case_no) is not null
         |UNION ALL
         |SELECT  null as id
         |        , 0 as main_case_no
         |        ,case_no_trim(single_connect_case_no) as case_no
         |        ,$table_id as rowkey
         |        ,${getStrToMap(other_cols)} as case_attribute
         |FROM    explode_$tmp_tab
         |WHERE   single_connect_case_no is not null
         |AND     case_no_trim(single_connect_case_no) is not null
         |""".stripMargin)
  }


  private def relationByGroup(): Unit = {
    val org_tab = s"$project.dwd_judicial_case_2"

    val dwd_last_ds = getLastPartitionsOrElse(org_tab, "0")
    spark.udf.register("case_equ", case_equ _)
    spark.udf.register("str_sort", (v1: String, v2: String) => Seq(v1, v2).filter(_ != null).sorted.mkString(""))
    spark.udf.register("match_case_no", (case_no: String) => pat matches case_no)

    //---        winhc_eci_dev.xjk_test_ttttt
    sql(
      s"""
         | SELECT  *
         | FROM    $org_tab
         | WHERE   ds = '$dwd_last_ds'
         |--- AND     case_no IS NOT NULL
         |--- AND     case_no <> ''
         |--- AND     match_case_no(case_no)
         |""".stripMargin)
      .repartition(500)
      //      .cache()
      .createTempView("dwd_judicial_case_tmp")

    //需要区分group by ,只用一个
    sql(
      s"""
         |SELECT  case_no,party,collect_set(id) as connect_case_id
         |FROM    (
         |            SELECT  concat_ws('$separation',id,tn) as id
         |                    ,case_no
         |                    ,tn
         |                    ,main_case_no
         |                    ,case_attribute
         |                    ,party
         |            FROM    dwd_judicial_case_tmp
         |            LATERAL VIEW OUTER explode(split(concat_ws('\\n',case_attribute['yg_name'],case_attribute['bg_name']) ,'\\n')) t AS party
         |            WHERE   main_case_no = 1
         |        ) AS t1
         |WHERE   length(t1.party) > 4
         |GROUP BY case_no,party
         |UNION ALL
         |SELECT  case_no,null as party,collect_set(id) as connect_case_id
         |FROM    (
         |            SELECT  concat_ws('$separation',id,tn) as id
         |                    ,case_no
         |                    ,tn
         |                    ,main_case_no
         |                    ,case_attribute
         |                    ,party
         |            FROM    dwd_judicial_case_tmp
         |            LATERAL VIEW OUTER explode(split(concat_ws('\\n',case_attribute['yg_name'],case_attribute['bg_name']) ,'\\n')) t AS party
         |            WHERE   tn in ('company_dishonest_info','company_dishonest_info_person','company_zxr','company_zxr_person')
         |        ) AS t1
         |GROUP BY case_no
         |""".stripMargin).rdd
      //(tn = 'wenshu_detail' and main_case_no = 1) and
      .flatMap(r => {
        val party = r.getAs[String]("party")
        val case_no = r.getAs[String]("case_no")
        val connect_case_id = r.getAs[Seq[String]]("connect_case_id")
        val list = ArrayBuffer[(String, String, String, String, String, String, Int)]()
        if (connect_case_id.length < 2) {
          val e_1 = connect_case_id.head.split(separation)
          list.append((e_1(0), null, case_no, null, e_1(1), null, 2))
        }
        for (i <- 0 to connect_case_id.length - 2) {
          val e_1 = connect_case_id(i).split(separation)
          val e_2 = connect_case_id(i + 1).split(separation)
          list.append((e_1(0), e_2(0), case_no, case_no, e_1(1), e_2(1), 2))
        }
        list
      })
      .toDF("id_1", "id_2", "case_no_1", "case_no_2", "tn_1", "tn_2", "connect_type")
      .createTempView("connect_tmp_1")

    sql(
      s"""
         |SELECT  t1.id AS id_1
         |        ,t2.id AS id_2
         |        ,t1.case_no AS case_no_1
         |        ,t2.case_no AS case_no_2
         |        ,t1.tn AS tn_1
         |        ,t2.tn AS tn_2
         |        ,1 AS connect_type
         |        ,str_sort(
         |            concat_ws('',t1.id,t1.tn)
         |            ,concat_ws('',t2.id,t2.tn)
         |        ) AS xjk_sorted
         |FROM    (
         |            SELECT  *
         |            FROM    dwd_judicial_case_tmp
         |            WHERE   main_case_no = 1
         |            AND     tn = 'wenshu_detail'
         |        ) AS t1
         |FULL JOIN (
         |              SELECT  *
         |              FROM    dwd_judicial_case_tmp
         |              WHERE   main_case_no = 0
         |              UNION ALL
         |              SELECT  *
         |              FROM    dwd_judicial_case_tmp
         |              WHERE   main_case_no = 1
         |              AND     tn <> 'wenshu_detail'
         |          ) AS t2
         |ON      t1.case_no = t2.case_no
         |AND     t1.id <> t2.id
         |AND     case_equ(t1.case_attribute , t2.case_attribute,t1.tn,t2.tn)
         |
         |UNION ALL
         |
         |SELECT  t1.id AS id_1
         |        ,t2.id AS id_2
         |        ,t1.case_no AS case_no_1
         |        ,t2.case_no AS case_no_2
         |        ,t1.tn AS tn_1
         |        ,t2.tn AS tn_2
         |        ,1 AS connect_type
         |        ,str_sort(
         |            concat_ws('',t1.id,t1.tn)
         |            ,concat_ws('',t2.id,t2.tn)
         |        ) AS xjk_sorted
         |FROM    (
         |            SELECT  *
         |            FROM    dwd_judicial_case_tmp
         |            WHERE   main_case_no = 0
         |            AND     tn = 'wenshu_detail'
         |        ) AS t1
         |FULL JOIN (
         |              SELECT  *
         |              FROM    dwd_judicial_case_tmp
         |              WHERE   main_case_no = 1
         |              UNION ALL
         |              SELECT  *
         |              FROM    dwd_judicial_case_tmp
         |              WHERE   main_case_no = 0
         |              AND     tn <> 'wenshu_detail'
         |          ) AS t2
         |ON      t1.case_no = t2.case_no
         |AND     t1.id <> t2.id
         |AND     case_equ(t1.case_attribute , t2.case_attribute,t1.tn,t2.tn)
         |
         |""".stripMargin)
      .createTempView("connect_tmp_2")

    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE winhc_eci_dev.ads_judicial_case_relation
         |SELECT  id_1
         |        ,id_2
         |        ,case_no_1
         |        ,case_no_2
         |        ,tn_1
         |        ,tn_2
         |        ,connect_type
         |FROM    (
         |            SELECT  *
         |                    ,ROW_NUMBER() OVER(PARTITION BY xjk_sorted ORDER BY connect_type) AS num
         |            FROM    (
         |                        SELECT  id_1
         |                                ,id_2
         |                                ,case_no_1
         |                                ,case_no_2
         |                                ,tn_1
         |                                ,tn_2
         |                                ,connect_type
         |                                ,str_sort(concat_ws('',id_1,tn_1),concat_ws('',id_2,tn_2)) AS xjk_sorted
         |                        FROM    connect_tmp_1
         |                        UNION ALL
         |                        SELECT  id_1
         |                                ,id_2
         |                                ,case_no_1
         |                                ,case_no_2
         |                                ,tn_1
         |                                ,tn_2
         |                                ,connect_type
         |                                ,xjk_sorted
         |                        FROM    connect_tmp_2
         |                    ) AS t1
         |        ) AS t2
         |WHERE   t2.num = 1
         |""".stripMargin)

  }

  private def getStrToMap(cols: Seq[String]): String = {
    val set = cols.toSet
    val str = set.map(e => {
      s"concat_ws('\001','$e',cast($e as string))"
    }).mkString(",")
    s"str_to_map(concat_ws('\002',$str),'\002','\001')"
  }

  private def getVal(map: Map[String, String], key: String): String = {
    val v = map.getOrElse(key, "")
    if (v == null) {
      ""
    } else {
      v
    }
  }

  private def case_equ(m1: Map[String, String], m2: Map[String, String], tn1: String, tn2: String): Boolean = {
    try {
      val current_case_party_list_org: Seq[String] = getVal(m1, "yg_name").split("\n") ++ getVal(m1, "bg_name").split("\n")
      val connect_case_party_list_org: Seq[String] = getVal(m2, "yg_name").split("\n") ++ getVal(m2, "bg_name").split("\n")

      val current_case_no = getVal(m1, "case_no")
      val connect_case_no = getVal(m2, "case_no")
      val current_court_name = getVal(m1, "court_name")
      val connect_court_name = getVal(m2, "court_name")

      case_connect_utils.isConnect(current_case_party_list_org, connect_case_party_list_org, current_case_no, connect_case_no, current_court_name, connect_court_name,tn1,tn2)
    } catch {
      case ex: Exception => {
        logError(ex.getMessage)
        println("error")
        println(m1)
        println(m2)
        throw new RuntimeException(ex)
      }
        false
    }
  }

}

object JudicialCaseRelation_CaseAgg {

  def main(args: Array[String]): Unit = {

    val Array(ds) = args

    println(
      s"""
         |ds: $ds
         |""".stripMargin)
    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_eci_dev",
      "spark.hadoop.odps.spark.local.partition.amt" -> "1000"
    )
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)
    val jcr = JudicialCaseRelation_CaseAgg(spark, project = "winhc_eci_dev")
    jcr.etl(ds)
    spark.stop()
  }
}