package com.winhc.bigdata.spark.jobs.message

import java.util.Date

import com.winhc.bigdata.spark.jobs.dynamic.CompanyDynamic.env
import com.winhc.bigdata.spark.jobs.dynamic.CompanyDynamicHandleUtils
import com.winhc.bigdata.spark.udf.MapAggs
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{BaseUtil, DateUtils, LoggingUtils, SparkUtils}
import org.apache.commons.lang3.time.DateFormatUtils
import org.apache.spark.sql.SparkSession
import org.json4s.DefaultFormats
import org.json4s.jackson.Json

import scala.annotation.meta.getter
import scala.collection.mutable
import scala.collection.mutable.ListBuffer

/**
 * @Description: 知识产权动态变化
 * @author π
 * @date 2020/7/2719:15
 */
object IntellectualMessage {
  def main(args: Array[String]): Unit = {

    val project = "winhc_eci_dev"

    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> project,
      "spark.hadoop.odps.spark.local.partition.amt" -> "1000"
    )
    val spark = SparkUtils.InitEnv("IntellectualMessage", config)
    IntellectualMessage(spark, null, project).calc()
    //    IntellectualMessage(spark, project).transForm()
    spark.stop()
  }

}


case class IntellectualMessage(s: SparkSession, project: String,
                               ds: String,
                               runOld: Boolean = false) extends LoggingUtils {

  @(transient@getter) val spark: SparkSession = s

  import spark.implicits._

  val t1 = s"company_tm" //商标
  val t2 = s"company_patent_list" //专利
  val t3 = s"company_icp" //网站
  val t4 = s"company_copyright_reg_list" //著作权
  val t5 = s"company_certificate" //证书
  val t6 = s"company_copyright_works_list" //软著作权
  val res_tb = s"$project.tmp_xf_ads_intellectual_message" //聚合结果表
  //  val res_tb_res = s"$project.tmp_xf_ads_intellectual_message_res" //转换输出结果表
  val res_tb_res = s"${getEnvProjectName(env, project)}.ads_company_dynamic" //转换输出结果表

  val tn = "intellectual"

  //  var ds = BaseUtil.getPartion(s"$project.inc_ads_$t1", spark)
  val remainDs = BaseUtil.getPartion(s"$project.ads_$t1", spark)
  val mapDs = BaseUtil.getPartion(s"$project.base_company_mapping", spark)

  def col2Map(pkg: String, day: String): Map[String, String] = {
    Map(pkg -> day)
  }

  def calc(): Unit = {
    println("start calc" + new Date())
    spark.udf.register("col2Map", col2Map _)
    spark.udf.register("form_date", DateUtils.formatterDate _)
    spark.udf.register("MapAggs", new MapAggs())

    sql(
      s"""
         |SELECT  cid as new_cid
         |FROM    $project.ads_change_extract
         |WHERE   ds = $ds
         |AND     tn in ('$t1','$t2','$t3','$t4','$t5','$t6')
         |AND     type = 'insert'
         |GROUP by cid
         |""".stripMargin).cache().createOrReplaceTempView("mapping")

    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $res_tb PARTITION(ds=$ds)
         |select new_cid,date,m,
         |max(coalesce(split(m[1], ',')[2],'0')) over (partition by new_cid) cnt1,
         |max(coalesce(split(m[2], ',')[2],'0')) over (partition by new_cid) cnt2,
         |max(coalesce(split(m[3], ',')[2],'0')) over (partition by new_cid) cnt3,
         |max(coalesce(split(m[4], ',')[2],'0')) over (partition by new_cid) cnt4,
         |max(coalesce(split(m[5], ',')[2],'0')) over (partition by new_cid) cnt5,
         |max(coalesce(split(m[6], ',')[2],'0')) over (partition by new_cid) cnt6
         |from (
         |select new_cid,date,MapAggs(m1) m from (
         |select *,
         |col2Map(type,CONCAT_WS(',',inc_cnt,del_cnt,total_cnt)) m1
         |from ${assemblySQL(s"$project", s"$t1", "app_date", "1")}
         |union all
         |select *,
         |col2Map(type,CONCAT_WS(',',inc_cnt,del_cnt,total_cnt)) m1
         |from ${assemblySQL(s"$project", s"$t2", "app_date", "2")}
         |union all
         |select *,
         |col2Map(type,CONCAT_WS(',',inc_cnt,del_cnt,total_cnt)) m1
         |from ${assemblySQL(s"$project", s"$t3", "examine_date", "3")}
         |union all
         |select *,
         |col2Map(type,CONCAT_WS(',',inc_cnt,del_cnt,total_cnt)) m1
         |from ${assemblySQL(s"$project", s"$t4", "reg_time", "4")}
         |union all
         |select *,
         |col2Map(type,CONCAT_WS(',',inc_cnt,del_cnt,total_cnt)) m1
         |from ${assemblySQL(s"$project", s"$t5", "start_date", "5")}
         |union all
         |select *,
         |col2Map(type,CONCAT_WS(',',inc_cnt,del_cnt,total_cnt)) m1
         |from ${assemblySQL(s"$project", s"$t6", "reg_time", "6")}
         |)a
         |group by new_cid,date
         |)h
         |""".stripMargin)

    //转换输出格式
    transForm()
    println("end calc" + new Date())
  }

  def transForm(): Unit = {
    println("start transForm" + new Date())
    sql(
      s"""
         |select a.*,b.cname from
         |(
         |select *
         |from ${res_tb}
         |where ds = $ds and date is not null
         |)a
         |left join
         |(
         |select cid,cname
         |from
         |$project.base_company_mapping
         |where ds =$mapDs
         |)b on a.cid = b.cid
         |""".stripMargin).map(r => {
      val cid = r.getAs[String]("cid")
      val cname = r.getAs[String]("cname")
      val info_type = tn
      val change_time = r.getAs[String]("date").concat(" 00:00:00")
      val biz_id = s"${cid}"
      val sub_info_type = ""
      val info_risk_level = "2"
      val winhc_suggest = ""
      val create_time = DateFormatUtils.format(new Date(), "yyyy-MM-dd HH:mm:ss")
      val m = r.getAs[Map[String, String]](2)
      val rta_desc = descStr(m)
      var id = "-1"
      try {
        id = CompanyDynamicHandleUtils.getDynamicId(cid, "", biz_id, change_time)

      } catch {
        case ex: Exception => {
          logError(ex.getMessage)
        }
      }
      val change_content: String = Json(DefaultFormats).write(
        mutable.LinkedHashMap("商标数量" -> r.getAs[String]("cnt1"),
          "专利数量" -> r.getAs[String]("cnt2"),
          "域名数量" -> r.getAs[String]("cnt3"),
          "著作权数量" -> r.getAs[String]("cnt4"),
          "证书数量" -> r.getAs[String]("cnt5"),
          "软著数量" -> r.getAs[String]("cnt6")
        )
      )

      (id, cid, cname, info_type, rta_desc, change_content, change_time, biz_id,
        sub_info_type, info_risk_level, winhc_suggest, create_time)
    }).toDF("id", "cid", "cname", "info_type", "rta_desc", "change_content", "change_time", "biz_id",
      "sub_info_type", "info_risk_level", "winhc_suggest", "create_time").createOrReplaceTempView("res")

    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE ${res_tb_res} PARTITION (ds='$ds',tn='$tn')
         |select * from res
         |where id <> '-1'
         |and id is not null
         |and to_timestamp(change_time) <= now()
         |""".stripMargin)

    println("end transForm" + new Date())
  }

  def descStr(m: Map[String, String]) = {
    val list = ListBuffer[String]()
    if (m.contains("1")) getStr(m("1"), "商标", list)
    if (m.contains("2")) getStr(m("2"), "专利", list)
    if (m.contains("3")) getStr(m("3"), "域名", list)
    if (m.contains("4")) getStr(m("4"), "著作权", list)
    if (m.contains("5")) getStr(m("5"), "证书", list)
    if (m.contains("6")) getStr(m("6"), "软著", list)
    list.toList.mkString(",")
  }

  def getStr(s: String, name: String, list: ListBuffer[String]) = {
    val Array(inc_cnt, del_cnt, total_cnt) = s.split(",")
    if (!inc_cnt.equals("0")) {
      list += s"新增${name}数量${inc_cnt}个"
    }
    if (!del_cnt.equals("0")) {
      list += s"移除${name}数量${del_cnt}个"
    }
  }

  //增量,存量历史计算
  def assemblySQL(project: String, table: String, date: String, tp: String) = {
    if (runOld) { //存量
      s"""
         |(
         |SELECT  new_cid
         |        ,form_date(SUBSTR(CAST($date AS STRING),1,10)) AS DATE
         |        ,sum(CASE WHEN deleted = 0 THEN 1 ELSE 0 END) OVER(PARTITION BY new_cid,$date) inc_cnt
         |        ,0 del_cnt
         |        ,sum(CASE WHEN deleted = 0 THEN 1 ELSE 0 END) OVER(PARTITION BY new_cid) total_cnt
         |        ,$tp type
         |FROM    (
         |         select * from (
         |            SELECT  rowkey
         |                    ,new_cid
         |                    ,$date
         |                    ,deleted
         |            FROM    $project.inc_ads_$table x
         |            WHERE   x.ds > 0
         |            union all
         |            SELECT  rowkey
         |                    ,new_cid
         |                    ,$date
         |                    ,deleted
         |            FROM    $project.ads_$table y
         |            WHERE   y.ds > 0
         |         )z
         |            GROUP BY rowkey
         |                     ,new_cid
         |                     ,$date
         |                     ,deleted
         |        ) b
         |)
         |""".stripMargin
    } else { //增量
      s"""
         |(
         |select * from
         |(
         |SELECT  new_cid
         |        ,form_date(SUBSTR(CAST(date AS STRING),1,10)) AS DATE
         |        ,count(rowkey) OVER(PARTITION BY new_cid,date) inc_cnt
         |        ,0 del_cnt
         |        ,count(rowkey) OVER(PARTITION BY new_cid) total_cnt
         |        ,$tp TYPE
         |        ,f
         |FROM    (
         |            SELECT  b.*,"1" f
         |            FROM    mapping a
         |            JOIN    (
         |                        SELECT  rowkey
         |                                ,new_cid
         |                                ,$date as date
         |                        FROM    (
         |                                    SELECT  *
         |                                            ,row_number() OVER (PARTITION BY rowkey ORDER BY update_time DESC) num
         |                                    FROM    (
         |                                                SELECT  rowkey
         |                                                        ,new_cid
         |                                                        ,$date
         |                                                        ,deleted
         |                                                        ,update_time
         |                                                FROM    $project.ads_$table
         |                                                WHERE   ds = $remainDs
         |                                                UNION ALL
         |                                                SELECT  rowkey
         |                                                        ,new_cid
         |                                                        ,$date
         |                                                        ,deleted
         |                                                        ,update_time
         |                                                FROM    $project.inc_ads_$table
         |                                                WHERE   ds > $remainDs
         |                                                AND     ds < $ds
         |                                            ) c
         |                                ) d
         |                        WHERE   num = 1
         |                    ) b
         |            ON      a.new_cid = b.new_cid
         |            UNION ALL
         |            SELECT  rowkey
         |                    ,cid new_cid
         |                    ,biz_date date
         |                    ,"0" f
         |            FROM    $project.ads_change_extract
         |            WHERE   ds = $ds
         |            AND     tn = '$table'
         |            AND     type = 'insert'
         |        ) e
         |)
         |--where f = "0"
         |)
         |""".stripMargin
    }

  }

}
