package com.winhc.bigdata.spark.jobs.monitor

import java.util
import java.util.Collections

import com.winhc.bigdata.spark.utils.BaseUtil
import org.apache.http.entity.ContentType
import org.apache.http.nio.entity.NStringEntity
import org.apache.http.util.EntityUtils
import org.apache.spark.internal.Logging

/**
 * @Description: 财产监控
 * @author π
 * @date 2020/12/8 14:48
 */
trait CompanyMonitorHandle extends Logging {

  //风险等级映射 1->低级 2->中级 3->高级
  private val info_risk_level_map = Map(
    "wenshu_detail_combine_v2" -> "2" //新增胜诉案件
    , "company_dishonest_info" -> "3" //失信移除
    , "company_zxr_list" -> "3" //被执行人移除
    , "company_zxr_restrict" -> "3" //限制高消费移除
    , "company_court_open_announcement_list" -> "2" //有恢复执行案件即将开庭
    , "wenshu_detail_combine" -> "1" //有恢复执行案件-文书
    , "company_equity_info_list" -> "3" //股权出质(出质人)
    , "company_land_mortgage" -> "3" //土地抵押（抵押人）
    , "company_land_mortgage_v2" -> "3" //土地抵押（抵押权人）
    , "company_land_announcement" -> "3" //购地信息
    , "company_finance" -> "3" //融资信息
    , "bankruptcy_open_case" -> "2" //破产公告
    , "company_bid_list" -> "3" //招投标
    , "company_mortgage_info" -> "3" //动产抵押
    , "company_tm" -> "1" //商标
    , "company_patent_list" -> "2" //专利
    , "company_copyright_reg_list" -> "1" //软件著作权
    , "company_copyright_works_list" -> "1" //作品著作权
    , "company_holder_add" -> "2" //股东新增
    , "company_holder_deleted" -> "1" //股东移除
    , "increase_registered_capital_info" -> "2" //增资记录
    , "auction_tracking_list" -> "3" //司法拍卖
    , "zxr_evaluate" -> "3" //有资产选定询价评估机构
    , "zxr_evaluate_results" -> "3" //有资产完成询价评估

  )

  //财产流向 0 -> 流入 1 -> 流出  2->流向未知
  private val flow_type_map = Map(
    "wenshu_detail_combine_v2" -> "0" //新增胜诉案件
    , "company_dishonest_info" -> "1" //失信移除
    , "company_zxr_list" -> "1" //被执行人移除
    , "company_zxr_restrict" -> "1" //被执行人移除
    , "company_court_open_announcement_list" -> "1" //有恢复执行案件即将开庭
    , "wenshu_detail_combine" -> "1" //有恢复执行案件-文书
    , "company_equity_info_list" -> "0" //股权出质(出质人)
    , "company_land_mortgage" -> "0" //土地抵押（抵押人）
    , "company_land_mortgage_v2" -> "0" //土地抵押（抵押权人）
    , "company_land_announcement" -> "1" //购地信息
    , "company_finance" -> "0" //融资信息
    , "bankruptcy_open_case" -> "1" //破产公告
    , "company_bid_list" -> "0" //招投标
    , "company_mortgage_info" -> "0" //动产抵押
    , "company_tm" -> "0" //商标
    , "company_patent_list" -> "0" //专利
    , "company_copyright_reg_list" -> "0" //软件著作权
    , "company_copyright_works_list" -> "0" //作品著作权
    , "company_holder_add" -> "2" //股东新增
    , "company_holder_deleted" -> "1" //股东移除
    , "increase_registered_capital_info" -> "0" //增资记录
    , "auction_tracking_list" -> "0" //司法拍卖
    , "zxr_evaluate" -> "0" //有资产选定询价评估机构
    , "zxr_evaluate_results" -> "0" //有资产完成询价评估
  )


  //维度映射
  private val table_type_map = Map(
    "bankruptcy_open_case" -> "1" //BANKRUPTCY_OPEN_CASE//HBase
    , "increase_registered_capital_info" -> "2" //INCREASE_REGISTERED_CAPITAL_INFO//HBase
    , "company_bid_list" -> "3" //COMPANY_BID_LIST//HBase
    , "company_copyright_reg_list" -> "4" //COMPANY_COPYRIGHT_REG_LIST//HBase
    , "company_copyright_works_list" -> "5" //COMPANY_COPYRIGHT_WORKS_LIST//HBase
    , "company_court_open_announcement_list" -> "6" //COMPANY_COURT_OPEN_ANNOUNCEMENT_LIST//HBase
    , "company_dishonest_info" -> "7" //COMPANY_DISHONEST_INFO//HBase
    , "company_equity_info_list" -> "8" //COMPANY_EQUITY_INFO_LIST//HBase
    , "company_finance" -> "9" //COMPANY_FINANCE//HBase
    , "company_holder_v2" -> "10" //COMPANY_HOLDER//HBase
    , "company_land_announcement" -> "11" //COMPANY_LAND_ANNOUNCEMENT//HBase
    , "company_land_mortgage" -> "12" //COMPANY_LAND_MORTGAGE//HBase
    , "company_land_mortgage_v2" -> "12" //COMPANY_LAND_MORTGAGE//HBase
    , "company_mortgage_info" -> "13" //COMPANY_MORTGAGE_INFO//HBase
    , "company_patent_list" -> "14" //COMPANY_PATENT_LIST//HBase
    , "company_tm" -> "15" //COMPANY_TM//HBase
    , "company_zxr_list" -> "16" //COMPANY_ZXR_LIST//HBase
    , "company_zxr_restrict" -> "17" //COMPANY_ZXR_RESTRICT//HBase
    , "wenshu_detail_combine" -> "18" //WENSHU_DETAIL//ES
    , "wenshu_detail_combine_v2" -> "18" //WENSHU_DETAIL//ES
    , "auction_tracking_list" -> "19" //AUCTION_TRACKING_LIST//HBase
    , "zxr_evaluate" -> "20" ////zxr_evaluate//HBase
    , "zxr_evaluate_results" -> "21" ////zxr_evaluate_results//HBase
  )

  //类别
  private val type_map = Map(
    "bankruptcy_open_case" -> "1" //新增破产公告
    , "increase_registered_capital_info" -> "2" //注册资本上升
    , "company_bid_list" -> "3" //新增招标信息
    , "company_copyright_reg_list" -> "4" //软件著作权
    , "company_copyright_works_list" -> "5" //作品著作权
    , "company_court_open_announcement_list" -> "6" //有恢复执行案件即将开庭
    , "company_dishonest_info" -> "7" //失信信息移除
    , "company_equity_info_list" -> "8" //出质了持有股权
    , "company_finance" -> "9" //有新的融资
    , "company_holder_add" -> "10" //新增了股东
    , "company_holder_remove" -> "11" //股东退出
    , "company_land_announcement" -> "12" //购买了新的土地
    , "company_land_mortgage" -> "13" //抵押了公司土地
    , "company_mortgage_info" -> "14" //资产被抵押
    , "company_patent_list" -> "15" //新增专利信息
    , "company_tm" -> "16" //新增商标信息
    , "company_zxr_list" -> "17" //被执行人信息移除
    , "company_zxr_restrict" -> "18" //限制高消费移除
    , "wenshu_detail_combine" -> "19" //有恢复执行案件
    , "wenshu_detail_combine_v2" -> "20" //新增胜诉案件
    , "company_land_mortgage_v2" -> "21" //接受了他人土地抵押
    , "auction_tracking_list" -> "22" //有资产即将被司法拍卖
    , "zxr_evaluate" -> "23" //有资产选定询价评估机构
    , "zxr_evaluate_results" -> "24" //有资产完成询价评估
  )

  /**
   *
   * @param rowkey
   * @param cid
   * @param change_fields
   * @param old_map
   * @param new_map
   * @return
   * cid
   * cname
   * info_type
   * flow_type
   * rta_desc
   * change_time
   * biz_id
   * info_risk_level
   * type
   */
  def handle(rowkey: String, bizDate: String, cid: String, change_fields: Seq[String], old_map: Map[String, String], new_map: Map[String, String], cname: String = null): Seq[(String, String, String, String, String, String, String, String, String, String)] = {
    val rta_desc = get_rta_desc(old_map, new_map)
    if (rta_desc == null) {
      return Seq.empty
    }
    try {
      Seq((cid
        , cname
        , get_table_type()
        , get_flow_type()
        , rta_desc
        , get_change_time(bizDate, new_map)
        , get_biz_id(rowkey, new_map)
        , get_info_risk_level(old_map, new_map)
        , get_type()
        , getMoney(new_map)
      ))
    } catch {
      case e: Exception => {
        logError(e.getMessage, e)
      }
        Seq.empty
    }

  }

  /**
   * 来源表的变更类型，insert or update
   *
   * @return
   */
  def org_type(): Seq[String] = Seq("insert")


  /**
   * 信息描述
   *
   * @param old_map
   * @param new_map
   * @return
   */
  protected def get_rta_desc(old_map: Map[String, String], new_map: Map[String, String]): String

  /**
   * 类别映射
   *
   * @return
   */
  protected def get_type(): String = type_map(getClass.getSimpleName)

  /**
   * 维度映射
   *
   * @return
   */
  protected def get_table_type(): String = table_type_map(getClass.getSimpleName)

  /**
   * 风险等级
   *
   * @param old_map
   * @param new_map
   * @return
   */
  protected def get_info_risk_level(old_map: Map[String, String], new_map: Map[String, String]): String = info_risk_level_map(getClass.getSimpleName)

  /**
   * 财产流向
   * 0 流入 1 流出
   *
   * @return
   */
  protected def get_flow_type(): String = flow_type_map(getClass.getSimpleName)


  /**
   * 变更时间
   *
   * @param new_map
   * @return
   */
  protected def get_change_time(bizDate: String, new_map: Map[String, String]): String = {
    var res = bizDate
    if (bizDate.length == 10) {
      res = res.concat(" 00:00:00")
    }
    res
  }

  /**
   * 业务id
   *
   * @param rowkey
   * @return
   */
  protected def get_biz_id(rowkey: String, new_map: Map[String, String]): String = rowkey

  /**
   * 条件过滤
   *
   * @return
   */
  def get_conditional_filter(): String = ""


  protected def getMoney(new_map: Map[String, String]) : String = "0"

  protected def processMoney(amt : String) :String ={
    var amtstr = amt
    if (amtstr == null ){
      "0"
    }else{
      amtstr = amtstr.replaceAll("[^\\d.]", "")

      if(amtstr.equals("") || amtstr.split("\\.").size > 2  || amtstr.startsWith(".") || amtstr.endsWith(".") ){
        amtstr = "0.0"
      }
      try{
        var x = amtstr.toDouble
      }catch {
        case ex : Exception =>{
          amtstr = "0"
        }

      }


      amtstr
    }
  }


}