package com.winhc.bigdata.spark.jobs.monitor.alldata.monitor.tables

import com.winhc.bigdata.spark.jobs.monitor.CompanyMonitorHandle
import com.winhc.bigdata.spark.utils.RegCapitalAmount
import org.apache.commons.lang3.StringUtils

/**
 * @Date: 2020/8/12 18:50
 * @Description: 工商数据
 */
case class company() extends CompanyMonitorHandle {
  /**
   * 信息描述
   *
   * @param old_map
   * @param new_map
   * @return
   */
  override protected def get_rta_desc(old_map: Map[String, String], new_map: Map[String, String]): String =
    s"""注册资本：${new_map("reg_capital")}
       |变更时间：${new_map("update_time")}""".stripMargin

  //注册资本上升
  override def handle(rowkey: String, bizDate: String, cid: String, change_fields: Seq[String], old_map: Map[String, String], new_map: Map[String, String], cname: String = null): Seq[(String, String, String, String, String, String, String, String, String, String)] = {
    if (!compareAmount(old_map, new_map)) {
      return Seq.empty
    }
    try {
      Seq((cid
        , cname
        , get_table_type()
        , get_flow_type()
        , get_rta_desc(old_map, new_map)
        , get_change_time(bizDate, new_map)
        , get_biz_id(rowkey, new_map)
        , get_info_risk_level(old_map, new_map)
        , get_type()
        , getMoney(new_map)
      ))
    } catch {
      case e: Exception => {
        logError(e.getMessage, e)
      }
        Seq.empty
    }
  }

  /**
   * 来源表的变更类型，insert or update
   *
   * @return
   */
  override def org_type(): Seq[String] = {
    Seq("update")
  }

  /**
   * 注册资本比较
   */
  def compareAmount(old_map: Map[String, String], new_map: Map[String, String]): Boolean = {
    if (old_map == null || new_map == null) {
      return false
    }
    val old = old_map("reg_capital")
    val now = new_map("reg_capital")
    if (StringUtils.isBlank(old) || StringUtils.isBlank(now)) {
      return false
    }
    val old_num = RegCapitalAmount.getAmount(old)
    val now_num = RegCapitalAmount.getAmount(now)
    if (StringUtils.isBlank(old_num) || StringUtils.isBlank(now_num)) {
      return false
    }
    old_num.toDouble < now_num.toDouble
  }

}
