package com.winhc.bigdata.spark.model

import java.util.Date

import com.winhc.bigdata.calc.DimScoreV2
import com.winhc.bigdata.spark.utils.BaseUtil.atMonthsBefore
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, Maxcomputer2Hbase, SparkUtils}
import org.apache.spark.sql.{Row, SparkSession}

import scala.annotation.meta.getter
import scala.collection.mutable


object CompanyEmploymentScore {
  def main(args: Array[String]): Unit = {
    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_eci_dev",
      "spark.hadoop.odps.spark.local.partition.amt" -> "1000"
    )
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)
    CompanyEmploymentScore(spark, "company_employment", "", "208", "start_date", "经营情况", "招聘", "0", "winhc_eci_dev").calc()
    spark.stop()
  }
}

/**
 * 招聘得分
 */
case class CompanyEmploymentScore(s: SparkSession, sourceTable: String, tableView: String = "",
                                  flag: String, time: String, kind: String, project: String,
                                  tp: String = "0", namespace: String = ""
                                 ) extends LoggingUtils {

  @(transient@getter) val spark: SparkSession = s

  import spark.implicits._

  def calc(): Unit = {
    println(s"${this.getClass.getSimpleName} calc start! " + new Date().toString)
    //val targetTable = "ads_company_total_score"
    val adsTable = namespace + ".ads_" + sourceTable
    val incAdsTable = namespace + ".inc_ads_" + sourceTable
    val targetTable = namespace + ".ads_" + sourceTable + "_score"
    var ds = ""

    //区别有无分区表
    var appsql2 = ""
    var tb = adsTable
    if ("1".equals(tp)) {
      tb = tableView
      ds = BaseUtil.getPartion(incAdsTable, spark)
    } else {
      ds = BaseUtil.getPartion(adsTable, spark)
      appsql2 = s"AND  ds = ${ds}"
    }

    val df = sql(
      s"""
         |SELECT  *
         |FROM    (
         |        SELECT
         |                *
         |                ,COUNT(new_cid) OVER(PARTITION BY new_cid ) AS cnt1
         |                ,SUM(CASE WHEN cast(start_date as string) >= '${atMonthsBefore(12)}' THEN 1 ELSE 0 END) OVER(PARTITION BY new_cid ) AS cnt2
         |                ,ROW_NUMBER() OVER(PARTITION BY new_cid ORDER BY $time DESC ) AS num
         |        FROM    $tb
         |        WHERE   new_cid IS NOT NULL
         |        ${appsql2}
         |        ) a
         |WHERE   num =1
         |""".stripMargin)

    df.map(r => {
      trans(r, flag, kind, project)
    }).toDF("id", "cid", "kind", "kind_code", "project", "project_code", "type",
      "score", "total", "extraScore")
      .createOrReplaceTempView(s"t1_view")

    sql(s"insert overwrite table ${targetTable} " +
      s"partition (ds='${ds}')  select * from t1_view")

    //同步hbase
    if ("1".equals(tp)) { //存量计算不用同步hbase
      val dataFrame = sql(
        s"""
           |select
           |CONCAT_WS('_',cid,project_code) AS rowkey,
           |id,cid,kind,kind_code,project,project_code,type,score,total,extraScore
           |from t1_view
           |""".stripMargin)
      Maxcomputer2Hbase(dataFrame, "COMPANY_SCORE").syn()
    }
    println(s"${this.getClass.getSimpleName} calc end! " + new Date().toString)
  }

  def trans(r: Row, flag: String, kind: String, prpject: String) = {
    val id = r.getAs[Long]("id")
    val cid = r.getAs[Long]("new_cid").toString
    val cnt1 = r.getAs[Long]("cnt1")
    val cnt2 = r.getAs[Long]("cnt2")
    flag match {
      case "208" => employmentScore(id, cid, cnt1, cnt2, kind, prpject)
    }
  }

  //招聘
  def employmentScore(id: Long, cid: String, cnt1: Long, cnt2: Long, kind: String, project: String) = {
    var score = 0f
    val total = 5f
    val extraScore = 0f
    var ty = ""
    if (cnt2 > 0) {
      score = 5f
      ty = "近一年内有招聘信息"
    } else if (cnt1 > 0) {
      score = 4f
      ty = "有招聘信息，但是近1年内无招聘信息"
    } else {
      score = 3f
      ty = "无招聘信息"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project, DimScoreV2.newsEventMap.get(project), ty,
      score, total, extraScore)
  }

}