package com.winhc.bigdata.spark.model

import com.winhc.bigdata.spark.udf.CompanyMapping
import com.winhc.bigdata.spark.utils.SparkUtils
import org.apache.commons.logging.LogFactory
import org.apache.spark.sql.SparkSession

import scala.collection.mutable

object CompanyInfoCalculator extends CompanyMapping {

  private val LOG = LogFactory.getLog(this.getClass)

  def main(args: Array[String]): Unit = {

    if (args.length != 3) {
      println("请配置计算资源: instances, cores, memory .")
      System.exit(-1)
    }

    var config = mutable.Map.empty[String, String]
    val Array(instances, cores, memory) = args;

    println(
      s"""
         |instances : $instances,
         |cores : $cores,
         |memory : $memory
         |""".stripMargin)

    config = mutable.Map("spark.executor.instances" -> instances,
      "spark.executor.cores" -> cores,
      "spark.executor.memory" -> memory
    )

    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)

    import spark._
    println("company calc start!   ")

    prepare(spark)

    val tableName = "ads_company"
    val resultTable = "ads_company_score"

    val df = sql(
      s"""
         |SELECT  id
         |        ,company_name
         |        ,legal_person_name
         |        ,reg_capital
         |        ,reg_capital_score(reg_capital) AS reg_capital_score
         |        ,reg_capital_num
         |        ,actual_capital
         |        ,actual_capital_score(actual_capital,reg_capital) AS actual_capital_score
         |        ,category_code
         |        ,industry_score(category_code) AS industry_score
         |        ,company_type
         |        ,company_org_type
         |        ,company_type_score(company_org_type) AS company_type_score
         |        ,reg_status
         |        ,company_status_score(reg_status) AS company_status_score
         |        ,estiblish_time
         |        ,start_date_score(CAST(estiblish_time AS STRING)) AS start_date_score
         |        ,from_time
         |        ,to_time
         |        ,end_date_score(CAST(from_time AS STRING) ,CAST(to_time AS STRING)) AS end_date_score
         |        ,reg_location
         |        ,reg_location_score(reg_location) AS reg_location_score
         |FROM    ${tableName}
         |""".stripMargin)

    df.write.mode("overwrite").insertInto(resultTable)

    println("company calc end!   ")

    spark.stop();
  }


}
