package com.winhc.bigdata.spark.model

import java.util.Date

import com.winhc.bigdata.calc.DimScoreV2
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, Maxcomputer2Hbase, SparkUtils}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.broadcast.Broadcast
import org.apache.spark.sql.{Row, SparkSession}

import scala.collection.mutable

/**
 * @Description:公司基本信息评分
 * @author π
 * @date 2020/9/14 16:52
 */
object CompanyInfoCalculatorV2 {

  def main(args: Array[String]): Unit = {

    val project = "winhc_eci_dev"
    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> s"$project",
      "spark.hadoop.odps.spark.local.partition.amt" -> "1000"
    )
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)
    CompanyInfoCalculatorV2(spark, "company", "", "", "", "", "", "0", s"$project").calc()
    spark.stop()
  }

}


case class CompanyInfoCalculatorV2(s: SparkSession, sourceTable: String, tableView: String = "",
                                   flag: String, time: String, kind: String, project: String,
                                   tp: String = "0", namespace: String = "") extends LoggingUtils {

  override protected val spark: SparkSession = s

  import spark.implicits._

  def calc(): Unit = {
    println("company calc start! " + new Date().toString)

    val adsTable = s"$namespace.ads_company" //公司基本信息表
    val incAdsTable = s"$namespace.inc_ads_company" //增量公司基本信息表
    val company_score = s"$namespace.ads_company_score"
    val company_category = s"$namespace.const_company_category_code"
    val company_stock = s"$namespace.ods_company_stock"

    var ds = ""
    //区别有无分区表
    var appsql2 = ""
    var tb = adsTable
    if ("1".equals(tp)) {
      tb = tableView
      ds = BaseUtil.getPartion(incAdsTable, spark)
    } else {
      ds = BaseUtil.getPartion(adsTable, spark)
      appsql2 = s"AND  ds = ${ds}"
    }

    //所属行业
    val code2Name: Broadcast[Map[String, String]] = spark.sparkContext.broadcast(sql(
      s"""
         |select category_code,category_str_big
         |from $company_category
      """.stripMargin).collect().map(r => {
      (r.getString(0), r.getString(1))
    }).toMap)

    //上市信息
    val stock: Broadcast[Map[String, String]] = spark.sparkContext.broadcast(sql(
      s"""
         |select cid,name
         |from $company_stock
      """.stripMargin).collect().map(r => {
      (r.getLong(0).toString, "1")
    }).toMap)

    sql(
      s"""
         |SELECT  id,cid,name
         |        ,reg_capital
         |        ,reg_capital_amount
         |        ,reg_capital_currency
         |        ,actual_capital_amount
         |        ,actual_capital_currency
         |        ,category_code
         |        ,company_type
         |        ,company_org_type
         |        ,reg_status
         |        ,CAST(estiblish_time AS STRING) estiblish_time
         |        ,CAST(from_time AS STRING) from_time
         |        ,CAST(to_time AS STRING) to_time
         |        ,reg_location
         |FROM    ${tb}
         |where   cid is not null ${appsql2}
         |""".stripMargin).flatMap(r => {
      trans(stock, code2Name, r)
    }).toDF("id", "cid", "kind", "kind_code", "project", "project_code", "type", "score", "total", "extraScore")
      .createOrReplaceTempView(s"${sourceTable}_tmp_view")

    sql(s"select * from ${sourceTable}_tmp_view").show(100)

    sql(s"insert overwrite table ${company_score}  partition (ds='${ds}') select * from ${sourceTable}_tmp_view")

    //同步hbase
    if ("1".equals(tp)) { //存量计算不用同步hbase
      val dataFrame = sql(
        s"""
           |select
           |CONCAT_WS('_',cid,project_code) AS rowkey,
           |id,cid,kind,kind_code,project,project_code,type,score,total,extraScore
           |from ${sourceTable}_tmp_view
           |""".stripMargin)

      Maxcomputer2Hbase(dataFrame, "COMPANY_SCORE").syn()
    }

    println("company calc end! " + new Date().toString)
  }

  private def trans(stock: Broadcast[Map[String, String]], code2Name: Broadcast[Map[String, String]], r: Row) = {

    val id = r.getAs[Long]("id")
    val cid = r.getAs[Long]("cid").toString
    //val name = r.getAs[String]("name")
    val reg_capital = r.getAs[String]("reg_capital")
    val actual_capital_amount = r.getAs[Long]("actual_capital_amount")
    val actual_capital_currency = r.getAs[String]("actual_capital_currency")
    val category_code = r.getAs[String]("category_code")
    val company_org_type = r.getAs[String]("company_org_type")
    val reg_status = r.getAs[String]("reg_status")
    val estiblish_time = r.getAs[String]("estiblish_time")
    val to_time = r.getAs[String]("to_time")
    val reg_location = r.getAs[String]("reg_location")

    var actual_capital = ""
    //实缴资本单位 分转换元
    if (actual_capital_amount > 0 && StringUtils.isNotBlank(actual_capital_currency)) {
      actual_capital = actual_capital_amount / 100 + "元" + actual_capital_currency
    }

    val r1 = DimScoreV2.bean2Map(DimScoreV2.registCapiScore(reg_capital))
    val r2 = DimScoreV2.bean2Map(DimScoreV2.recCapScore(actual_capital, reg_capital))
    val r3 = DimScoreV2.bean2Map(DimScoreV2.industryScore(code2Name.value.getOrElse(category_code, null)))
    val r4 = DimScoreV2.bean2Map(DimScoreV2.econKindScore(company_org_type, stock.value.getOrElse(cid, null))) //注意解决是否上市
    val r5 = DimScoreV2.bean2Map(DimScoreV2.eciStatusScore(reg_status))
    val r6 = DimScoreV2.bean2Map(DimScoreV2.startDateScore(estiblish_time))
    val r7 = DimScoreV2.bean2Map(DimScoreV2.termScore(to_time))
    val r8 = DimScoreV2.bean2Map(DimScoreV2.addressScore(reg_location))

    Seq(r1, r2, r3, r4, r5, r6, r7, r8)
      .map(m => (id, cid, m.get("kind"), m.get("kind_code"), m.get("project"), m.get("project_code"),
        m.get("type"), m.get("score").toFloat, m.get("total").toFloat, m.get("extraScore").toFloat))
  }


}