package com.winhc.bigdata.spark.model

import java.util.Date

import com.winhc.bigdata.calc.DimScoreV2
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, Maxcomputer2Hbase, SparkUtils}
import org.apache.spark.sql.{Row, SparkSession}

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * 知识产权类得分
 * 软件著作权 | 作品著作权 | 专利 | 商标 | 网站
 * π
 */

object CompanyIntellectualsScore {

  val tabMapping: Map[String, (String, String, String, String)] =
    Map("company_copyright_reg_list" -> ("1", "publish_time", "资产权益", "软著作权"), //软件著作权
      "company_copyright_works_list" -> ("2", "finish_time", "资产权益", "著作权"), //作品著作权
      "company_patent_list" -> ("3;4", "pub_date", "资产权益", "实用新型、外观设计专利;发明专利"), //专利
      "company_icp" -> ("5", "examine_date", "资产权益", "网站"), //网站
      "company_tm" -> ("6", "app_date", "资产权益", "商标"), //商标
      "company_land_announcement" -> ("7", "commit_time", "资产权益", "购地信息"), //购地信息
      "company_land_publicity" -> ("8", "publication_start_date", "资产权益", "土地公示"), //地块公示
      "company_employment" -> ("208", "start_date", "经营情况", "招聘") //招聘
    )

  def main(args: Array[String]): Unit = {

    val (sourceTable, flag, time, kind, project, namespace) = valid(args)

    var config = mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_eci_dev",
      "spark.hadoop.odps.spark.local.partition.amt" -> "10"
    )
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)

    println(s"company ${this.getClass.getSimpleName} calc start! " + new Date().toString)

    //专利分成两部分
    if (flag.contains(";")) {
      flag.split(";").foreach(f => {
        new CompanyIntellectualsScore(spark, sourceTable, "", f, time, kind, project, "0", namespace + ".").calc()
      })
    } else {
      new CompanyIntellectualsScore(spark, sourceTable, "", flag, time, kind, project, "0", namespace + ".").calc()
    }

    println(s"company ${this.getClass.getSimpleName} calc end! " + new Date().toString)
    spark.stop()

  }

  def valid(args: Array[String]) = {
    if (args.length != 2) {
      println("please insert namespace,table !!!! ")
      sys.exit(-1)
    }
    val Array(namespace, sourceTable) = args

    val (flag, time, kind, project) = tabMapping.getOrElse(sourceTable, ("", "", "", ""))
    if (flag.isEmpty || time.isEmpty || kind.isEmpty || project.isEmpty) {
      println("table not find!!!   ")
      sys.exit(-1)
    }

    (sourceTable, flag, time, kind, project, namespace)
  }

  //知识产权模型计算
  def start(spark: SparkSession, namespace: String, sourceTable: String, tableView: String): Unit = {
    val (flag, time, kind, project) = tabMapping.getOrElse(sourceTable, ("", "", "", ""))
    if (flag.isEmpty || time.isEmpty || kind.isEmpty || project.isEmpty) {
      println("model table not find !!! ")
      //sys.exit(0)
      return
    }
    //招聘
    if (flag.equals("208")) {
      new CompanyEmploymentScore(spark, sourceTable, tableView, flag, time, kind, project, "1", namespace).calc()
      return
    }
    //专利分成两部分
    if (flag.contains(";")) {
      flag.split(";").foreach(f => {
        new CompanyIntellectualsScore(spark, sourceTable, tableView, f, time, kind, project, "1", namespace + ".").calc()
      })
    } else {
      new CompanyIntellectualsScore(spark, sourceTable, tableView, flag, time, kind, project, "1", namespace + ".").calc()
    }
  }
}

case class CompanyIntellectualsScore(s: SparkSession, sourceTable: String, tableView: String = "",
                                     flag: String, time: String, kind: String, project: String,
                                     tp: String = "0", namespace: String = ""
                                    ) extends LoggingUtils {

  @(transient@getter) val spark: SparkSession = s

  import spark.implicits._

  def calc(): Unit = {
    println(s"${this.getClass.getSimpleName} calc start! " + new Date().toString)
    //val targetTable = "ads_company_total_score"
    val adsTable = namespace + "ads_" + sourceTable
    val incAdsTable = namespace + "inc_ads_" + sourceTable
    val targetTable = namespace + "ads_" + sourceTable + "_score"
    var ds = ""

    //区别有无分区表
    var appsql2 = ""
    var tb = adsTable
    if ("1".equals(tp)) {
      tb = tableView
      ds = BaseUtil.getPartion(incAdsTable, spark)
    } else {
      ds = BaseUtil.getPartion(adsTable, spark)
      appsql2 = s"AND  ds = ${ds}"
    }

    var appsql = ""
    var apptab = ""

    // 专利类型(1（A）.发明公布，2（B,C）.发明授权，3（U，Y）.实用新型，4（D，S）.外观设计)
    if ("3".equals(flag)) { //实用新型、外观设计专利
      appsql = "AND SUBSTR(pub_number, -1) in ('U','Y','D','S')"
      apptab = "_s"
    } else if ("4".equals(flag)) { //发明专利
      appsql = "AND SUBSTR(pub_number, -1) in ('A','B','C')"
      apptab = "_f"
    }

    val df = sql(
      s"""
         |SELECT  *
         |FROM    (
         |        SELECT
         |                *
         |                ,COUNT(new_cid) OVER(PARTITION BY new_cid ) AS cnt1
         |                ,row_number() OVER(PARTITION BY new_cid ORDER BY $time DESC ) AS num
         |        FROM    $tb
         |        WHERE   new_cid IS NOT NULL
         |        ${appsql2} ${appsql}
         |        ) a
         |WHERE   num =1
         |""".stripMargin)

    df.map(r => {
      trans(r, flag, kind, project)
    }).toDF("id", "cid", "kind", "kind_code", "project", "project_code", "type",
      "score", "total", "extraScore")
      .createOrReplaceTempView(s"t1_view")

    //    logInfo(
    //      s"""
    //         |- - - - - - - - - - - - - - - - - - - - - - - - -
    //         |${showString(sql(s"select * from t1_view"))}
    //         |- - - - - - - - - - - - - - - - - - - - - - - - -
    //               """.stripMargin)

    sql(s"insert overwrite table ${targetTable}${apptab} " +
      s"partition (ds='${ds}')  select * from t1_view")

    val dataFrame = sql(
      s"""
         |select
         |CONCAT_WS('_',cid,project_code) AS rowkey,
         |id,cid,kind,kind_code,project,project_code,type,score,total,extraScore
         |from t1_view
         |""".stripMargin)

    //同步hbase
    if ("1".equals(tp)) { //存量计算不用同步hbase
      Maxcomputer2Hbase(dataFrame, "COMPANY_SCORE").syn()
    }
    println(s"${this.getClass.getSimpleName} calc end! " + new Date().toString)
  }

  def trans(r: Row, flag: String, kind: String, prpject: String) = {
    val id = r.getAs[Long]("id")
    val cid = r.getAs[Long]("new_cid").toString
    val cnt1 = r.getAs[Long]("cnt1")
    flag match {
      case "1" => softCopyRightScore(id, cid, cnt1, kind, prpject)
      case "2" => copyRightScore(id, cid, cnt1, kind, prpject)
      case "3" => ordinaryPatentScore(id, cid, cnt1, kind, prpject)
      case "4" => inventScore(id, cid, cnt1, kind, prpject)
      case "5" => webSiteScore(id, cid, cnt1, kind, prpject)
      case "6" => tradeMarkScore(id, cid, cnt1, kind, prpject)
      case "7" => immovableScore(id, cid, cnt1, kind, prpject)
      case "8" => land_publicity(id, cid, cnt1, kind, prpject)
    }
  }

  //软著作权
  def softCopyRightScore(id: Long, cid: String, cnt1: Long, kind: String, project: String) = {
    var score = 0f
    val total = 3f
    val extraScore = 0f
    var ty = ""
    if (cnt1 > 50) {
      score = 8f
      ty = "软件著作权数量＞50，该项分数上加5分"
    } else if (cnt1 > 5) {
      score = 3f
      ty = "授予数量＞5"
    } else if (cnt1 > 0) {
      score = 2f
      ty = "授予数量≤5"
    } else {
      score = 1f
      ty = "无"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project, DimScoreV2.newsEventMap.get(project), ty,
      score, total, extraScore)
  }

  //著作权
  def copyRightScore(id: Long, cid: String, cnt1: Long, kind: String, project: String) = {
    var score = 0f
    val total = 2f
    val extraScore = 0f
    var ty = ""
    if (cnt1 > 3) {
      score = 2f
      ty = "授予数量＞3"
    } else if (cnt1 == 0) {
      score = 0f
      ty = "无"
    } else {
      score = 1f
      ty = "授予数量≤3"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project, DimScoreV2.newsEventMap.get(project), ty,
      score, total, extraScore)
  }

  //普通专利
  def ordinaryPatentScore(id: Long, cid: String, cnt1: Long, kind: String, project: String) = {
    val project_s = project.split(";")(0)
    var score = 0f
    val total = 3f
    val extraScore = 0f
    var ty = ""
    if (cnt1 == 0) {
      score = 1f
      ty = "无"
    } else if (cnt1 <= 5) {
      score = 2f
      ty = "授予数量≤5"
    } else {
      score = 3f
      ty = "授予数量＞5"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project_s, DimScoreV2.newsEventMap.get(project_s), ty,
      score, total, extraScore)
  }

  //发明专利
  def inventScore(id: Long, cid: String, cnt1: Long, kind: String, project: String) = {
    val project_s = project.split(";")(1)
    var score = 0f
    val total = 7f
    val extraScore = 0f
    var ty = ""
    if (cnt1 == 0) {
      score = 3f
      ty = "无"
    } else if (cnt1 <= 3) {
      score = 5f
      ty = "授予数量≤3"
    } else if (cnt1 <= 50) {
      score = 7f
      ty = "授予数量＞3"
    } else {
      score = 12f
      ty = "发明专利数量＞50，该项分数上加5分"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project_s, DimScoreV2.newsEventMap.get(project_s), ty,
      score, total, extraScore)
  }

  //网站
  def webSiteScore(id: Long, cid: String, cnt1: Long, kind: String, project: String) = {
    var score = 0f
    val total = 1f
    val extraScore = 0f
    var ty = ""
    if (cnt1 > 20) {
      score = 4f
      ty = "网站域名数量＞20，该项分数上加3分"
    } else if (cnt1 > 0) {
      score = 1f
      ty = "有"
    } else {
      score = 0f
      ty = "无"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project, DimScoreV2.newsEventMap.get(project), ty,
      score, total, extraScore)
  }

  //商标
  def tradeMarkScore(id: Long, cid: String, cnt1: Long, kind: String, project: String) = {
    var score = 0f
    val total = 4f
    val extraScore = 0f
    var ty = ""
    if (cnt1 == 0) {
      score = 0f
      ty = "无"
    } else if (cnt1 <= 2) {
      score = 2f
      ty = "授予数量≤2"
    } else if (cnt1 <= 8) {
      score = 3f
      ty = "授予数量＞2，≤8"
    } else if (cnt1 <= 100) {
      score = 4f
      ty = "授予数量＞8"
    } else {
      score = 9f
      ty = "商标数量＞100，该项分数上加5分"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project, DimScoreV2.newsEventMap.get(project), ty,
      score, total, extraScore)
  }

  //购地信息
  def immovableScore(id: Long, cid: String, cnt1: Long, kind: String, project: String) = {
    var score = 0f
    val total = 15f
    val extraScore = 0f
    var ty = ""
    if (cnt1 == 0) {
      score = 7f
      ty = "无"
    } else if (cnt1 <= 2) {
      score = 12f
      ty = "≤2"
    } else {
      score = 15f
      ty = "＞2"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project, DimScoreV2.newsEventMap.get(project), ty,
      score, total, extraScore)
  }

  //地块公示
  def land_publicity(id: Long, cid: String, cnt1: Long, kind: String, project: String) = {
    var score = 0f
    val total = 15f
    val extraScore = 0f
    var ty = ""
    if (cnt1 == 0) {
      score = 7f
      ty = "无"
    } else if (cnt1 <= 2) {
      score = 12f
      ty = "≤2"
    } else {
      score = 15f
      ty = "＞2"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project, DimScoreV2.newsEventMap.get(project), ty,
      score, total, extraScore)
  }

  //招聘
  def company_employment(id: Long, cid: String, cnt1: Long, kind: String, project: String) = {
    var score = 0f
    val total = 5f
    val extraScore = 0f
    var ty = ""
    if (cnt1 == 0) {
      score = 3f
      ty = "无招聘信息"
    } else if (cnt1 <= 2) {
      score = 12f
      ty = "有招聘信息，但是近1年内无招聘信息"
    } else {
      score = 15f
      ty = "近一年内有招聘信息"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project, DimScoreV2.newsEventMap.get(project), ty,
      score, total, extraScore)
  }

}
