package com.winhc.bigdata.spark.model

import java.util.Date
import com.winhc.bigdata.calc.DimScoreV2
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, Maxcomputer2Hbase, SparkUtils}
import org.apache.spark.sql.{Row, SparkSession}
import scala.annotation.meta.getter
import scala.collection.mutable


/**
 * @Description:地块公示,购地信息
 * @author π
 * @date 2020/9/316:52
 */
object CompanyLandScoreV1 {
  def main(args: Array[String]): Unit = {
    val Array(project) = args
    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> s"$project",
      "spark.hadoop.odps.spark.local.partition.amt" -> "1000"
    )
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)
    //土地公示
    CompanyLandScoreV1(spark, "company_land_publicity", "", "304", "update_time", "资产权益", "土地公示", "0", s"$project").calc()
    //购地信息
    CompanyLandScoreV1(spark, "company_land_announcement", "", "305", "update_time", "资产权益", "购地信息", "0", s"$project").calc()
    spark.stop()
  }
}

case class CompanyLandScoreV1(s: SparkSession, sourceTable: String, tableView: String = "",
                              flag: String, time: String, kind: String, project: String,
                              tp: String = "0", namespace: String = ""
                             ) extends LoggingUtils {

  @(transient@getter) val spark: SparkSession = s

  import spark.implicits._

  def calc(): Unit = {
    println(s"${this.getClass.getSimpleName} calc start! " + new Date().toString)
    val adsTable = namespace + ".ads_" + sourceTable
    val incAdsTable = namespace + ".inc_ads_" + sourceTable
    val targetTable = namespace + ".ads_" + sourceTable + "_score"
    var ds = ""

    //区别有无分区表
    var appsql2 = ""
    var tb = adsTable
    if ("1".equals(tp)) {
      tb = tableView
      ds = BaseUtil.getPartion(incAdsTable, spark)
    } else {
      ds = BaseUtil.getPartion(adsTable, spark)
      appsql2 = s"AND  ds = ${ds}"
    }

    val df = sql(
      s"""
         |SELECT  *
         |FROM    (
         |        SELECT
         |                *
         |                ,COUNT(new_cid) OVER(PARTITION BY new_cid ) AS cnt1
         |                ,ROW_NUMBER() OVER(PARTITION BY new_cid ORDER BY $time DESC ) AS num
         |        FROM    $tb
         |        WHERE   new_cid IS NOT NULL
         |        ${appsql2}
         |        ) a
         |WHERE   num =1
         |""".stripMargin)

    df.map(r => {
      trans(r, flag, kind, project)
    }).toDF("id", "cid", "kind", "kind_code", "project", "project_code", "type",
      "score", "total", "extraScore")
      .createOrReplaceTempView(s"t1_view")

    sql("select * from t1_view").show(20, false)

    sql(s"insert overwrite table ${targetTable} " +
      s"partition (ds='${ds}')  select * from t1_view")

    //同步hbase
    if ("1".equals(tp)) { //存量计算不用同步hbase
      val dataFrame = sql(
        s"""
           |select
           |CONCAT_WS('_',cid,project_code) AS rowkey,
           |id,cid,kind,kind_code,project,project_code,type,score,total,extraScore
           |from t1_view
           |""".stripMargin)

      Maxcomputer2Hbase(dataFrame, "COMPANY_SCORE").syn()
    }
    println(s"${this.getClass.getSimpleName} calc end! " + new Date().toString)
  }

  def trans(r: Row, flag: String, kind: String, prpject: String) = {
    val id = r.getAs[Long]("id")
    val cid = r.getAs[Long]("new_cid").toString
    val cnt1 = r.getAs[Long]("cnt1")

    flag match {
      case "304" => getLandpubScore(id, cid, cnt1, kind, prpject)
      case "305" => getLandbuyScore(id, cid, cnt1, kind, prpject)
    }
  }

  //地块公示
  def getLandpubScore(id: Long, cid: String, cnt1: Long, kind: String, project: String) = {
    var score = 0f
    val total = 15f
    val extraScore = 0f
    var ty = ""
    if (cnt1 == 0) {
      score = 7f
      ty = "无"
    } else if(cnt1 <= 2) {
      score = 12f
      ty = "≤2"
    }else{
      score = 15f
      ty = "＞2"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project, DimScoreV2.newsEventMap.get(project), ty,
      score, total, extraScore)
  }

  //购地信息
  def getLandbuyScore(id: Long, cid: String, cnt1: Long, kind: String, project: String) = {
    var score = 0f
    val total = 15f
    val extraScore = 0f
    var ty = ""
    if (cnt1 == 0) {
      score = 7f
      ty = "无"
    } else if(cnt1 <= 2) {
      score = 12f
      ty = "≤2"
    }else{
      score = 15f
      ty = "＞2"
    }
    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project, DimScoreV2.newsEventMap.get(project), ty,
      score, total, extraScore)
  }

}