package com.winhc.bigdata.spark.model

import java.util.Date

import cn.hutool.core.util.StrUtil
import com.winhc.bigdata.calc.DimScoreV2
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, Maxcomputer2Hbase, SparkUtils}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.{Row, SparkSession}

import scala.annotation.meta.getter
import scala.collection.mutable


/**
 * @Description:行政处罚,行政处罚信用中国得分
 * @author π
 * @date 2020/9/3 16:52
 */
object CompanyPunishmentInfo {
  def main(args: Array[String]): Unit = {
    val project = "winhc_eci_dev"
    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> s"$project",
      "spark.hadoop.odps.spark.local.partition.amt" -> "1000"
    )
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)

    //行政处罚
    CompanyPunishmentInfo(spark, "company_punishment_info", "", "410", "update_time", "经营风险", "行政处罚", "0", s"$project").calc()
    //行政处罚信用中国
    CompanyPunishmentInfo(spark, "company_punishment_info_creditchina", "", "410", "update_time", "经营风险", "行政处罚", "0", s"$project").calc()
    spark.stop()
  }
}


case class CompanyPunishmentInfo(s: SparkSession, sourceTable: String, tableView: String = "",
                                 flag: String, time: String, kind: String, project: String,
                                 tp: String = "0", namespace: String = ""
                                ) extends LoggingUtils {

  @(transient@getter) val spark: SparkSession = s

  import spark.implicits._

  def calc(): Unit = {
    println(s"${this.getClass.getSimpleName} calc start! " + new Date().toString)
    val adsTable = namespace + ".ads_" + sourceTable
    val incAdsTable = namespace + ".inc_ads_" + sourceTable
    val targetTable = namespace + ".ads_" + sourceTable + "_score"
    var ds = ""

    //行政处罚内容字段
    var content = "content"
    if (sourceTable.contains(s"creditchina")) {
      content = "result"
    }

    //区别有无分区表
    var appsql2 = ""
    var tb = adsTable
    if ("1".equals(tp)) {
      tb = tableView
      ds = BaseUtil.getPartion(incAdsTable, spark)
    } else {
      ds = BaseUtil.getPartion(adsTable, spark)
      appsql2 = s"AND  ds = ${ds}"
    }

    val df = sql(
      s"""
         |SELECT
         |        new_cid,concat_ws(',' , collect_set($content)) content
         |FROM    $tb
         |WHERE   new_cid IS NOT NULL
         |${appsql2}
         |GROUP BY new_cid
         |""".stripMargin)

    df.map(r => {
      trans(r, flag, kind, project)
    }).toDF("id", "cid", "kind", "kind_code", "project", "project_code", "type",
      "score", "total", "extraScore")
      .createOrReplaceTempView(s"t1_view")

    sql("select * from t1_view").show(20, false)

    sql(s"insert overwrite table ${targetTable} " +
      s"partition (ds='${ds}')  select * from t1_view")

    //同步hbase
    if ("1".equals(tp)) { //存量计算不用同步hbase
      val dataFrame = sql(
        s"""
           |select
           |CONCAT_WS('_',cid,project_code) AS rowkey,
           |id,cid,kind,kind_code,project,project_code,type,score,total,extraScore
           |from t1_view
           |""".stripMargin)

      Maxcomputer2Hbase(dataFrame, "COMPANY_SCORE").syn()
    }
    println(s"${this.getClass.getSimpleName} calc end! " + new Date().toString)
  }

  def trans(r: Row, flag: String, kind: String, prpject: String) = {
    val id = -1
    val cid = r.getAs[Long]("new_cid").toString
    val content = r.getAs[String]("content")
    getScore(id, cid, kind, prpject, content)
  }

  //行政处罚
  def getScore(id: Long, cid: String, kind: String, project: String, content: String) = {
    var score = 0f
    val total = 5f
    val extraScore = 0f
    var ty = ""
    if (StringUtils.isNotBlank(content)) {
      val bool1 = StrUtil.containsAny(content, "停业关闭", "暂扣", "吊销", "许可证", "执照")
      if (bool1) {
        score = 0f
        ty = "责令停产停业关闭、暂扣或吊销许可证、执照的行政处罚"
      } else {
        score = 2f
        ty = "警告、罚款、停产整顿、没收违法所得、行政拘留等其他行政处罚"
      }
    }

    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project, DimScoreV2.newsEventMap.get(project), ty,
      score, total, extraScore)
  }

}