package com.winhc.bigdata.spark.ng.change

import com.winhc.bigdata.spark.utils.BaseUtil.cleanup
import org.apache.commons.lang3.StringUtils
import org.apache.spark.internal.Logging

import scala.annotation.meta.{getter, setter}

/**
 * @Author: XuJiakai
 * @Date: 2020/7/9 16:44
 * @Description:
 */

trait NgCompanyChangeHandle extends Serializable with Logging {
  @getter
  @setter
  protected val equCols: Seq[String]
  @getter
  @setter
  protected val is_inc: Boolean //false 为存量

  /**
   * 主入口函数
   *
   * @param rowkey
   * @param oldMap
   * @param newMap
   * @return rowkey,company_id,类型【insert or update or deleted】,老数据,新数据，更新字段，更新标题，数据标签,业务时间
   */
  def handle(rowkey: String, oldMap: Map[String, String], newMap: Map[String, String]): (String, String, String, Map[String, String], Map[String, String], String, String) = {
    var dynamic_type: NgCompanyUpdateType.UpdateType = NgCompanyUpdateType.Other

    val company_id = getCompanyId(rowkey, newMap)
    var update_fields: String = null

    val biz_time = getBizDate(newMap)

    if (oldMap == null) {
      newMap.getOrElse("deleted", "0") match {
        case "0" => {
          dynamic_type = NgCompanyUpdateType.Insert
        }
        case "1" => {
          dynamic_type = NgCompanyUpdateType.Deleted
        }
        case _ => null
      }
    } else {
      val new_deleted = newMap.getOrElse("deleted", "0")
      val old_deleted = oldMap.getOrElse("deleted", "0")
      update_fields = getNotEquFields(oldMap, newMap)
      if (update_fields == null) {
        //没有发生字段变化则直接过滤
        return null
      }
      s"$old_deleted$new_deleted" match {
        case "90" => dynamic_type = NgCompanyUpdateType.Insert

        case "00" => dynamic_type = NgCompanyUpdateType.Update

        case "01" | "91" => dynamic_type = NgCompanyUpdateType.Deleted

        case "09" | "19" => dynamic_type = NgCompanyUpdateType.Remove

        case "10" | "11" | "99" => null

        case _ => null
      }
    }

    (rowkey, company_id, dynamic_type.toString, oldMap, newMap,
      update_fields
      , biz_time)
  }

  /**
   * 获取公司company_id，默认为rowkey前半段
   *
   * @param rowkey
   * @param newMap
   * @return
   */
  protected def getCompanyId(rowkey: String, newMap: Map[String, String]): String = rowkey.split("_")(0)

  /**
   * 获取变更的业务时间
   *
   * @param newMap
   * @return
   */
  protected def getBizDate(newMap: Map[String, String]): String

  /**
   * 判断哪些字段不相同
   *
   * @param oldMap
   * @param newMap
   * @return true（相同） 差异字段
   */
  protected def getNotEquFields(oldMap: Map[String, String], newMap: Map[String, String]): String = {
    var cols: Seq[String] = equCols
    if (oldMap.contains("deleted") && newMap.contains("deleted")) {
      cols = equCols :+ "deleted"
    }
    val tmp = cols.distinct.map(f => (f, cleanup(newMap(f)).equals(cleanup(oldMap(f)))))
    val eq = tmp.map(_._2).reduce((a1, a2) => a1 && a2)
    if (eq) {
      null
    } else {
      tmp.filter(!_._2).map(_._1).mkString(",")
    }
  }

  protected def getValueOrNull(value: String, callBack: String): String = if (StringUtils.isNotBlank(value)) callBack else null
}