package com.winhc.bigdata.spark.ng.credit_punishment

import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.LoggingUtils
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter

/**
 * @author: XuJiakai
 * @date: 2021/5/13 14:59
 *
 *        将信用惩戒主体id标记删除
 */
case class CreditPunishmentEntityRemove(s: SparkSession
                                       ) extends LoggingUtils with BaseFunc {
  @(transient@getter) val spark: SparkSession = s


  val target_tab = CreditPunishment.entityRemoveTargetTab


  def init(): Unit = {
    sql(
      s"""
         |CREATE TABLE IF NOT EXISTS $target_tab
         |(
         |    credit_punishment_case_id  STRING COMMENT '信用惩戒对象主体id，没有人的id采用的是案件id'
         |    ,deleted BIGINT COMMENT '0：未删除'
         |)
         |COMMENT 'bds_credit_punishment_entity_info表需要移除的主体id'
         |PARTITIONED BY (ds STRING COMMENT '分区')
         |""".stripMargin)
  }


  def calc(ds: String): Unit = {
    sql(
      s"""
         |INSERT OVERWRITE TABLE $target_tab PARTITION(ds='$ds')
         |SELECT  t1.credit_punishment_case_id
         |        ,9 AS deleted
         |FROM    (
         |            SELECT  *
         |            FROM    ${CreditPunishment.caseTargetTab}
         |            WHERE   ds = '$ds'
         |        ) AS t1
         |JOIN    (
         |            SELECT  *
         |            FROM    (
         |                        SELECT  *
         |                                ,ROW_NUMBER() OVER(PARTITION BY credit_punishment_case_id ORDER BY ds DESC) AS num
         |                        FROM    (
         |                                    SELECT  *
         |                                    FROM    ${CreditPunishment.caseTargetTab}
         |                                    WHERE   ds < '$ds'
         |                                )
         |                    )
         |            WHERE   num = 1
         |        ) AS t2
         |ON      t1.credit_punishment_case_id = t2.credit_punishment_case_id
         |AND     t1.credit_punishment_entity_id <> t2.credit_punishment_entity_id
         |AND     t1.credit_punishment_case_id <> t1.credit_punishment_entity_id
         |AND     t2.credit_punishment_case_id = t2.credit_punishment_entity_id
         |""".stripMargin)

  }

}
