package com.winhc.bigdata.spark.ng.credit_punishment.udf

import org.apache.spark.sql.Row
import org.apache.spark.sql.expressions.{MutableAggregationBuffer, UserDefinedAggregateFunction}
import org.apache.spark.sql.types.{DataType, LongType, StructField, StructType}

/**
 * @author: XuJiakai
 * @date: 2021/5/12 10:26
 *
 *        用于合并deleted状态
 *        0 1 => 0
 *        1 1 => 1
 *        0 0 => 0
 */
class DeletedMergeUDF extends UserDefinedAggregateFunction {
  override def inputSchema: StructType = StructType(Array[StructField](
    StructField("deleted", LongType)
  ))

  override def bufferSchema: StructType = StructType(Array[StructField](
    StructField("deleted", LongType)
  ))

  override def dataType: DataType = LongType

  override def deterministic: Boolean = false

  override def initialize(buffer: MutableAggregationBuffer): Unit = buffer.update(0, 1l)

  override def update(buffer: MutableAggregationBuffer, input: Row): Unit = {
    val del = input.getLong(0)
    if (del == 0l) {
      buffer(0) = 0l
    }
  }

  override def merge(buffer1: MutableAggregationBuffer, buffer2: Row): Unit = {
    val del = buffer2.getLong(0)
    if (del == 0l) {
      buffer1(0) = 0l
    }
  }

  override def evaluate(buffer: Row): Any = buffer.getLong(0)
}
