package com.winhc.bigdata.spark.ng.dynamic

import com.alibaba.fastjson.JSON
import com.winhc.bigdata.spark.ng.dynamic.NgCompanyRiskLevelType.NgCompanyRiskLevelType
import com.winhc.bigdata.spark.ng.dynamic.utils.CompanyDynamicUtils
import com.winhc.bigdata.spark.utils.BaseUtil
import org.apache.commons.lang3.StringUtils
import org.apache.spark.internal.Logging

import scala.annotation.meta.{getter, setter}
import scala.collection.mutable

/**
 * @author: XuJiakai
 * @date: 2021/6/23 10:35
 */
trait NgCompanyDynamicHandle extends Serializable with Logging {

  @getter
  @setter
  protected val is_inc: Boolean //false 为存量


  def filter: (String, String, Seq[String], Map[String, String], Map[String, String]) => Boolean = CompanyDynamicUtils.default_filter

  def flat_map: (ChangeExtract) => Seq[CompanyDynamicRecord]


  /**
   * group_by前置处理程。flat_map
   *
   * @return
   */
  def group_by_pre: (CompanyDynamicRecord) => Seq[CompanyDynamicRecord] = null


  def group_by_key: (CompanyDynamicRecord) => String = null

  def group_by_flat_map: (Seq[CompanyDynamicRecord]) => Seq[CompanyDynamicRecord] = null


  protected def getCompanyDynamicRecord(change_extract: ChangeExtract
                                        , dynamic_code: String
                                        , dynamic_info: Map[String, Any]
                                        , update_field: String
                                        , association_entity_info: Seq[AssociationEntityInfo]
                                        , create_time: String = null
                                       ): CompanyDynamicRecord = {
    val tn = change_extract.tn
    val update_type = change_extract.update_type
    val old_data = change_extract.old_data
    val new_data = change_extract.new_data
    val biz_date = change_extract.biz_date
    val rowkey = change_extract.rowkey
    val update_time = change_extract.update_time
    val date = if (is_inc) update_time else biz_date


    CompanyDynamicRecord(
      id = CompanyDynamicUtils.generateId(rowkey, date, tn, update_field)
      , association_entity_info = association_entity_info
      , rowkey = rowkey
      , tn = tn
      , update_type = update_type
      , dynamic_code = dynamic_code
      , dynamic_info = dynamic_info
      , agg_detail_text = null
      , agg_detail_rowkey = null
      , biz_time = biz_date
      , dynamic_time = date
      , update_time = update_time
      , create_time = create_time
    )
  }

  protected def getEntity(json: String, id_key: String, name_key: String
                          , risk_level: NgCompanyRiskLevelType //变更风险等级
                          , rta_info: String //描述
                         ): Seq[AssociationEntityInfo] = {
    if (StringUtils.isEmpty(json)) {
      Seq.empty
    } else {
      val array = JSON.parseArray(json)
      var list: mutable.Seq[AssociationEntityInfo] = mutable.Seq.empty
      for (i <- 0 until array.size()) {
        val jSONObject = array.getJSONObject(i)
        val keyno = jSONObject.getString(id_key)
        val name = jSONObject.getString(name_key)
        list = list :+ AssociationEntityInfo(keyno = keyno, name = name, risk_level = risk_level, rta_info = rta_info)
      }
      list
    }
  }

  protected def changeEquals(v1: String, v2: String): Boolean = {
    if (v1 == null || v2 == null) {
      true
    } else {
      BaseUtil.toDBC(v1).equals(BaseUtil.toDBC(v2))
    }

  }


  def formatDate(date: String): String = {
    if (StringUtils.isEmpty(date)) {
      null
    } else {
      if (date.contains(" ")) date.split(" ")(0) else date
    }
  }

}
