package com.winhc.bigdata.spark.ng.dynamic.agg

import com.winhc.bigdata.spark.implicits.BaseHelper._
import com.winhc.bigdata.spark.implicits.CaseClass2JsonHelper._
import com.winhc.bigdata.spark.ng.dynamic.utils.{BusinessInfoDynamicInfoMap, ChangeContent}
import com.winhc.bigdata.spark.ng.dynamic.{AcrossTabAggHandle, CompanyDynamicRecord}
import org.apache.commons.lang3.StringUtils

/**
 * @author: XuJiakai
 * @date: 2021/7/28 17:36
 */
case class BusinessInfo() extends AcrossTabAggHandle {

  private val dynamic_code_map = Map(
    "109001,109002,109005" -> "109001"
    , "101001" -> "101001"
    , "101002" -> "101002"
    , "101003" -> "101003"
    , "101004" -> "101004"
    , "101005" -> "101005"
    , "101006" -> "101006"
    , "101301" -> "101301"
  ).flatMap(r => {
    r._1.split(",").map((_, r._2))
  })


  /**
   * 需要聚合的维度
   *
   * @return
   */
  override def getTables: Seq[String] = Seq("company", "company_staff")

  /**
   * 聚合的key，相同的key会聚合到一起
   *
   * @return
   */
  override def group_by_key: CompanyDynamicRecord => String = (cdr: CompanyDynamicRecord) => {
    val info = cdr.association_entity_info.filter(r => StringUtils.isNotEmpty(r.keyno)).find(r => r.keyno.length == 32 || r.keyno.length == 33).orNull
    if (info == null) {
      s"${cdr.id}_${cdr.dynamic_info("base").asInstanceOf[BusinessInfoDynamicInfoMap].label}"
    } else {
      s"${info.keyno}_${cdr.dynamic_info("base").asInstanceOf[BusinessInfoDynamicInfoMap].label}"
    }
  }

  /**
   * 聚合处理程序  flat_map
   *
   * @return
   */
  override def group_by_flat_map: Seq[CompanyDynamicRecord] => Seq[CompanyDynamicRecord] = (seq: Seq[CompanyDynamicRecord]) => {
    val contents: Seq[ChangeContent] = seq.flatMap(r => {
      r.dynamic_info("base").asInstanceOf[BusinessInfoDynamicInfoMap].content
    })
    val label = seq.map(r => r.dynamic_info("base").asInstanceOf[BusinessInfoDynamicInfoMap].label).distinct.mkString("，")
    val str = BusinessInfoDynamicInfoMap(label = label, content = contents).toJson()
    val id = seq.map(_.id).sorted.mkString(",")
    Seq(seq(0).customCopy(id = id, dynamic_code = get_new_code(seq), dynamic_info = str.toAnyMap()))
  }


  private def get_new_code(seq: Seq[CompanyDynamicRecord]): String = {
    val code = seq.map(_.dynamic_code).distinct.map(dynamic_code_map(_)).distinct
    if (code.length != 1) {
      val str = seq.map(_.dynamic_code).distinct.sorted.mkString(",")
      import com.winhc.bigdata.spark.implicits.CaseClass2JsonHelper._
      throw new RuntimeException("dynamic code error : " + str + "  record:\n " + seq.toJson())
    }
    code.head
  }
}
