package com.winhc.bigdata.spark.ng.dynamic.agg

import com.winhc.bigdata.spark.ng.dynamic.{AcrossTabAggHandle, AssociationEntityInfo, CompanyDynamicRecord}

/**
 * @author: XuJiakai
 * @date: 2021/7/20 10:42
 */
case class company_mortgage() extends AcrossTabAggHandle {
  /**
   * 需要聚合的维度
   *
   * @return
   */
  override def getTables: Seq[String] = Seq("company_mortgage_people", "company_mortgage_info")

  /**
   * 聚合的key，相同的key会聚合到一起
   *
   * @return
   */
  override def group_by_key: CompanyDynamicRecord => String = (cdr: CompanyDynamicRecord) => {
    cdr.tn match {
      case "company_mortgage_people" => cdr.dynamic_info("main_id").asInstanceOf[String]
      case "company_mortgage_info" => cdr.rowkey
      case _ => throw new RuntimeException("123")
    }
  }


  /**
   * 聚合处理程序  flat_map
   *
   * @return
   */
  override def group_by_flat_map: Seq[CompanyDynamicRecord] => Seq[CompanyDynamicRecord] = (seq: Seq[CompanyDynamicRecord]) => {
    val main_cdr: CompanyDynamicRecord = seq.find(p => p.tn.equals("company_mortgage_info")).orNull
    if (main_cdr == null) {
      Seq.empty[CompanyDynamicRecord]
    } else {
      val entityInfo: Seq[AssociationEntityInfo] = seq.filter(r => r.tn.equals("company_mortgage_people")).flatMap(r => r.association_entity_info)

      val association_entity_info: Seq[AssociationEntityInfo] = main_cdr.association_entity_info ++ entityInfo

      val map: Map[String, Any] = main_cdr.dynamic_info ++ Map("pledgee" -> entityInfo)

      val record: CompanyDynamicRecord = CompanyDynamicRecord(id = main_cdr.id
        , association_entity_info = association_entity_info
        , rowkey = main_cdr.rowkey
        , tn = main_cdr.tn
        , update_type = main_cdr.update_type
        , dynamic_code = main_cdr.dynamic_code
        , dynamic_info = map
        , agg_detail_text = main_cdr.agg_detail_text
        , agg_detail_rowkey = main_cdr.agg_detail_rowkey
        , biz_time = main_cdr.biz_time
        , dynamic_time = main_cdr.dynamic_time
        , update_time = main_cdr.update_time
        , create_time = main_cdr.create_time
      )
      Seq(record)
    }
  }
}
