package com.winhc.bigdata.spark.ng.dynamic.handle

import com.winhc.bigdata.spark.implicits.BaseHelper._
import com.winhc.bigdata.spark.implicits.Bool._
import com.winhc.bigdata.spark.implicits.CaseClass2JsonHelper._
import com.winhc.bigdata.spark.ng.dynamic.NgCompanyRiskLevelType.NgCompanyRiskLevelType
import com.winhc.bigdata.spark.ng.dynamic._
import com.winhc.bigdata.spark.ng.dynamic.utils._
import com.winhc.bigdata.spark.utils.RegCapitalAmount
import org.apache.commons.lang3.StringUtils
import org.json4s.JsonAST.{JField, JObject, JString}
import org.json4s.jackson.Serialization
import org.json4s.jackson.Serialization.read
import org.json4s.{Formats, JArray, NoTypeHints}

import scala.collection.mutable

/**
 * @author: XuJiakai
 * @date: 2021/6/23 10:43
 */
case class company_holder(is_inc: Boolean) extends NgCompanyDynamicHandle {


  private val dynamic_code_map = Map(
    "109003,109004" -> "109003"
    , "101101,101102" -> "101101"
  ).flatMap(r => {
    r._1.split(",").map((_, r._2))
  })


  override def flat_map: (ChangeExtract) => Seq[CompanyDynamicRecord] = (change_extract: ChangeExtract) => {
    val change_fields = change_extract.change_fields
    val tn = change_extract.tn
    val company_id = change_extract.company_id
    var company_name = change_extract.company_name
    val update_type = change_extract.update_type
    val old_data = change_extract.old_data
    val new_data = change_extract.new_data
    val biz_date = change_extract.biz_date
    val rowkey = change_extract.rowkey
    val update_time = change_extract.update_time

    if (StringUtils.isEmpty(company_name)) company_name = new_data("company_name")

    def getRecordPro(dynamicCode: String
                     , name: String
                     , keyno: String
                     , label: String
                     , risk_level: NgCompanyRiskLevelType
                     , content: Seq[ChangeContent]
                    ): CompanyDynamicRecord = {
      val map = BusinessInfoDynamicInfoMap(label = label, content = content)

      val date = if (is_inc) update_time else biz_date
      CompanyDynamicRecord(
        id = CompanyDynamicUtils.generateId(rowkey, date, tn, dynamicCode)
        , association_entity_info = Seq(AssociationEntityInfo(keyno = keyno, name = name, risk_level = risk_level, rta_info = null))
        , rowkey = rowkey
        , tn = tn
        , update_type = update_type
        , dynamic_code = dynamicCode
        , dynamic_info = map.toMap()
        , agg_detail_text = null
        , agg_detail_rowkey = null
        , biz_time = formatDate(biz_date)
        , dynamic_time = formatDate(date)
        , update_time = update_time
        , create_time = update_time
      )
    }


    var list: mutable.Seq[CompanyDynamicRecord] = mutable.Seq.empty


    //股东出资变化
    def shareholderCapitalChange(name: String, keyno: String, before: String, after: String): Seq[CompanyDynamicRecord] = {
      val bool = RegCapitalAmount.isAdditionalShare(before, after)
      if (bool == 0)
        return Seq.empty

      val before_d = RegCapitalAmount.getAmount(before).toDouble
      val after_d = RegCapitalAmount.getAmount(after).toDouble

      if (0 == before_d || after_d == 0)
        return Seq.empty

      if (before_d / after_d == 10000 || after_d / before_d == 10000)
        return Seq.empty

      val flag = (bool > 0) ? "1" | "0"
      val r = (bool > 0) ? NgCompanyRiskLevelType.Positive | NgCompanyRiskLevelType.Caution
      val cdr = getRecordPro(
        dynamicCode = "101101"
        , keyno = company_id
        , name = company_name
        , label = "股东变化"
        , risk_level = r
        , content = Seq(ChangeContent("101101"
          , before = Entity(name, keyno, info = before)
          , after = Entity(name, keyno, info = after)
          , Entity(name = company_name, keyno = company_id)
          , flag = flag
        ))
      )

      val cdr2 = getRecordPro(
        dynamicCode = "109003"
        , keyno = keyno
        , name = name
        , label = "对外投资"
        , risk_level = r
        , content = Seq(ChangeContent("109003"
          , before = Entity(name = company_name, keyno = company_id, info = before)
          , after = Entity(name = company_name, keyno = company_id, info = after)
          , Entity(name = name, keyno = keyno)
          , flag = flag
        ))
      )

      Seq(cdr2, cdr)
    }

    //股东人员变化
    def shareholder(beforeName: String, beforeKeyno: String, beforePercent: String, afterName: String, afterKeyno: String, afterPercent: String): Seq[CompanyDynamicRecord] = {
      if (beforeName == null) {
        val cdr = getRecordPro(
          dynamicCode = "101102"
          , keyno = company_id
          , name = company_name
          , label = "股东变化"
          , risk_level = NgCompanyRiskLevelType.Prompt
          , content = Seq(ChangeContent("101102", before = null
            , after = Entity(afterName, afterKeyno, afterPercent)
            , entity = Entity(name = company_name, keyno = company_id)))
        )

        val cdr2 = getRecordPro(
          dynamicCode = "109004"
          , keyno = afterKeyno
          , name = afterName
          , label = "对外投资"
          , risk_level = NgCompanyRiskLevelType.Prompt
          , content = Seq(ChangeContent("109004", before = null
            , after = Entity(name = company_name, keyno = company_id, afterPercent)
            , entity = Entity(afterName, afterKeyno)))
        )
        Seq(cdr, cdr2)
      } else {
        val cdr = getRecordPro(
          dynamicCode = "101102"
          , keyno = company_id
          , name = company_name
          , label = "股东变化"
          , risk_level = NgCompanyRiskLevelType.Prompt
          , content = Seq(ChangeContent("101102"
            , before = Entity(beforeName, beforeKeyno, beforePercent)
            , after = null
            , entity = Entity(name = company_name, keyno = company_id)))
        )

        val cdr2 = getRecordPro(
          dynamicCode = "109004"
          , keyno = beforeKeyno
          , name = beforeName
          , label = "对外投资"
          , risk_level = NgCompanyRiskLevelType.Prompt
          , content = Seq(ChangeContent("109004", before = Entity(name = company_name, keyno = company_id, beforePercent)
            , after = null
            , entity = Entity(beforeName, beforeKeyno)))
        )
        Seq(cdr, cdr2)
      }

    }

    //股东新增对外投资
    def holderNew(name: String, keyno: String, companyName: String, companyId: String, percent: String): Seq[CompanyDynamicRecord] = {
      val cdr = getRecordPro(
        dynamicCode = "109004"
        , keyno = keyno
        , name = name
        , label = "对外投资"
        , risk_level = NgCompanyRiskLevelType.Prompt
        , content = Seq(ChangeContent("109004", before = null
          , after = Entity(companyName, companyId, percent)
          , entity = Entity(name = name, keyno = keyno)))
      )
      Seq(cdr)
    }


    update_type match {
      case "insert" => {
        //新增股东
        val n_company_id = new_data("company_id")
        val n_company_name = new_data("company_name")
        val n_holder_name = new_data("holder_name")
        val n_holder_id = new_data("holder_id")
        val n_percent = new_data("percent")

        list = list ++ shareholder(null, null, null, n_holder_name, n_holder_id, n_percent)
      }
      case "create" => {
        //公司新成立
        val n_holder_name = new_data("holder_name")
        val n_holder_id = new_data("holder_id")
        val n_percent = new_data("percent")
        list = list ++ holderNew(n_holder_name, n_holder_id, company_name, company_id, n_percent)
      }

      case "update" => {
        //股东出资额发变化
        if (change_fields.contains("amount")) {
          val o_capital = getRealCapital(old_data("capital"))
          val o_percent = getRealCapital(old_data("percent"))
          val n_capital = getRealCapital(new_data("capital"))
          val n_percent = getRealCapital(new_data("percent"))

          val old_amount = (o_capital == null) ? old_data("amount") | o_capital
          val new_amount = (n_capital == null) ? new_data("amount") | n_capital

          val n_company_id = new_data("company_id")
          val n_company_name = new_data("company_name")
          val n_holder_name = new_data("holder_name")
          val n_holder_id = new_data("holder_id")

          list = list ++ shareholderCapitalChange(n_holder_name, n_holder_id, old_amount, new_amount)
        }
      }

      case "deleted" => {
        //新增股东
        val n_company_id = new_data("company_id")
        val n_company_name = new_data("company_name")
        val n_holder_name = new_data("holder_name")
        val n_holder_id = new_data("holder_id")
        val n_percent = new_data("percent")
        list = list ++ shareholder(n_holder_name, n_holder_id, n_percent, null, null, null)
      }
    }
    list
  }

  /**
   * group_by前置处理程。flat_map
   *
   * @return
   */
  override def group_by_pre: CompanyDynamicRecord => Seq[CompanyDynamicRecord] = (cdr: CompanyDynamicRecord) => {
    val _id = cdr.id
    val _association_entity_info = cdr.association_entity_info
    val _rowkey = cdr.rowkey
    val _tn = cdr.tn
    val _update_type = cdr.update_type
    val _dynamic_code = cdr.dynamic_code
    val _dynamic_info = cdr.dynamic_info
    val _agg_detail_text = cdr.agg_detail_text
    val _agg_detail_rowkey = cdr.agg_detail_rowkey
    val _biz_time = cdr.biz_time
    val _dynamic_time = cdr.dynamic_time
    val _update_time = cdr.update_time
    val _create_time = cdr.create_time
    _association_entity_info.filter(r => StringUtils.isNotEmpty(r.keyno)).map(r => {
      CompanyDynamicRecord(id = _id, association_entity_info = Seq(r), rowkey = _rowkey, tn = _tn, update_type = _update_type, dynamic_code = _dynamic_code, dynamic_info = _dynamic_info, agg_detail_text = _agg_detail_text, agg_detail_rowkey = _agg_detail_rowkey, biz_time = _biz_time, dynamic_time = _dynamic_time, update_time = _update_time, create_time = _create_time)
    })
  }

  override def group_by_key: CompanyDynamicRecord => String = (cdr: CompanyDynamicRecord) => {
    val info = cdr.association_entity_info.filter(r => StringUtils.isNotEmpty(r.keyno)).find(r => r.keyno.length == 32 || r.keyno.length == 33).orNull

    if (info == null) {
      s"${cdr.id}_${cdr.dynamic_info("base").asInstanceOf[BusinessInfoDynamicInfoMap].label}"
    } else {
      s"${info.keyno}_${cdr.dynamic_info("base").asInstanceOf[BusinessInfoDynamicInfoMap].label}"
    }
  }


  override def group_by_flat_map: Seq[CompanyDynamicRecord] => Seq[CompanyDynamicRecord] = (seq: Seq[CompanyDynamicRecord]) => {
    val contents: Seq[ChangeContent] = seq.flatMap(r => {
      r.dynamic_info("base").asInstanceOf[BusinessInfoDynamicInfoMap].content
    })
    val label = seq.map(r => r.dynamic_info("base").asInstanceOf[BusinessInfoDynamicInfoMap].label).distinct.mkString("，")
    val str = BusinessInfoDynamicInfoMap(label = label, content = contents).toJson()
    val id = seq.map(_.id).sorted.mkString(",")
    Seq(seq(0).customCopy(id = id, dynamic_code = get_new_code(seq), dynamic_info = str.toAnyMap()))
  }


  private def get_new_code(seq: Seq[CompanyDynamicRecord]): String = {
    val code = seq.map(_.dynamic_code).distinct.map(dynamic_code_map(_)).distinct
    if (code.length != 1) {
      val str = seq.map(_.dynamic_code).distinct.sorted.mkString(",")

      import com.winhc.bigdata.spark.implicits.CaseClass2JsonHelper._
      throw new RuntimeException("dynamic code error : " + str + "  record:\n " + seq.toJson())
    }
    code.head
  }


  def getRealCapital(str: String): String = {
    if (StringUtils.isEmpty(str))
      return null
    try {
      implicit val formats: AnyRef with Formats = Serialization.formats(NoTypeHints)
      val json = read[JArray](str)

      val as = for {
        JObject(child) <- json
        JField("amomon", JString(amount)) <- child
      } yield amount
      if (as.isEmpty)
        return null
      val head = as.head
      val v = RegCapitalAmount.getAmount(head)
      if (v == null) {
        return null
      }
      head
    } catch {
      case ex: Exception => {}
        null
    }
  }
}
