package com.winhc.bigdata.spark.ng.dynamic.handle

import com.winhc.bigdata.spark.ng.dynamic.NgCompanyRiskLevelType.NgCompanyRiskLevelType
import com.winhc.bigdata.spark.ng.dynamic._
import com.winhc.bigdata.spark.ng.dynamic.utils.{BusinessInfoDynamicInfoMap, ChangeContent, CompanyDynamicUtils, Entity}
import org.apache.commons.lang3.StringUtils

import scala.collection.mutable

/**
 * @author: XuJiakai
 * @date: 2021/7/26 16:15
 */
case class company_staff(is_inc: Boolean) extends NgCompanyDynamicHandle {

  private val dynamic_code_map = Map(
    "109005" -> "109005"
    , "101301" -> "101301"
  ).flatMap(r => {
    r._1.split(",").map((_, r._2))
  })


  private val pattern = "[^\\u4e00-\\u9fa50-9a-zA-Z ]".r

  override def flat_map: ChangeExtract => Seq[CompanyDynamicRecord] = (change_extract: ChangeExtract) => {
    val change_fields = change_extract.change_fields
    val tn = change_extract.tn
    val company_id = change_extract.company_id
    var company_name = change_extract.company_name
    val update_type = change_extract.update_type
    val old_data = change_extract.old_data
    val new_data = change_extract.new_data
    val biz_date = change_extract.biz_date
    val rowkey = change_extract.rowkey
    val update_time = change_extract.update_time

    if (StringUtils.isEmpty(company_name)) company_name = new_data("company_name")

    def getRecordPro(dynamicCode: String
                     , name: String
                     , keyno: String
                     , label: String
                     , risk_level: NgCompanyRiskLevelType
                     , content: Seq[ChangeContent]
                    ): CompanyDynamicRecord = {
      val map = BusinessInfoDynamicInfoMap(label = label, content = content)

      val date = if (is_inc) update_time else biz_date
      CompanyDynamicRecord(
        id = CompanyDynamicUtils.generateId(rowkey, date, tn, dynamicCode)
        , association_entity_info = Seq(AssociationEntityInfo(keyno = keyno, name = name, risk_level = risk_level, rta_info = null))
        , rowkey = rowkey
        , tn = tn
        , update_type = update_type
        , dynamic_code = dynamicCode
        , dynamic_info = map.toMap()
        , agg_detail_text = null
        , agg_detail_rowkey = null
        , biz_time = formatDate(biz_date)
        , dynamic_time = formatDate(date)
        , update_time = update_time
        , create_time = update_time
      )
    }


    var list: mutable.Seq[CompanyDynamicRecord] = mutable.Seq.empty

    //新增主要成员
    def staffNew(staffName: String, staffId: String, staffType: String): Seq[CompanyDynamicRecord] = {
      val cdr = getRecordPro(
        dynamicCode = "101301"
        , keyno = company_id
        , name = company_name
        , label = "主要成员"
        , risk_level = NgCompanyRiskLevelType.Prompt
        , content = Seq(ChangeContent("109005", before = null
          , after = Entity(name = staffName, keyno = staffId, info = staffType)
          , entity = Entity(company_name, company_id)))
      )

      val cdr2 = getRecordPro(
        dynamicCode = "109005"
        , keyno = staffId
        , name = staffName
        , label = "任职变化"
        , risk_level = NgCompanyRiskLevelType.Prompt
        , content = Seq(ChangeContent("109005", before = null
          , after = Entity(name = company_name, keyno = company_id, info = staffType)
          , entity = Entity(staffName, staffId)))
      )
      Seq(cdr, cdr2)
    }

    //公司新成立
    def staffCreate(staffName: String, staffId: String, staffType: String): Seq[CompanyDynamicRecord] = {
      val cdr = getRecordPro(
        dynamicCode = "109005"
        , keyno = staffId
        , name = staffName
        , label = "任职变化"
        , risk_level = NgCompanyRiskLevelType.Prompt
        , content = Seq(ChangeContent("109005", before = null
          , after = Entity(name = company_name, keyno = company_id, info = staffType)
          , entity = Entity(staffName, staffId)))
      )
      Seq(cdr)
    }

    //职位变化
    def staffTypeChange(staffName: String, staffId: String, beforeType: String, afterType: String): Seq[CompanyDynamicRecord] = {
      if (beforeType == null || afterType == null)
        return Seq.empty

      val v1 = pattern.split(beforeType).distinct.sorted.mkString(",")
      val v2 = pattern.split(afterType).distinct.sorted.mkString(",")
      if (v1.equals(v2))
        return Seq.empty

      val cdr = getRecordPro(
        dynamicCode = "101301"
        , keyno = company_id
        , name = company_name
        , label = "主要成员"
        , risk_level = NgCompanyRiskLevelType.Prompt
        , content = Seq(ChangeContent("101301"
          , before = Entity(staffName, staffId, info = beforeType)
          , after = Entity(staffName, staffId, info = afterType)
          , entity = Entity(name = company_name, keyno = company_id)
        ))
      )

      val cdr2 = getRecordPro(
        dynamicCode = "109005"
        , keyno = staffId
        , name = staffName
        , label = "任职变化"
        , risk_level = NgCompanyRiskLevelType.Prompt
        , content = Seq(ChangeContent("109005"
          , before = Entity(name = company_name, keyno = company_id, info = beforeType)
          , after = Entity(name = company_name, keyno = company_id, info = afterType)
          , entity = Entity(name = staffName, keyno = staffId)
        ))
      )
      Seq(cdr, cdr2)
    }


    //主要成员移除
    def staffDeleted(staffName: String, staffId: String, staffType: String): Seq[CompanyDynamicRecord] = {
      val cdr = getRecordPro(
        dynamicCode = "101301"
        , keyno = company_id
        , name = company_name
        , label = "主要成员"
        , risk_level = NgCompanyRiskLevelType.Prompt
        , content = Seq(ChangeContent("109005", before = Entity(name = staffName, keyno = staffId, info = staffType)
          , after = null
          , entity = Entity(company_name, company_id)))
      )

      val cdr2 = getRecordPro(
        dynamicCode = "109005"
        , keyno = staffId
        , name = staffName
        , label = "任职变化"
        , risk_level = NgCompanyRiskLevelType.Prompt
        , content = Seq(ChangeContent("109005", before = Entity(name = company_name, keyno = company_id, info = staffType)
          , after = null
          , entity = Entity(staffName, staffId)))
      )
      Seq(cdr, cdr2)
    }


    update_type match {
      case "insert" => {
        //新增主要成员
        val n_staff_name = new_data("staff_name")
        val n_hid = new_data("hid")
        val n_staff_type = new_data("staff_type")

        list = list ++ staffNew(n_staff_name, n_hid, n_staff_type)
      }
      case "create" => {
        //公司新成立
        val n_staff_name = new_data("staff_name")
        val n_hid = new_data("hid")
        val n_staff_type = new_data("staff_type")

        list = list ++ staffCreate(n_staff_name, n_hid, n_staff_type)
      }

      case "update" => {
        //主要成员职位变化
        if (change_fields.contains("staff_type")) {
          val n_staff_name = new_data("staff_name")
          val n_hid = new_data("hid")
          val n_staff_type = new_data("staff_type")
          val o_staff_type = old_data("staff_type")

          list = list ++ staffTypeChange(n_staff_name, n_hid, o_staff_type, n_staff_type)
        }
      }

      case "deleted" => {
        //移除主要成员
        val n_staff_name = new_data("staff_name")
        val n_hid = new_data("hid")
        val n_staff_type = new_data("staff_type")

        list = list ++ staffDeleted(n_staff_name, n_hid, n_staff_type)
      }
    }
    list
  }


  /**
   * group_by前置处理程。flat_map
   *
   * @return
   */
  override def group_by_pre: CompanyDynamicRecord => Seq[CompanyDynamicRecord] = (cdr: CompanyDynamicRecord) => {
    val _id = cdr.id
    val _association_entity_info = cdr.association_entity_info
    val _rowkey = cdr.rowkey
    val _tn = cdr.tn
    val _update_type = cdr.update_type
    val _dynamic_code = cdr.dynamic_code
    val _dynamic_info = cdr.dynamic_info
    val _agg_detail_text = cdr.agg_detail_text
    val _agg_detail_rowkey = cdr.agg_detail_rowkey
    val _biz_time = cdr.biz_time
    val _dynamic_time = cdr.dynamic_time
    val _update_time = cdr.update_time
    val _create_time = cdr.create_time
    _association_entity_info.filter(r => StringUtils.isNotEmpty(r.keyno)).map(r => {
      CompanyDynamicRecord(id = _id, association_entity_info = Seq(r), rowkey = _rowkey, tn = _tn, update_type = _update_type, dynamic_code = _dynamic_code, dynamic_info = _dynamic_info, agg_detail_text = _agg_detail_text, agg_detail_rowkey = _agg_detail_rowkey, biz_time = _biz_time, dynamic_time = _dynamic_time, update_time = _update_time, create_time = _create_time)
    })
  }

  override def group_by_key: CompanyDynamicRecord => String = (cdr: CompanyDynamicRecord) => {
    val info = cdr.association_entity_info.filter(r => StringUtils.isNotEmpty(r.keyno)).find(r => r.keyno.length == 32 || r.keyno.length == 33).orNull
    if (info == null) {
      cdr.id
    } else {
      info.keyno
    }
  }


  override def group_by_flat_map: Seq[CompanyDynamicRecord] => Seq[CompanyDynamicRecord] = (seq: Seq[CompanyDynamicRecord]) => {
    val contents: Seq[ChangeContent] = seq.flatMap(r => {
      r.dynamic_info("base").asInstanceOf[BusinessInfoDynamicInfoMap].content
    })
    val label = seq.map(r => r.dynamic_info("base").asInstanceOf[BusinessInfoDynamicInfoMap].label).distinct.mkString("，")
    val str = BusinessInfoDynamicInfoMap(label = label, content = contents)
    val id = seq.map(_.id).sorted.mkString(",")
    Seq(seq(0).customCopy(id = id,dynamic_code = get_new_code(seq), dynamic_info = str.toMap()))
  }


  private def get_new_code(seq: Seq[CompanyDynamicRecord]): String = {
    val code = seq.map(_.dynamic_code).distinct.map(dynamic_code_map(_)).distinct
    if (code.length != 1) {
      val str = seq.map(_.dynamic_code).distinct.sorted.mkString(",")
      import com.winhc.bigdata.spark.implicits.CaseClass2JsonHelper._
      throw new RuntimeException("dynamic code error : " + str + "  record:\n " + seq.toJson())
    }
    code.head
  }
}

