package com.winhc.bigdata.spark.ng.dynamic.utils

import com.winhc.bigdata.spark.implicits.BaseHelper._
import com.winhc.bigdata.spark.implicits.Bool._
import com.winhc.bigdata.spark.ng.dynamic.NgCompanyRiskLevelType.NgCompanyRiskLevelType
import com.winhc.bigdata.spark.ng.dynamic.{AssociationEntityInfo, CompanyDynamicRecord, NgCompanyRiskLevelType}
import com.winhc.bigdata.spark.utils.{BaseUtil, RegCapitalAmount}
import org.apache.commons.lang3.StringUtils
import com.winhc.bigdata.spark.implicits.CaseClass2JsonHelper._
/**
 * @author: XuJiakai
 * @date: 2021/8/5 15:55
 */
object BusinessTotalDynamicUtils {

  private def getRecordPro(dynamicCode: String
                           , rowkey: String
                           , name: String
                           , keyno: String
                           , label: String
                           , risk_level: NgCompanyRiskLevelType
                           , before: String
                           , after: String
                           , entity: List[Map[String, String]]
                           , change_time: String
                          ): CompanyDynamicRecord = {
    val tn = "company_change"
    val update_time = BaseUtil.nowDate()


    val map = Map[String, Any](
      "code" -> dynamicCode
      , "label" -> label
      , "before" -> before
      , "after" -> after
      , "entity" -> entity
    )

    val date = if (change_time.contains(" ")) change_time.split(" ")(0) else change_time
    CompanyDynamicRecord(
      id = CompanyDynamicUtils.generateId(rowkey, date, tn, dynamicCode)
      , association_entity_info = Seq(AssociationEntityInfo(keyno = keyno, name = name, risk_level = risk_level, rta_info = null))
      , rowkey = rowkey
      , tn = tn
      , update_type = "update"
      , dynamic_code = dynamicCode
      , dynamic_info = map
      , agg_detail_text = null
      , agg_detail_rowkey = null
      , biz_time = date
      , dynamic_time = date
      , update_time = update_time
      , create_time = update_time
    )
  }


  private def getRecordPro(dynamicCode: String
                           , rowkey: String
                           , name: String
                           , keyno: String
                           , label: String
                           , risk_level: NgCompanyRiskLevelType
                           , content: Seq[ChangeContent]
                           , change_time: String
                          ): CompanyDynamicRecord = {
    val map = BusinessInfoDynamicInfoMap(label = label, content = content)
    val tn = "company_change"
    val update_time = BaseUtil.nowDate()

    val date = if (change_time.contains(" ")) change_time.split(" ")(0) else change_time
    CompanyDynamicRecord(
      id = CompanyDynamicUtils.generateId(rowkey, date, tn, dynamicCode)
      , association_entity_info = Seq(AssociationEntityInfo(keyno = keyno, name = name, risk_level = risk_level, rta_info = null))
      , rowkey = rowkey
      , tn = tn
      , update_type = "update"
      , dynamic_code = dynamicCode
      , dynamic_info = map.toJson().toAnyMap()
      , agg_detail_text = null
      , agg_detail_rowkey = null
      , biz_time = date
      , dynamic_time = date
      , update_time = update_time
      , create_time = update_time
    )
  }

  def getSingleRecordChange(code: String
                            , rowkey: String
                            , label: String
                            , keyno: String
                            , name: String
                            , before: String
                            , after: String
                            , date: String
                            , riskLevel: NgCompanyRiskLevelType = NgCompanyRiskLevelType.Prompt, flag: String = null): CompanyDynamicRecord = {
    getRecordPro(
      dynamicCode = code
      , rowkey = rowkey
      , keyno = keyno
      , name = name
      , label = label
      , risk_level = riskLevel
      , content = Seq(ChangeContent(code, before = Entity(info = before), after = Entity(info = after), Entity(name = name, keyno = keyno), flag = flag))
      , change_time = date
    )
  }


  def companyNameFilter(category: String,
                        change_item: String,
                        change_info: String,
                        content_before: String,
                        content_after: String): Boolean = {

    if (StringUtils.isEmpty(category)) {
      false
    } else {
      category.equals("公司名称变更")
    }
  }


  def companyNameTransform(company_id: String,
                           company_name: String,
                           category: String,
                           change_item: String,
                           change_info: String,
                           content_before: String,
                           content_after: String,
                           change_time: String
                          ): Seq[CompanyDynamicRecord] = {
    val record = getSingleRecordChange(
      code = "101001"
      , rowkey = company_id
      , label = "公司更名"
      , keyno = company_id
      , name = company_name
      , before = content_before
      , after = content_after
      , date = change_time
    )
    Seq(record)
  }


  def registeredCapitalFilter(category: String,
                              change_item: String,
                              change_info: String,
                              content_before: String,
                              content_after: String): Boolean = {

    if (StringUtils.isEmpty(category) || !category.equals("注册资本变更")) {
      return false
    }
    if (change_item.contains("实")) {
      return false
    }
    true
  }


  def registeredCapitalTransform(company_id: String,
                                 company_name: String,
                                 category: String,
                                 change_item: String,
                                 change_info: String,
                                 content_before: String,
                                 content_after: String,
                                 change_time: String
                                ): Seq[CompanyDynamicRecord] = {
    val bool = RegCapitalAmount.isAdditionalShare(content_before, content_after)
    if (bool == 0)
      return Seq.empty
    if (0 == RegCapitalAmount.getAmount(content_before).toDouble || RegCapitalAmount.getAmount(content_after).toDouble == 0)
      return Seq.empty
    val flag = (bool > 0) ? "1" | "0"
    val r = (bool > 0) ? NgCompanyRiskLevelType.Positive | NgCompanyRiskLevelType.Caution

    val record = getSingleRecordChange("101002"
      , company_id
      , "注册资本"
      , company_id
      , company_name
      , content_before
      , content_after
      , riskLevel = r
      , date = change_time
      , flag = flag)
    Seq(record)
  }


  def registeredAddressFilter(category: String,
                              change_item: String,
                              change_info: String,
                              content_before: String,
                              content_after: String): Boolean = {

    if (StringUtils.isEmpty(category) || !category.equals("住所变更")) {
      return false
    }
    true
  }


  def registeredAddressTransform(company_id: String,
                                 company_name: String,
                                 category: String,
                                 change_item: String,
                                 change_info: String,
                                 content_before: String,
                                 content_after: String,
                                 change_time: String
                                ): Seq[CompanyDynamicRecord] = {
    val record = getSingleRecordChange(
      "101004"
      , company_id
      , "注册地址变化"
      , company_id
      , company_name
      , content_before
      , content_after
      , change_time
    )
    Seq(record)
  }


  def businessScopeFilter(category: String,
                          change_item: String,
                          change_info: String,
                          content_before: String,
                          content_after: String): Boolean = {

    if (StringUtils.isEmpty(category) || !category.equals("经营范围变更")) {
      return false
    }
    true
  }


  def businessScopeTransform(company_id: String,
                             company_name: String,
                             category: String,
                             change_item: String,
                             change_info: String,
                             content_before: String,
                             content_after: String,
                             change_time: String
                            ): Seq[CompanyDynamicRecord] = {
    if (StringUtils.isEmpty(content_before) || StringUtils.isEmpty(content_after))
      return Seq.empty
    val record = getSingleRecordChange(
      "101005"
      , company_id
      , "经营范围变化"
      , company_id
      , company_name
      , content_before
      , content_after
      , change_time
    )
    Seq(record)
  }


  def legalRepresentativeFilter(category: String,
                                change_item: String,
                                change_info: String,
                                content_before: String,
                                content_after: String): Boolean = {

    if (StringUtils.isEmpty(category) || !category.equals("法定代表人、负责人变更")) {
      return false
    }
    true
  }


  def legalRepresentativeTransform(company_id: String,
                                   company_name: String,
                                   category: String,
                                   change_item: String,
                                   change_info: String,
                                   content_before: String,
                                   content_after: String,
                                   change_time: String
                                  ): Seq[CompanyDynamicRecord] = {
    if (StringUtils.isEmpty(content_before) || StringUtils.isEmpty(content_after))
      return Seq.empty

    var entity: List[Map[String, String]] = List.empty
    if (change_info != null) {
      try {
        val map: Map[String, Any] = change_info.toAnyMap()
        entity = map("entity").asInstanceOf[List[Map[String, String]]]
      } catch {
        case ex: Exception => {}
      }
    }

    val cdr = getRecordPro(
      dynamicCode = "110101"
      , rowkey = company_id
      , keyno = company_id
      , name = company_name
      , label = "法定代表人变更"
      , risk_level = NgCompanyRiskLevelType.Prompt
      , before = content_before
      , after = content_after
      , entity = entity
      , change_time = change_time
    )
    Seq(cdr)


   /* var beforeKeyno: String = null
    var afterKeyno: String = null
    try {
      val map: Map[String, Any] = change_info.toAnyMap()
      val entity = map("entity").asInstanceOf[List[Map[String, String]]]
      for (elem <- entity) {
        try {
          val keyno = elem("keyno")
          val name = elem("name")
          if (content_before.equals(name)) {
            beforeKeyno = keyno
          }
          if (content_after.equals(name)) {
            afterKeyno = keyno
          }
        } catch {
          case e: Exception => {
          }
        }
      }
    } catch {
      case ex: Exception => {
      }
    }


    val cdr = getRecordPro(
      dynamicCode = "110101"
      , rowkey = company_id
      , keyno = company_id
      , name = company_name
      , label = "法定代表人变更"
      , risk_level = NgCompanyRiskLevelType.Prompt
      , content = Seq(ChangeContent("101003", before = Entity(content_before, beforeKeyno)
        , after = Entity(content_after, afterKeyno)
        , Entity(name = company_name, keyno = company_id)))
      , change_time = change_time
    )
    Seq(cdr)*/
  }


  def holderFilter(category: String,
                   change_item: String,
                   change_info: String,
                   content_before: String,
                   content_after: String): Boolean = {

    if (StringUtils.isEmpty(category) || !category.equals("股东、发起人变更")) {
      return false
    }
    true
  }


  def holderTransform(company_id: String,
                      company_name: String,
                      category: String,
                      change_item: String,
                      change_info: String,
                      content_before: String,
                      content_after: String,
                      change_time: String
                     ): Seq[CompanyDynamicRecord] = {
    if (StringUtils.isEmpty(content_before) || StringUtils.isEmpty(content_after))
      return Seq.empty

    var entity: List[Map[String, String]] = List.empty
    if (change_info != null) {
      try {
        val map: Map[String, Any] = change_info.toAnyMap()
        entity = map("entity").asInstanceOf[List[Map[String, String]]]
      } catch {
        case ex: Exception => {}
      }
    }

    val cdr = getRecordPro(
      dynamicCode = "110102"
      , rowkey = company_id
      , keyno = company_id
      , name = company_name
      , label = "股东、发起人变更"
      , risk_level = NgCompanyRiskLevelType.Prompt
      , before = content_before
      , after = content_after
      , entity = entity
      , change_time = change_time
    )
    Seq(cdr)
  }


  def staffFilter(category: String,
                  change_item: String,
                  change_info: String,
                  content_before: String,
                  content_after: String): Boolean = {

    if (StringUtils.isEmpty(category) || !category.equals("高级管理人员备案")) {
      return false
    }
    true
  }


  def staffTransform(company_id: String,
                     company_name: String,
                     category: String,
                     change_item: String,
                     change_info: String,
                     content_before: String,
                     content_after: String,
                     change_time: String
                    ): Seq[CompanyDynamicRecord] = {
    if (StringUtils.isEmpty(content_before) || StringUtils.isEmpty(content_after))
      return Seq.empty
    var entity: List[Map[String, String]] = List.empty
    if (change_info != null) {
      try {
        val map: Map[String, Any] = change_info.toAnyMap()
        entity = map("entity").asInstanceOf[List[Map[String, String]]]
      } catch {
        case ex: Exception => {}
      }
    }
    val cdr = getRecordPro(
      dynamicCode = "110103"
      , rowkey = company_id
      , keyno = company_id
      , name = company_name
      , label = "主要成员变更"
      , risk_level = NgCompanyRiskLevelType.Prompt
      , before = content_before
      , after = content_after
      , entity = entity
      , change_time = change_time
    )
    Seq(cdr)
  }


  def main(args: Array[String]): Unit = {
    val company_id = "170ab62a5934f62c69f68a916a2b92dc"
    val company_name = "科右前旗爱日克奶食品专业合作社"
    val change_info = "{\"entity\":[{\"name\":\"达胡巴雅尔\",\"keyno\":\"5d2f687cd40ed30cd8b9e1cb29f1bfa0\"}]}"
    val content_before = "翡翠"
    val content_after = "达胡巴雅尔"
    val change_time = "2019-07-18 00:00:00"

    val records = legalRepresentativeTransform(company_id, company_name, "", "", change_info, content_before, content_after, change_time)
    println(records(0))

  }


}
