package com.winhc.bigdata.spark.ng.jobs

import com.alibaba.fastjson.JSON
import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.ng.utils.{StartAndEndDsUtils, explode_tab}
import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{DateUtils, LoggingUtils, SparkUtils}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @author: XuJiakai
 * @date: 2020/12/21 10:13
 *        ng空间下企业数据通用处理程序
 */
case class general_handler(s: SparkSession,
                           project: String, //表所在工程名
                           job_args: args_company_job
                          ) extends LoggingUtils with BaseFunc {
  @(transient@getter) val spark: SparkSession = s

  private val md5_fields: Seq[String] = job_args.md5_fields
  private val tn: String = job_args.tableName
  private val is_super_filter: Boolean = job_args.is_super_filter
  private val where: String = job_args.where


  private val ods_tab = s"$project.ods_$tn"
  private val inc_ods_tab = s"$project.inc_ods_$tn"

  private val ads_tab = s"$project.ads_$tn"
  private val inc_ads_tab = s"$project.inc_ads_$tn"

  private val inter_cols = getColumns(ods_tab).intersect(getColumns(inc_ods_tab)).diff(Seq("rowkey"))


  verify()
  reg_udf()

  private def verify(): Unit = {
    val catalog = spark.catalog
    if (!(catalog.tableExists(ods_tab) && catalog.tableExists(inc_ods_tab) && catalog.tableExists(inc_ads_tab) && catalog.tableExists(ads_tab))) {
      println("table is not exists !!!")
      sys.exit(-1)
    }
  }

  private def reg_udf(): Unit = {
    cleanup()
    case_no_trim_udf_v2()
    json_verify()

    def replace_rowkey(use_user_defined_rowkey: String, new_rowkey: String): String = {
      if (StringUtils.isEmpty(use_user_defined_rowkey)) {
        new_rowkey
      } else {
        use_user_defined_rowkey
      }
    }

    def company_holder_rowkey(holder_type: Long, holder_id: String, holder_name: String): String = {
      if (2 == holder_type) {
        holder_id
      } else {
        holder_name
      }
    }

    def get_fixed_val(key_no: String, name: String): String = {
      if (StringUtils.isEmpty(key_no)) {
        name
      } else {
        if (key_no.length == 32) {
          key_no
        } else {
          name
        }
      }
    }

    spark.udf.register("use_user_defined_rowkey", replace_rowkey _)

    spark.udf.register("holder_switch_rowkey", company_holder_rowkey _)

    spark.udf.register("split_date", DateUtils.splitDate _)

    spark.udf.register("get_fixed_val", get_fixed_val _)


    def equity_info_rowkey(pledgee_info: String, pledgor_info: String): String = {
      try {
        def get_seq(key: String, value: String): String = {
          val jSONArray = JSON.parseArray(value)
          if (jSONArray.isEmpty) {
            return ""
          }
          val arr = mutable.ArrayBuffer[String]()
          for (i <- 0 until jSONArray.size()) {
            val jSONObject = jSONArray.getJSONObject(i)
            arr.append(jSONObject.getString(key))
          }
          arr.distinct.sorted.mkString("、")
        }

        val str = get_seq("pledgee", pledgee_info) + " " + get_seq("pledgor", pledgor_info)
        str
      } catch {
        case ex: Exception => {
          ""
        }
      }
    }

    spark.udf.register("equity_info_rowkey", equity_info_rowkey _)

    def get_text_from_json(json: String, name: String): String = {
      try {
        def get_seq(key: String, value: String): String = {
          val jSONArray = JSON.parseArray(value)
          if (jSONArray.isEmpty) {
            return ""
          }
          val arr = mutable.ArrayBuffer[String]()
          for (i <- 0 until jSONArray.size()) {
            val jSONObject = jSONArray.getJSONObject(i)
            arr.append(jSONObject.getString(key))
          }
          arr.mkString("")
        }

        val str = get_seq(name, json)
        str
      } catch {
        case ex: Exception => {
          ""
        }
      }
    }

    spark.udf.register("get_text_from_json", get_text_from_json _)

  }




  private def get_rowkey_udf(): String = {
    def get_row(): String = {
      if (StringUtils.isNotEmpty(job_args.rowkey_udf)) {
        return job_args.rowkey_udf
      }

      if (md5_fields.isEmpty) {
        s"company_id"
      } else {
        if (inter_cols.contains("company_id") && job_args.rowkey_add_company_id) {
          s"concat_ws('_',company_id,md5(cleanup(concat_ws('',${md5_fields.mkString(",")}))))"
        } else {
          s"md5(cleanup(concat_ws('',${md5_fields.mkString(",")})))"
        }
      }
    }

    if (job_args.id_user_defined_rowkey) {
      //使用ods层rowkey
      s" use_user_defined_rowkey(rowkey,${get_row()}) "
    } else {
      get_row()
    }


  }

  private def get_clean_up(): String = {
    s"""
       |${
      if (inter_cols.contains("company_id") && job_args.verify_company_id)
        s"""
           |AND company_id <> '0'
           |AND company_id is not null
           |AND trim(company_id) <> ''
           |""".stripMargin
      else
        s""
    }
       |${
      if (StringUtils.isEmpty(where))
        ""
      else
        s"AND ($where)"
    }
       |${
      if (is_super_filter)
        s"AND ${md5_fields.map(" " + _ + " is not null ").mkString("AND")}"
      else
        ""
    }
       |AND trim(concat_ws('',${md5_fields.mkString(",")})) <> ''
       |""".stripMargin
  }

  private def get_partition_order_by(): String = {
    if (inter_cols.contains("update_time")) {
      " ds DESC,update_time DESC "
    } else {
      " ds DESC "
    }
  }


  private val rowkey_f = get_rowkey_udf()


  private val clean_up = get_clean_up()


  private val up = get_partition_order_by()


  def all(): Unit = {
    val inc_ods_ds = getLastPartitionsOrElse(inc_ods_tab, getLastPartitionsOrElse(ods_tab, null))

    if (inc_ods_ds == null) {
      println("ds is null !!!")
      return
    }

    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $ads_tab PARTITION(ds='$inc_ods_ds')
         |SELECT  ${
        getColumns(ads_tab).diff(Seq("ds"))
          .map(f => {
            if (f.equalsIgnoreCase("deleted"))
              "COALESCE(deleted,0) as deleted"
            else
              f
          }).mkString(",")
      }
         |FROM    (
         |            SELECT  *
         |                    ,ROW_NUMBER() OVER(PARTITION BY rowkey ORDER BY $up) AS row_num
         |            FROM    (
         |                        SELECT  $rowkey_f as rowkey
         |                                ,${inter_cols.mkString(",")}
         |                        FROM    $ods_tab
         |                        WHERE   ds > 0
         |                        $clean_up
         |                        UNION ALL
         |                        SELECT  $rowkey_f as rowkey
         |                                ,${inter_cols.mkString(",")}
         |                        FROM    $inc_ods_tab
         |                        WHERE   ds > 0
         |                        $clean_up
         |                    ) AS t1
         |        ) AS t2
         |WHERE   t2.row_num = 1
         |""".stripMargin)
    explode_calc()
  }

  def inc(): Unit = {
    var org_ds: String = null
    var target_ds: String = null

    val inc_ods_ds = getLastPartitionsOrElse(inc_ods_tab, null)
    val inc_ads_ds = getLastPartitionsOrElse(inc_ads_tab, null)
    val ads_ds = getLastPartitionsOrElse(ads_tab, null)


    if (inc_ods_ds == null) {
      //没有inc_ods_tab数据，直接重跑全量ods数据
      all()
      return
    }
    target_ds = inc_ods_ds

    if (inc_ads_ds == null) {
      org_ds = ads_ds
    } else {
      org_ds = inc_ads_ds
    }

    if (org_ds.equals(target_ds)) {
      val inc_ads_sec_ds = getSecondLastPartitionOrElse(inc_ads_tab, null)
      if (inc_ads_sec_ds == null) {
        org_ds = ads_ds
      } else {
        org_ds = inc_ads_sec_ds
      }
    }


    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $inc_ads_tab PARTITION(ds='$target_ds')
         |SELECT  ${
        getColumns(inc_ads_tab).diff(Seq("ds"))
          .map(f => {
            if (f.equalsIgnoreCase("deleted"))
              "COALESCE(deleted,0) as deleted"
            else
              f
          }).mkString(",")
      }
         |FROM    (
         |            SELECT  *
         |                    ,ROW_NUMBER() OVER(PARTITION BY rowkey ORDER BY ${up} ) AS row_num
         |            FROM    (
         |                        SELECT  $rowkey_f as rowkey
         |                                ,${inter_cols.mkString(",")}
         |                        FROM    $inc_ods_tab
         |                        WHERE   ds > $org_ds
         |                        $clean_up
         |                    ) AS t1
         |        ) AS t2
         |WHERE   t2.row_num = 1
         |""".stripMargin)

    addEmptyPartitionOrSkip(inc_ads_tab, target_ds)
    explode_calc()
  }


  def calc(): Unit = {
    val ads_ds = getLastPartitionsOrElse(ads_tab, null)
    if (ads_ds == null)
      all()
    else
      inc()
  }


  def explode_calc(): Unit = {
    if (job_args.explode_args == null) {
      return
    }

    val ads_explode = s"winhc_ng.ads_${tn}_explode"
    val inc_ads_explode = s"winhc_ng.inc_ads_${tn}_explode"


    val all_date_tmp_view = s"insert_${tn}"
    val all_date_explode_tmp_view = s"${all_date_tmp_view}_explode"


    val args = StartAndEndDsUtils(spark).get_start_and_end_args(ads_tab, inc_ads_tab, ads_explode, inc_ads_explode)

    if (args.inc) {
      sql(
        s"""
           |SELECT  *
           |FROM    (
           |            SELECT  *
           |                    ,ROW_NUMBER() OVER(PARTITION BY rowkey ORDER BY $up ) AS row_num
           |            FROM    (
           |                        SELECT  *
           |                        FROM    $inc_ads_tab
           |                        WHERE   ds > ${args.inc_tab_gt_ds}
           |                    )
           |        )
           |WHERE   row_num = 1
           |""".stripMargin)
        .createTempView(all_date_tmp_view)
    } else {
      sql(
        s"""
           |SELECT  *
           |FROM    (
           |            SELECT  *
           |                    ,ROW_NUMBER() OVER(PARTITION BY rowkey ORDER BY $up ) AS row_num
           |            FROM    (
           |                        SELECT  *
           |                        FROM    $ads_tab
           |                        WHERE   ds > 0
           |                        UNION ALL
           |                        SELECT  *
           |                        FROM    $inc_ads_tab
           |                        WHERE   ds > 0
           |                    )
           |        )
           |WHERE   row_num = 1
           |""".stripMargin)
        .createTempView(all_date_tmp_view)
    }

    val explode_tab_name = args.inc match {
      case true => inc_ads_explode
      case false => ads_explode
    }

    explode_tab(spark, all_date_tmp_view, job_args.explode_args)
      .calc(all_date_explode_tmp_view)

    if (!spark.catalog.tableExists(explode_tab_name)) {
      //表不存在
    }
    sql(
      s"""
         |INSERT OVERWRITE TABLE $explode_tab_name PARTITION(ds='${args.target_ds}')
         |SELECT ${getColumns(explode_tab_name).diff(Seq("ds")).mkString(",")}
         |FROM
         |    $all_date_explode_tmp_view
         |""".stripMargin)
  }
}

object general_handler {

  def run(s: SparkSession, project: String, tn: String): Unit = {
    val args_job = args_company_job.get_args_company_job(tn)

    tn match {
      case _ => {
        general_handler(s = s, project = project, args_job).calc()
      }
    }
  }

  def main(args: Array[String]): Unit = {
    val Array(tn) = args
    val project = "winhc_ng"
    val config = EsConfig.getEsConfigMap ++ mutable.Map(
      "spark.hadoop.odps.project.name" -> project,
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "100"
    )
    val spark = SparkUtils.InitEnv(this.getClass.getSimpleName + ":" + tn, config)
    general_handler.run(spark, project, tn)
    spark.stop()

  }
}
