package com.winhc.bigdata.spark.ng.relation

import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.udf.{BaseFunc, CompanyMapping}
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{CompanyIndexUtils, CompanyRelationUtils, LoggingUtils, SparkUtils}
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @author: π
 * @date: 2021/01/10 09:36
 */
@deprecated
case class inc_company_relation(s: SparkSession,
                                project: String, //表所在工程名
                                ds: String //分区
                               ) extends LoggingUtils with BaseFunc with CompanyMapping {
  @(transient@getter) val spark: SparkSession = s

  val inc_ads_company = "winhc_ng.inc_ads_company"
  val ads_company = "winhc_ng.ads_company"
  val inc_ads_company_legal_entity = "winhc_ng.inc_ads_company_legal_entity"
  val inc_ads_company_holder = "winhc_ng.inc_ads_company_holder"
  val inc_ads_company_human_relation = "winhc_ng.inc_ads_company_human_relation"
  val inc_ads_company_staff = "winhc_ng.inc_ads_company_staff"

  val inc_ads_company_node = "winhc_ng.inc_ads_company_node"
  val inc_ads_relation_holder = "winhc_ng.inc_ads_relation_holder"
  val inc_ads_relation_staff = "winhc_ng.inc_ads_relation_staff"
  val inc_ads_relation_legal_entity = "winhc_ng.inc_ads_relation_legal_entity"

  def register_fun(): Unit = {
    prepareFunctions(spark)

    def get_company_node(id: String, name: String, deleted: String, topic_type: String): String = CompanyRelationUtils.get_company_node(id, name, deleted, topic_type)

    def get_relation_holder(start_id: String, start_name: String, end_id: String,
                            end_name: String, percent: Double, deleted: Int, holder_type: Int, topic_type: String): String =
      CompanyRelationUtils.get_relation_holder(start_id, start_name, end_id, end_name, percent.toString, deleted.toString, holder_type.toString, topic_type)

    def get_relation_staff(start_id: String, start_name: String, end_id: String,
                           end_name: String, staff_type: String, deleted: Int, topic_type: String): String =
      CompanyRelationUtils.get_relation_staff(start_id, start_name, end_id, end_name, staff_type, deleted.toString, topic_type)

    def get_relation_legal_entity(start_id: String, start_name: String, end_id: String,
                                  end_name: String, deleted: Int, legal_entity_type: String, topic_type: String): String =
      CompanyRelationUtils.get_relation_legal_entity(start_id, start_name, end_id, end_name, deleted.toString, legal_entity_type, topic_type)


    spark.udf.register("get_company_node", get_company_node _)
    spark.udf.register("get_relation_holder", get_relation_holder _)
    spark.udf.register("get_relation_staff", get_relation_staff _)
    spark.udf.register("get_relation_legal_entity", get_relation_legal_entity _)
  }

  def inc(): Unit = {

    //公司节点
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $inc_ads_company_node PARTITION(ds = '$ds')
         |SELECT  a.company_id id
         |        ,name_cleanup(a.name) name
         |        ,'企业'AS LABEL
         |        ,'0' deleted
         |FROM    (
         |            SELECT  company_id
         |                    ,name
         |            FROM    $inc_ads_company
         |            WHERE   ds = '$ds' AND deleted <> 9
         |        ) a
         |LEFT JOIN (
         |              SELECT  company_id,name
         |              FROM    (
         |                          SELECT  company_id,name
         |                                  ,ROW_NUMBER() OVER(PARTITION BY company_id,name ORDER BY ds DESC,update_time DESC) num
         |                          FROM    (
         |                                      SELECT  company_id
         |                                              ,name
         |                                              ,update_time
         |                                              ,ds
         |                                      FROM    $inc_ads_company
         |                                      WHERE   ds < '$ds' AND deleted <> 9
         |                                      UNION ALL
         |                                      SELECT  company_id
         |                                              ,name
         |                                              ,update_time
         |                                              ,ds
         |                                      FROM    $ads_company
         |                                      WHERE   ds > '0' AND deleted <> 9
         |                                  )
         |                      )
         |              WHERE   num = 1
         |          ) b
         |ON      a.company_id = b.company_id
         |AND     a.name = b.name
         |WHERE   b.company_id IS NULL
         |""".stripMargin)

    //增量法人表(新增,移除法人)
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $inc_ads_company_legal_entity PARTITION(ds= '$ds')
         |SELECT  c.*
         |FROM    (
         |            SELECT  CONCAT_WS('_',company_id,hash(legal_entity_name)) AS rowkey
         |                    ,company_id
         |                    ,name AS company_name
         |                    ,legal_entity_id
         |                    ,legal_entity_name
         |                    ,legal_entity_type
         |                    ,create_time
         |                    ,update_time
         |                    ,0 AS deleted
         |            FROM    $inc_ads_company
         |            WHERE   ds = '$ds'
         |            AND     legal_entity_id IS NOT NULL
         |            AND     length(trim(legal_entity_name)) > 0
         |            AND     deleted <> 9
         |        ) c
         |LEFT JOIN (
         |              SELECT  company_id
         |                      ,legal_entity_name
         |              FROM    (
         |                          SELECT  company_id
         |                                  ,legal_entity_name
         |                                  ,ROW_NUMBER() OVER(PARTITION BY company_id,legal_entity_name ORDER BY ds DESC,update_time DESC) num
         |                          FROM    $inc_ads_company_legal_entity
         |                          WHERE   ds < '$ds'
         |                      )
         |              WHERE   num = 1
         |          ) d
         |ON      c.company_id = d.company_id
         |AND     c.legal_entity_name = d.legal_entity_name
         |WHERE   d.company_id IS NULL
         |UNION ALL
         |SELECT  b.*
         |FROM    (
         |            SELECT  company_id
         |                    ,legal_entity_name
         |            FROM    (
         |                        SELECT  company_id
         |                                ,legal_entity_name
         |                                ,ROW_NUMBER() OVER(PARTITION BY company_id ORDER BY ds DESC,update_time DESC) num
         |                        FROM    $inc_ads_company
         |                        WHERE   ds = '$ds'
         |                        AND     legal_entity_id IS NOT NULL
         |                        AND     length(trim(legal_entity_name)) > 0
         |                        AND     deleted <> 9
         |                    )
         |            WHERE   num = 1
         |        ) a
         |JOIN    (
         |            SELECT  rowkey
         |                    ,company_id
         |                    ,company_name
         |                    ,legal_entity_id
         |                    ,legal_entity_name
         |                    ,legal_entity_type
         |                    ,create_time
         |                    ,update_time
         |                    ,deleted
         |            FROM    (
         |                        SELECT  rowkey
         |                                ,company_id
         |                                ,company_name
         |                                ,legal_entity_id
         |                                ,legal_entity_name
         |                                ,legal_entity_type
         |                                ,create_time
         |                                ,update_time
         |                                ,1 AS deleted
         |                                ,ROW_NUMBER() OVER(PARTITION BY company_id ORDER BY ds DESC,update_time DESC) num
         |                        FROM    $inc_ads_company_legal_entity
         |                        WHERE   ds <  '$ds'
         |                    )
         |            WHERE   num = 1
         |        ) b
         |ON      a.company_id = b.company_id
         |AND     a.legal_entity_name <> b.legal_entity_name
         |""".stripMargin)

    //历史人员去重缓存
    sql(
      s"""
         |SELECT  company_id,human_name,human_pid
         |FROM    (
         |          SELECT  company_id,human_name,human_pid
         |                            ,ROW_NUMBER() OVER (PARTITION BY company_id,human_name ORDER BY ds desc,update_time desc) num
         |           FROM    $inc_ads_company_human_relation
         |           WHERE   ds < '$ds'
         |          )
         |WHERE   num = 1
         |""".stripMargin).createOrReplaceTempView("company_human_relation_old")

    //新增人员关系表
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $inc_ads_company_human_relation PARTITION(ds= '$ds')
         |SELECT  CONCAT_WS('_',a.company_id,hash(a.human_name)) AS rowkey
         |        ,a.company_id
         |        ,a.company_name
         |        ,a.human_name
         |        ,hash(a.human_name) AS hid
         |        ,concat('p',md5(uuid())) human_pid
         |        ,a.STATUS
         |        ,a.create_time
         |        ,a.update_time
         |        ,a.deleted
         |FROM    (
         |            SELECT  *
         |            FROM    (
         |                        SELECT  company_id,company_name,human_name,hid,status,create_time,update_time,0 as deleted
         |                                ,ROW_NUMBER() OVER (PARTITION BY company_id,human_name ORDER BY ds desc,update_time desc) num
         |                        FROM    (
         |                                    --股东
         |                                    SELECT  company_id,company_name,holder_name human_name,holder_id hid,2 as status,create_time,update_time,0 as deleted,ds
         |                                    FROM    $inc_ads_company_holder
         |                                    WHERE   ds = '$ds'
         |                                    UNION ALL
         |                                    --主要成员
         |                                    SELECT  company_id,company_name,staff_name human_name,hid,2 as status,create_time,update_time,0 as deleted,ds
         |                                    FROM    $inc_ads_company_staff
         |                                    WHERE   ds = '$ds'
         |                                    UNION ALL
         |                                    --法人
         |                                    SELECT  company_id,company_name,legal_entity_name human_name,legal_entity_id as hid,2 as status,create_time,update_time,0 as deleted,ds
         |                                    FROM    $inc_ads_company_legal_entity
         |                                    WHERE   ds = '$ds'
         |                                )
         |                    )
         |            WHERE   num = 1
         |        ) a
         |LEFT JOIN (
         |SELECT * FROM company_human_relation_old
         |) b
         |ON      a.company_id = b.company_id
         |AND     a.human_name = b.human_name
         |WHERE   b.company_id IS NULL
         |""".stripMargin)

    //股东关系
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"}  table $inc_ads_relation_holder  PARTITION (ds= '$ds')
         |SELECT
         |start_id,start_name,end_id,end_name,percent,deleted,holder_type,label
         |FROM (
         |SELECT coalesce(c.human_pid,'') start_id,name_cleanup(c.human_name) start_name,a.company_id end_id,name_cleanup(a.company_name) end_name,percent,deleted,holder_type,label
         |from (
         |SELECT * ,
         |    '投资' as LABEL
         | FROM    $inc_ads_company_holder
         | WHERE   ds = '$ds'
         |and holder_type = 1
         |)a
         |JOIN
         |(
         |SELECT * FROM company_human_relation_old
         |) c
         |ON a.company_id = c.company_id and a.holder_name = c.human_name
         |UNION ALL
         |SELECT holder_id start_id,name_cleanup(holder_name) start_name,a.company_id end_id,name_cleanup(company_name) end_name,percent,deleted,holder_type,label
         |from (
         |SELECT * ,
         |    '投资' as LABEL
         | FROM    $inc_ads_company_holder
         |WHERE   ds = '$ds'
         |and holder_type = 2
         |)a
         |)
         |where start_id <> end_id and start_id is not null and end_id is not null
         |""".stripMargin)

    //主要成员关系
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"}  table $inc_ads_relation_staff  PARTITION (ds= '$ds')
         |SELECT
         |start_id,start_name,end_id,end_name,staff_type,deleted,label
         |FROM (
         |SELECT c.human_pid start_id,name_cleanup(c.human_name) start_name,a.company_id end_id,name_cleanup(a.company_name) end_name,name_cleanup(staff_type) staff_type,deleted,label
         |from (
         |SELECT * ,
         |    '高管' as LABEL
         | FROM    $inc_ads_company_staff
         |WHERE   ds = '$ds'
         |)a
         |JOIN
         |(
         |SELECT * FROM company_human_relation_old
         |) c
         |ON a.company_id = c.company_id and a.staff_name   = c.human_name
         |)
         |where start_id <> end_id and start_id is not null and end_id is not null
         |""".stripMargin)

    //法人关系
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} table $inc_ads_relation_legal_entity   PARTITION (ds='$ds')
         |SELECT
         |start_id,start_name,end_id,end_name,deleted,legal_entity_type,label
         |FROM (
         |SELECT c.human_pid start_id,name_cleanup(c.human_name) start_name,a.company_id end_id,name_cleanup(a.company_name) end_name,deleted,legal_entity_type,label
         |from (
         |SELECT * ,
         |    '法人' as LABEL
         | FROM    $inc_ads_company_legal_entity
         |WHERE   ds = '$ds'
         |and legal_entity_type = 1
         |)a
         |JOIN
         |(
         |SELECT * FROM company_human_relation_old
         |) c
         |ON a.company_id = c.company_id and a.legal_entity_name = c.human_name
         |UNION ALL
         |SELECT legal_entity_id start_id,name_cleanup(legal_entity_name) start_name,a.company_id end_id,name_cleanup(company_name) end_name,deleted,legal_entity_type,label
         |from (
         |SELECT * ,
         |    '法人' as LABEL
         | FROM    $inc_ads_company_legal_entity
         |WHERE   ds = '$ds'
         |and legal_entity_type = 2
         |)a
         |)
         |where start_id <> end_id and start_id is not null and end_id is not null
         |""".stripMargin)

  }

  def sendKafkaPre(): Unit = {
    val inc_ads_company_node_kafka = "winhc_ng.inc_ads_company_node_kafka"
    val inc_ads_relation_holder_v1_kafka = "winhc_ng.inc_ads_relation_holder_v1_kafka"
    val inc_ads_relation_holder_v2_kafka = "winhc_ng.inc_ads_relation_holder_v2_kafka"
    val inc_ads_relation_staff_kafka = "winhc_ng.inc_ads_relation_staff_kafka"
    val inc_ads_relation_legal_entity_v1_kafka = "winhc_ng.inc_ads_relation_legal_entity_v1_kafka"
    val inc_ads_relation_legal_entity_v2_kafka = "winhc_ng.inc_ads_relation_legal_entity_v2_kafka"
    //公司节点
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} table $inc_ads_company_node_kafka  PARTITION (ds='$ds')
         |select
         |id key,
         |get_company_node(id, name, deleted, '1') message
         |from $inc_ads_company_node
         |where ds = '$ds'
         |""".stripMargin).show(20, false)

    //股东关系(人 -> 公司)
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} table $inc_ads_relation_holder_v1_kafka  PARTITION (ds='$ds')
         |select
         |end_id key,
         |get_relation_holder(start_id, start_name, end_id, end_name, percent, deleted, holder_type, '2') message
         |from $inc_ads_relation_holder
         |where ds = '$ds'
         |and holder_type = 1
         |""".stripMargin).show(20, false)

    //股东关系(公司 -> 公司)
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} table $inc_ads_relation_holder_v2_kafka  PARTITION (ds='$ds')
         |select
         |end_id key,
         |get_relation_holder(start_id, start_name, end_id, end_name, percent, deleted, holder_type, '3') message
         |from $inc_ads_relation_holder
         |where ds = '$ds'
         |and holder_type = 2
         |""".stripMargin).show(20, false)

    //法人关系(人 -> 公司)
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} table $inc_ads_relation_legal_entity_v1_kafka  PARTITION (ds='$ds')
         |select
         |end_id key,
         |get_relation_legal_entity(start_id, start_name, end_id, end_name, deleted, legal_entity_type, '4') message
         |from $inc_ads_relation_legal_entity
         |where ds = '$ds'
         |and legal_entity_type = '1'
         |""".stripMargin).show(20, false)

    //法人关系(公司 -> 公司)
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} table $inc_ads_relation_legal_entity_v2_kafka  PARTITION (ds='$ds')
         |select
         |end_id key,
         |get_relation_legal_entity(start_id, start_name, end_id, end_name, deleted, legal_entity_type, '5') message
         |from $inc_ads_relation_legal_entity
         |where ds = '$ds'
         |and legal_entity_type = '2'
         |""".stripMargin).show(20, false)

    //主要成员关系(人 -> 公司)
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} table $inc_ads_relation_staff_kafka  PARTITION (ds='$ds')
         |select
         |end_id key,
         |get_relation_staff(start_id, start_name, end_id, end_name, staff_type, deleted, '6') message
         |from $inc_ads_relation_staff
         |where ds = '$ds'
         |""".stripMargin).show(20, false)

    //防止空分区
    addEmptyPartitionOrSkip(inc_ads_company_node_kafka, ds)
    addEmptyPartitionOrSkip(inc_ads_relation_holder_v1_kafka, ds)
    addEmptyPartitionOrSkip(inc_ads_relation_holder_v2_kafka, ds)
    addEmptyPartitionOrSkip(inc_ads_relation_staff_kafka, ds)
    addEmptyPartitionOrSkip(inc_ads_relation_legal_entity_v1_kafka, ds)
    addEmptyPartitionOrSkip(inc_ads_relation_legal_entity_v2_kafka, ds)

  }

}

object inc_company_relation {
  def main(args: Array[String]): Unit = {
    if (args.size != 2) {
      println("please set project ds.")
      sys.exit(-1)
    }
    val Array(project, ds) = args
    //    val project = "winhc_ng"
    //    val ds = "20210106"
    val config = EsConfig.getEsConfigMap ++ mutable.Map(
      "spark.hadoop.odps.project.name" -> project,
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "10000"
    )
    val spark = SparkUtils.InitEnv(this.getClass.getSimpleName, config)
    val re = inc_company_relation(s = spark, project = project, ds = ds)
    re.register_fun()
    re.inc()
    re.sendKafkaPre()
    spark.stop()
  }
}

