package com.winhc.bigdata.spark.ng.relation

import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.udf.{BaseFunc, CompanyMapping}
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{CompanyRelationUtils, LoggingUtils, SparkUtils}
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @author: π
 * @date: 2021/01/10 09:36
 */
case class inc_human_relation_merge(s: SparkSession,
                                    project: String, //表所在工程名
                                    ds: String //分区
                                   ) extends LoggingUtils with BaseFunc with CompanyMapping {
  @(transient@getter) val spark: SparkSession = s

  val inc_ads_company = "winhc_ng.inc_ads_company"
  val ads_company = "winhc_ng.ads_company"
  val inc_ads_company_legal_entity = "winhc_ng.inc_ads_company_legal_entity"
  val inc_ads_company_holder = "winhc_ng.inc_ads_company_holder"
  val inc_ads_company_human_relation = "winhc_ng.inc_ads_company_human_relation"
  val inc_ods_company_human_relation_merge = "winhc_ng.inc_ods_company_human_relation_merge"
  val inc_ads_company_human_relation_merge = "winhc_ng.inc_ads_company_human_relation_merge"
  val inc_ads_company_human_relation_merge_deleted = "winhc_ng.inc_ads_company_human_relation_merge_deleted"
  val inc_ads_company_human_relation_merge_update = "winhc_ng.inc_ads_company_human_relation_merge_update"
  val inc_ads_company_staff = "winhc_ng.inc_ads_company_staff"

  val ads_change_extract = "winhc_ng.bds_change_extract"

  val inc_ads_company_node = "winhc_ng.inc_ads_company_node"
  val inc_ads_relation_holder = "winhc_ng.inc_ads_relation_holder"
  val inc_ads_relation_staff = "winhc_ng.inc_ads_relation_staff"
  val inc_ads_relation_legal_entity = "winhc_ng.inc_ads_relation_legal_entity"

  def register_fun(): Unit = {
    prepareFunctions(spark)
  }

  def inc(): Unit = {

    //合并删除
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} table $inc_ads_company_human_relation_merge PARTITION (ds='$ds')
         |SELECT  CONCAT_WS('_',a.company_id,a.person_id) AS rowkey
         |        ,a.company_id
         |        ,b.company_name
         |        ,a.person_name human_name
         |        ,hash(a.person_name) hid
         |        ,a.person_id human_pid
         |        ,2 STATUS
         |        ,a.create_time
         |        ,a.update_time
         |        ,a.deleted
         |        ,b.province_code
         |        ,b.city_code
         |        ,b.county_code
         |        ,b.reg_capital_amount
         |        ,b.cate_first_code
         |        ,b.cate_second_code
         |        ,b.cate_third_code
         |FROM    (
         |           SELECT  *
         |           FROM    (
         |                       SELECT  company_id
         |                               ,person_name
         |                               ,person_id
         |                               ,create_time
         |                               ,update_time
         |                               ,label
         |                               ,ROW_NUMBER() OVER (PARTITION BY company_id,person_id ORDER BY label) num
         |                               ,MIN(deleted) OVER (PARTITION BY company_id,person_id) deleted
         |                       FROM    $inc_ods_company_human_relation_merge
         |                       WHERE   ds = '$ds'
         |                       AND     length(company_id) = 32
         |                   )
         |           WHERE num = 1
         |        ) a
         |JOIN    (
         |            SELECT  *
         |            FROM    (
         |                        SELECT  *
         |                                ,ROW_NUMBER() OVER(PARTITION BY company_id ORDER BY ds DESC,update_time DESC) num
         |                        FROM    (
         |                                    SELECT  company_id
         |                                            ,name AS company_name
         |                                            ,province_code
         |                                            ,city_code
         |                                            ,county_code
         |                                            ,reg_capital_amount
         |                                            ,cate_first_code
         |                                            ,cate_second_code
         |                                            ,cate_third_code
         |                                            ,update_time
         |                                            ,ds
         |                                    FROM    $inc_ads_company
         |                                    WHERE   ds > '0'
         |                                    UNION ALL
         |                                    SELECT  company_id
         |                                            ,name AS company_name
         |                                            ,province_code
         |                                            ,city_code
         |                                            ,county_code
         |                                            ,reg_capital_amount
         |                                            ,cate_first_code
         |                                            ,cate_second_code
         |                                            ,cate_third_code
         |                                            ,update_time
         |                                            ,ds
         |                                    FROM    $ads_company
         |                                    WHERE   ds > '0'
         |                                )
         |                    ) c
         |            WHERE   num = 1
         |        ) b
         |ON      a.company_id = b.company_id
         |UNION ALL
         |SELECT
         |         CONCAT_WS('_',company_id,a.person_id) AS rowkey
         |        ,company_id
         |        ,company_name
         |        ,a.person_name human_name
         |        ,hash(a.person_name) hid
         |        ,a.person_id human_pid
         |        ,STATUS
         |        ,a.create_time
         |        ,a.update_time
         |        ,a.deleted
         |        ,province_code
         |        ,city_code
         |        ,county_code
         |        ,reg_capital_amount
         |        ,cate_first_code
         |        ,cate_second_code
         |        ,cate_third_code
         |from (
         |  SELECT person_name
         |        ,person_id
         |        ,create_time
         |        ,update_time
         |        ,deleted
         |FROM    $inc_ods_company_human_relation_merge
         |WHERE   ds = '$ds'
         |AND     length(company_id) = 0
         |) a
         |JOIN (
         |    SELECT *
         |    FROM  $inc_ads_company_human_relation_merge
         |    WHERE   ds < '$ds'
         |    -- AND     deleted <> 9
         |    UNION ALL
         |    SELECT *
         |    FROM  $inc_ads_company_human_relation
         |    WHERE   ds > '0'
         |    -- AND     deleted <> 9
         |) b
         |on a.person_id = b.human_pid
         |""".stripMargin)

  }

  //导出ES数据准备
  def exportESPre(): Unit = {
    //删除数据
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $inc_ads_company_human_relation_merge_deleted PARTITION (ds='$ds')
         |SELECT
         |        human_pid
         |        ,max(deleted) deleted
         |FROM    (
         |        SELECT
         |                human_pid
         |                ,deleted
         |        FROM    $inc_ads_company_human_relation_merge
         |        WHERE   ds = '$ds'
         |        AND deleted = 9
         |        UNION ALL
         |        SELECT  human_pid,deleted
         |        FROM    (
         |                SELECT
         |                        human_pid
         |                        ,MIN(deleted) OVER(PARTITION BY human_pid ) deleted
         |                FROM    $inc_ads_company_human_relation_merge
         |                WHERE   ds = '$ds'
         |                AND deleted <> 9
         |                )
         |        WHERE   deleted > 0
         |        )
         |GROUP BY human_pid
         |""".stripMargin)

    //增量合并数据
    inc_human_relation_util
      .merge(s, ds, inc_ads_company_human_relation_merge, inc_ads_company_human_relation_merge_update)

  }

}


object inc_human_relation_merge {
  def main(args: Array[String]): Unit = {
    if (args.size != 2) {
      println("please set project ds.")
      sys.exit(-1)
    }
    val Array(project, ds) = args
    val config = EsConfig.getEsConfigMap ++ mutable.Map(
      "spark.hadoop.odps.project.name" -> project,
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "10000"
    )
    val spark = SparkUtils.InitEnv(this.getClass.getSimpleName, config)
    val re = inc_human_relation_merge(s = spark, project = project, ds = ds)
    re.register_fun()
    re.inc()
    re.exportESPre()
    spark.stop()
  }
}

object inc_human_relation_util {
  def merge(spark: SparkSession, ds: String, source_tab: String, target_tab: String): Unit = {
    //合并逻辑
    spark.sql(
      s"""
         |SELECT  *
         |        ,ROW_NUMBER() OVER(PARTITION BY human_pid ORDER BY tmp_val desc,province_num DESC ,human_pid) AS num_rank
         |FROM    (
         |            SELECT  human_pid
         |                    ,case
         |                     when province_code = '' then 0
         |                     when province_code = '0' then 0
         |                     when province_code is null then 0
         |                     else 1
         |                     end as tmp_val
         |                    ,COLLECT_SET(human_name)[0] AS human_name
         |                    ,province_code
         |                    ,COUNT(1) AS province_num
         |                    ,CONCAT_WS(',',collect_set(area_code)) AS area_code
         |                    ,CONCAT_WS(',',collect_set(cate_code)) AS cate_code
         |                    ,COLLECT_SET(first_company)[0] AS first_company
         |                    ,COLLECT_SET(first_company_reg_capital_amount)[0] AS first_company_reg_capital_amount
         |                    ,COLLECT_SET(total_compny_cnt)[0] AS total_compny_cnt
         |            FROM    (
         |                        SELECT  company_id
         |                                ,human_name
         |                                ,human_pid
         |                                ,deleted
         |                                ,CONCAT_WS('|', province_code,city_code,county_code) AS area_code
         |                                ,case
         |                                   when province_code = '' then '0'
         |                                   when province_code = '0' then '0'
         |                                   when province_code is null then '0'
         |                                else province_code
         |                                end as  province_code
         |                                ,CONCAT_WS('|', cate_first_code,cate_second_code,cate_third_code) AS cate_code
         |                                ,FIRST_VALUE(company_id) OVER (PARTITION BY human_pid,province_code ORDER BY reg_capital_amount DESC) AS first_company
         |                                ,FIRST_VALUE(reg_capital_amount) OVER (PARTITION BY human_pid,province_code ORDER BY reg_capital_amount DESC) AS first_company_reg_capital_amount
         |                                ,COUNT(company_id) OVER(PARTITION BY human_pid) AS total_compny_cnt
         |                        FROM    (
         |                                    SELECT  *
         |                                    FROM    $source_tab
         |                                    WHERE   ds = "$ds"
         |                                    AND deleted = 0
         |                                )
         |                    )
         |            GROUP BY human_pid
         |                     ,province_code
         |        )
         |""".stripMargin).createOrReplaceTempView("tmp_tab")

    spark.sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} table $target_tab PARTITION (ds='$ds')
         |SELECT  human_pid
         |        ,COLLECT_SET(human_name)[0] AS human_name
         |        ,CONCAT_ws(',',COLLECT_SET(area_code)) AS area_code
         |        ,CONCAT_ws(',',COLLECT_SET(cate_code)) AS cate_code
         |        ,CONCAT('[',CONCAT_WS(',',collect_set(code_company_num_tup)),']') AS code_company_num_map
         |        ,0 deleted
         |        ,$ds as update_time
         |        ,COLLECT_SET(total_compny_cnt)[0] AS total_compny_cnt
         |FROM    (
         |            SELECT  human_pid
         |                    ,COLLECT_SET(human_name)[0] AS human_name
         |                    ,CONCAT_WS(',',COLLECT_SET(area_code)) AS area_code
         |                    ,CONCAT_WS(',',COLLECT_SET(cate_code)) AS cate_code
         |                    ,CONCAT(
         |                        '{"province_code":"0","province_num":"',sum(province_num),'","company_id":"',collect_set(first_company)[0],'"}'
         |                    ) AS code_company_num_tup
         |                    ,COLLECT_SET(total_compny_cnt)[0] AS total_compny_cnt
         |            FROM    (
         |                        SELECT  human_pid
         |                                ,human_name
         |                                ,province_num
         |                                ,area_code
         |                                ,cate_code
         |                                ,first_company_reg_capital_amount
         |                                ,FIRST_VALUE(first_company) OVER (PARTITION BY human_pid ORDER BY first_company_reg_capital_amount DESC) AS first_company
         |                                ,total_compny_cnt
         |                        FROM    tmp_tab
         |                        WHERE   num_rank > 2
         |                    )
         |            GROUP BY human_pid
         |            UNION ALL
         |            SELECT  human_pid
         |                    ,human_name
         |                    ,area_code
         |                    ,cate_code
         |                    ,CONCAT(
         |                        '{"province_code":"'
         |                        ,province_code
         |                        ,'","province_num":"',province_num
         |                        ,'","company_id":"',first_company
         |                        ,'"}'
         |                    ) AS code_company_num_tup
         |                    ,total_compny_cnt
         |            FROM    tmp_tab
         |            WHERE   num_rank <= 2
         |        )
         |GROUP BY human_pid
         |""".stripMargin)
  }
}


