package com.winhc.bigdata.spark.ng.utils

import com.winhc.bigdata.spark.utils.SparkUtils

import scala.collection.mutable

/**
 * @Author: XuJiakai
 * @Date: 2020/10/10 10:23
 * @Description: winhc_ng空间下摘要增强版
 */
object CompanySummaryNg_new {

  private def get_default_summary_args(tableName: String): SummaryArgs = {
    SummaryArgs(
      table_name = tableName
      , companyIdField = "split(rowkey,'_')[0]"
      , where = "instr(rowkey,'_') != 0"
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"${tableName}_del_0")
        , ("1", s"${tableName}_del_1")
      ))
    )
  }

  private def get_default_summary_args(tableName: String, company_id: String): SummaryArgs = {
    SummaryArgs(
      table_name = tableName
      , companyIdField = company_id
      , where = s"$company_id is not null and length($company_id) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"${tableName}_del_0")
        , ("1", s"${tableName}_del_1")
      ))
    )
  }

  val start_args = Seq(
    get_default_summary_args("company_holder")
    , get_default_summary_args("company_staff")
    , get_default_summary_args("company_icp")
    , get_default_summary_args("company_tm", company_id = "keyno")
    , get_default_summary_args("company_app_info")
    , get_default_summary_args("finance_info_v3", "company_id")
    , get_default_summary_args("private_enterprise", "company_id")
    , get_default_summary_args("company_finance", "company_id")
    , get_default_summary_args("company_equity_pledge", "company_id")
    , get_default_summary_args("company_env_punishment", "company_id")
    , get_default_summary_args("company_liquidating_info", "company_id")
    , get_default_summary_args("company_certificate", "company_id")
    //    , get_default_summary_args("company_icp", "company_id")
    , get_default_summary_args("company_weibo", "company_id")
    , get_default_summary_args("company_customs_credit", "company_id")
    , get_default_summary_args("company_license", "company_id")
    , get_default_summary_args("company_license_creditchina", "company_id")
    , get_default_summary_args("company_license_entpub", "company_id")
    , get_default_summary_args("company_tax", "company_id")
    , get_default_summary_args("company_land_announcement", "company_id")
    , get_default_summary_args("company_land_publicity", "company_id")
    , get_default_summary_args("company_bond", "company_id")
    , get_default_summary_args("company_tele_license", "company_id")
    , get_default_summary_args("company_change", "company_id")


    , SummaryArgs(table_name = "company_court_open_announcement_explode"
      , companyIdField = "plaintiff_info_id_explode"
      , distinctField = "rowkey,plaintiff_info_id_explode"
      , where = "plaintiff_info_id_explode is not null and length(plaintiff_info_id_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_court_open_announcement_del_0_plaintiff")
        , ("1", s"company_court_open_announcement_del_1_plaintiff")
      ))
    )
    , SummaryArgs(table_name = "company_court_open_announcement_explode"
      , companyIdField = "defendant_info_id_explode"
      , distinctField = "rowkey,defendant_info_id_explode"
      , where = "defendant_info_id_explode is not null and length(defendant_info_id_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_court_open_announcement_del_0_defendant")
        , ("1", s"company_court_open_announcement_del_1_defendant")
      ))
    )

    , SummaryArgs(table_name = "company_dishonest_info"
      , companyIdField = "keyno"
      , where = "keyno is not null and length(keyno) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_dishonest_info_del_0")
        , ("1", s"company_dishonest_info_del_1")
      ))
    )
    , SummaryArgs(table_name = "company_zxr_restrict"
      , companyIdField = "company_id"
      , where = "company_id is not null and length(company_id) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_zxr_restrict_del_0")
        , ("1", s"company_zxr_restrict_del_1")
      ))
    )

    , SummaryArgs(table_name = "company_abnormal_info"
      , companyIdField = "company_id"
      , where = "company_id is not null and length(company_id) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_abnormal_info_del_0")
        , ("1", s"company_abnormal_info_del_1")
      ))
    )

    //todo
    , SummaryArgs(table_name = "company_public_announcement"
      , companyIdField = "company_id"
      , where = "company_id is not null and length(company_id) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_public_announcement_del_0")
        , ("1", s"company_public_announcement_del_1")
      ))
    )
    , SummaryArgs(table_name = "company_illegal_info"
      , companyIdField = "company_id"
      , where = "company_id is not null and length(company_id) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_illegal_info_del_0")
        , ("1", s"company_illegal_info_del_1")
      ))
    )


    , SummaryArgs(table_name = "company_land_mortgage"
      , companyIdField = "mortgagor_company_id"
      , where = "mortgagor_company_id is not null and length(mortgagor_company_id) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_land_mortgage_del_0_mortgagor")
        , ("1", s"company_land_mortgage_del_1_mortgagor")
      ))
    )


    , SummaryArgs(table_name = "company_land_mortgage"
      , companyIdField = "mortgagee_company_id"
      , where = "mortgagee_company_id is not null and length(mortgagee_company_id) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_land_mortgage_del_0_mortgagee")
        , ("1", s"company_land_mortgage_del_1_mortgagee")
      ))
    )


    , SummaryArgs(table_name = "company_judicial_assistance"
      , companyIdField = "company_id"
      , where = "company_id is not null and length(company_id) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_judicial_assistance_del_0_related")
        , ("1", s"company_judicial_assistance_del_1_related")
      ))
    )

    , SummaryArgs(table_name = "company_judicial_assistance"
      , companyIdField = "executed_person_id"
      , where = "executed_person_id is not null and length(executed_person_id) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_judicial_assistance_del_0_executed_person")
        , ("1", s"company_judicial_assistance_del_1_executed_person")
      ))
    )


    , SummaryArgs(table_name = "company_equity_info"
      , companyIdField = "related_company_id"
      , where = "related_company_id is not null and length(related_company_id) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_equity_info_del_0_related")
        , ("1", s"company_equity_info_del_1_related")
      ))
    )


    , SummaryArgs(table_name = "company_equity_info_explode"
      , companyIdField = "pledgor_keyno_explode"
      , distinctField = "rowkey,pledgor_keyno_explode"
      , where = "pledgor_keyno_explode is not null and length(pledgor_keyno_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_equity_info_del_0_pledgor")
        , ("1", s"company_equity_info_del_1_pledgor")
      ))
    )
    , SummaryArgs(table_name = "company_equity_info_explode"
      , companyIdField = "pledgee_keyno_explode"
      , distinctField = "rowkey,pledgee_keyno_explode"
      , where = "pledgee_keyno_explode is not null and length(pledgee_keyno_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_equity_info_del_0_pledgee")
        , ("1", s"company_equity_info_del_1_pledgee")
      ))
    )


    , SummaryArgs(table_name = "company_lawsuit_explode"
      , companyIdField = "plaintiff_info_id_explode"
      , distinctField = "rowkey,plaintiff_info_id_explode"
      , where = "plaintiff_info_id_explode is not null and length(plaintiff_info_id_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_lawsuit_del_0_plaintiff")
        , ("1", s"company_lawsuit_del_1_plaintiff")
      ))
    )
    , SummaryArgs(table_name = "company_lawsuit_explode"
      , companyIdField = "defendant_info_id_explode"
      , distinctField = "rowkey,defendant_info_id_explode"
      , where = "defendant_info_id_explode is not null and length(defendant_info_id_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_lawsuit_del_0_defendant")
        , ("1", s"company_lawsuit_del_1_defendant")
      ))
    )


    , SummaryArgs(table_name = "company_court_announcement_explode"
      , companyIdField = "plaintiff_info_id_explode"
      , distinctField = "rowkey,plaintiff_info_id_explode"
      , where = "plaintiff_info_id_explode is not null and length(plaintiff_info_id_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_court_announcement_del_0_plaintiff")
        , ("1", s"company_court_announcement_del_1_plaintiff")
      ))
    )
    , SummaryArgs(table_name = "company_court_announcement_explode"
      , companyIdField = "litigant_info_id_explode"
      , distinctField = "rowkey,litigant_info_id_explode"
      , where = "litigant_info_id_explode is not null and length(litigant_info_id_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_court_announcement_del_0_litigant")
        , ("1", s"company_court_announcement_del_1_litigant")
      ))
    )
    , SummaryArgs(table_name = "company_send_announcement_explode"
      , companyIdField = "plaintiff_info_id_explode"
      , distinctField = "rowkey,plaintiff_info_id_explode"
      , where = "plaintiff_info_id_explode is not null and length(plaintiff_info_id_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_send_announcement_del_0_plaintiff")
        , ("1", s"company_send_announcement_del_1_plaintiff")
      ))
    )
    , SummaryArgs(table_name = "company_send_announcement_explode"
      , companyIdField = "defendant_info_id_explode"
      , distinctField = "rowkey,defendant_info_id_explode"
      , where = "defendant_info_id_explode is not null and length(defendant_info_id_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_send_announcement_del_0_defendant")
        , ("1", s"company_send_announcement_del_1_defendant")
      ))
    ), SummaryArgs(table_name = "company_zxr_final_case"
      , companyIdField = "keyno"
      , where = "keyno is not null and length(keyno) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_zxr_final_case_del_0")
        , ("1", s"company_zxr_final_case_del_1")
      ))
    )

    , SummaryArgs(table_name = "company_court_register_explode"
      , companyIdField = "plaintiff_info_id_explode"
      , distinctField = "rowkey,plaintiff_info_id_explode"
      , where = "plaintiff_info_id_explode is not null and length(plaintiff_info_id_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_court_register_del_0_plaintiff")
        , ("1", s"company_court_register_del_1_plaintiff")
      ))
    )

    , SummaryArgs(table_name = "company_court_register_explode"
      , companyIdField = "defendant_info_id_explode"
      , distinctField = "rowkey,defendant_info_id_explode"
      , where = "defendant_info_id_explode is not null and length(defendant_info_id_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_court_register_del_0_defendant")
        , ("1", s"company_court_register_del_1_defendant")
      ))
    )

    // ====================================================

    , SummaryArgs(table_name = "bankruptcy_open_case_explode"
      , companyIdField = "applicant_info_id_explode"
      , distinctField = "rowkey,applicant_info_id_explode"
      , where = "applicant_info_id_explode is not null and length(applicant_info_id_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"bankruptcy_open_case_del_0_applicant")
        , ("1", s"bankruptcy_open_case_del_1_applicant")
      ))
    )

    , SummaryArgs(table_name = "bankruptcy_open_case_explode"
      , companyIdField = "respondent_info_id_explode"
      , distinctField = "rowkey,respondent_info_id_explode"
      , where = "respondent_info_id_explode is not null and length(respondent_info_id_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"bankruptcy_open_case_del_0_respondent")
        , ("1", s"bankruptcy_open_case_del_1_respondent")
      ))
    )


    , SummaryArgs(table_name = "company_ipr_pledge"
      , companyIdField = "related_company_id"
      , where = "related_company_id is not null and length(related_company_id) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_ipr_pledge_del_0_related")
        , ("1", s"company_ipr_pledge_del_1_related")
      ))
    )


    , SummaryArgs(table_name = "company_ipr_pledge_explode"
      , companyIdField = "pledgor_keyno_explode"
      , distinctField = "rowkey,pledgor_keyno_explode"
      , where = "pledgor_keyno_explode is not null and length(pledgor_keyno_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_ipr_pledge_del_0_pledgor")
        , ("1", s"company_ipr_pledge_del_1_pledgor")
      ))
    )

    , SummaryArgs(table_name = "company_ipr_pledge_explode"
      , companyIdField = "pledgee_keyno_explode"
      , distinctField = "rowkey,pledgee_keyno_explode"
      , where = "pledgee_keyno_explode is not null and length(pledgee_keyno_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_ipr_pledge_del_0_pledgee")
        , ("1", s"company_ipr_pledge_del_1_pledgee")
      ))
    )

    //====================================================

    , get_default_summary_args("zxr_evaluate", "keyno")
    , get_default_summary_args("zxr_evaluate_results", "keyno")
    , get_default_summary_args("restrictions_on_exit", "executed_person_keyno")
    , get_default_summary_args("company_zxr", "keyno")
    , get_default_summary_args("company_punishment_info", "company_id")
    , get_default_summary_args("company_tax_contravention", "company_id")
    , get_default_summary_args("company_own_tax", "company_id")
    , get_default_summary_args("company_check_info", "company_id")
    , get_default_summary_args("company_punishment_info_creditchina", "company_id")
    , get_default_summary_args("company_mortgage_info", "company_id")
    , get_default_summary_args("company_brief_cancel_announcement", "company_id")
    , get_default_summary_args("company_double_random_check_info", "company_id")

    , SummaryArgs(table_name = "auction_tracking_explode"
      , companyIdField = "company_info_id_explode"
      , distinctField = "rowkey,company_info_id_explode"
      , where = "company_info_id_explode is not null and length(company_info_id_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"auction_tracking_del_0")
        , ("1", s"auction_tracking_del_1")
      ))
    )

    , SummaryArgs(table_name = "company_copyright_reg_explode"
      , companyIdField = "author_nationality_info_keyno_explode"
      , distinctField = "rowkey,author_nationality_info_keyno_explode"
      , where = "author_nationality_info_keyno_explode is not null and length(author_nationality_info_keyno_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_copyright_reg_del_0")
        , ("1", s"company_copyright_reg_del_1")
      ))
    )

    , SummaryArgs(table_name = "company_copyright_works_explode"
      , companyIdField = "author_info_keyno_explode"
      , distinctField = "rowkey,author_info_keyno_explode"
      , where = "author_info_keyno_explode is not null and length(author_info_keyno_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_copyright_works_del_0")
        , ("1", s"company_copyright_works_del_1")
      ))
    )

    , SummaryArgs(table_name = "company_patent_explode"
      , companyIdField = "applicant_name_info_keyno_explode"
      , distinctField = "rowkey,applicant_name_info_keyno_explode"
      , where = "applicant_name_info_keyno_explode is not null and length(applicant_name_info_keyno_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_patent_del_0")
        , ("1", s"company_patent_del_1")
      ))
    )


    , SummaryArgs(table_name = "company_land_transfer"
      , companyIdField = "pre_keyno"
      , where = "pre_keyno is not null and length(pre_keyno) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_land_transfer_del_0_user_pre")
        , ("1", s"company_land_transfer_del_1_user_pre")
      ))
    )


    , SummaryArgs(table_name = "company_land_transfer"
      , companyIdField = "now_keyno"
      , where = "now_keyno is not null and length(now_keyno) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_land_transfer_del_0_user_now")
        , ("1", s"company_land_transfer_del_1_user_now")
      ))
    )

    , SummaryArgs(table_name = "company_equity_pledge_holder"
      , companyIdField = "company_id"
      , where = "company_id is not null and length(company_id) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_equity_pledge_holder_del_0_company")
        , ("1", s"company_equity_pledge_holder_del_1_company")
      ))
    )

    , SummaryArgs(table_name = "company_equity_pledge_holder"
      , companyIdField = "holder_id"
      , distinctField = "rowkey,holder_id"
      , where = "holder_id is not null and length(holder_id) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_equity_pledge_holder_del_0_holder")
        , ("1", s"company_equity_pledge_holder_del_1_holder")
      ))
    )
    , SummaryArgs(table_name = "company_equity_pledge_holder"
      , companyIdField = "pledgee_id"
      , distinctField = "rowkey,pledgee_id"
      , where = "pledgee_id is not null and length(pledgee_id) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_equity_pledge_holder_del_0_pledgee")
        , ("1", s"company_equity_pledge_holder_del_1_pledgee")
      ))
    )
    , SummaryArgs(table_name = "finance_info_v3"
      , companyIdField = "company_id"
      , where = "vendinc_dis = '1' "
      , groupByInfo = GroupByInfoNg(field = "vendinc_dis", value_alias = Seq(
        ("1", s"finance_info_v3_dis_1")
        , ("0", s"finance_info_v3_dis_0")
      ))
    )

    , SummaryArgs(table_name = "wenshu_detail_v2_explode"
      , companyIdField = "plaintiff_info_id_explode"
      , distinctField = "rowkey,plaintiff_info_id_explode"
      , where = "plaintiff_info_id_explode is not null and length(plaintiff_info_id_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"wenshu_detail_v2_del_0_plaintiff")
        , ("1", s"wenshu_detail_v2_del_1_plaintiff")
      ))
    )

    , SummaryArgs(table_name = "wenshu_detail_v2_explode"
      , companyIdField = "defendant_info_id_explode"
      , distinctField = "rowkey,defendant_info_id_explode"
      , where = "defendant_info_id_explode is not null and length(defendant_info_id_explode) = 32 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"wenshu_detail_v2_del_0_defendant")
        , ("1", s"wenshu_detail_v2_del_1_defendant")
      ))
    )

  )


  def main(args: Array[String]): Unit = {

    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_ng",
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "10000"
    )

    val spark = SparkUtils.InitEnv(getClass.getSimpleName, config)
    WinhcNgSummary_new(s = spark, project = "winhc_ng", target_tab = "winhc_ng.out_es_summary", args = start_args)
      .calc()
    spark.stop()
  }
}
