package com.winhc.bigdata.spark.ng.utils

import com.winhc.bigdata.spark.utils.SparkUtils

import scala.collection.mutable

/**
 * @Author: XuJiakai
 * @Date: 2020/10/10 10:23
 * @Description: winhc_ng空间下摘要增强版
 */
object PersonSummaryNg_new {

  private def get_default_summary_args(tableName: String, person_id: String): SummaryArgs = {
    SummaryArgs(
      table_name = tableName
      , companyIdField = person_id
      , where = s"$person_id is not null and length($person_id) = 33 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"${tableName}_del_0")
        , ("1", s"${tableName}_del_1")
      ))
    )
  }

  val start_args = Seq(
    get_default_summary_args("company_zxr", "keyno")
    , get_default_summary_args("company_dishonest_info", "keyno")
    , get_default_summary_args("company_zxr_restrict", "pid")
    //    , get_default_summary_args("company_judicial_assistance", "executed_person_id")
    , get_default_summary_args("restrictions_on_exit", "limited_person_pid")
    , get_default_summary_args("company_zxr_final_case", "keyno")
    , get_default_summary_args("zxr_evaluate_results", "keyno")


    , SummaryArgs(table_name = "company_judicial_assistance"
      , companyIdField = "executed_person_id"
      , where = "executed_person_id is not null and length(executed_person_id) = 33 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_judicial_assistance_del_0_executed_person")
        , ("1", s"company_judicial_assistance_del_1_executed_person")
      ))
    )


    , SummaryArgs(table_name = "company_court_open_announcement_explode"
      , companyIdField = "plaintiff_info_id_explode"
      , distinctField = "rowkey,plaintiff_info_id_explode"
      , where = "plaintiff_info_id_explode is not null and length(plaintiff_info_id_explode) = 33"
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_court_open_announcement_del_0_plaintiff")
        , ("1", s"company_court_open_announcement_del_1_plaintiff")
      ))
    )
    , SummaryArgs(table_name = "company_court_open_announcement_explode"
      , companyIdField = "defendant_info_id_explode"
      , distinctField = "rowkey,defendant_info_id_explode"
      , where = "defendant_info_id_explode is not null and length(defendant_info_id_explode) = 33 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_court_open_announcement_del_0_defendant")
        , ("1", s"company_court_open_announcement_del_1_defendant")
      ))
    )


    , SummaryArgs(table_name = "company_lawsuit_explode"
      , companyIdField = "plaintiff_info_id_explode"
      , distinctField = "rowkey,plaintiff_info_id_explode"
      , where = "plaintiff_info_id_explode is not null and length(plaintiff_info_id_explode) = 33 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_lawsuit_del_0_plaintiff")
        , ("1", s"company_lawsuit_del_1_plaintiff")
      ))
    )
    , SummaryArgs(table_name = "company_lawsuit_explode"
      , companyIdField = "defendant_info_id_explode"
      , distinctField = "rowkey,defendant_info_id_explode"
      , where = "defendant_info_id_explode is not null and length(defendant_info_id_explode) = 33 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_lawsuit_del_0_defendant")
        , ("1", s"company_lawsuit_del_1_defendant")
      ))
    )


    , SummaryArgs(table_name = "company_court_announcement_explode"
      , companyIdField = "plaintiff_info_id_explode"
      , distinctField = "rowkey,plaintiff_info_id_explode"
      , where = "plaintiff_info_id_explode is not null and length(plaintiff_info_id_explode) = 33"
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_court_announcement_del_0_plaintiff")
        , ("1", s"company_court_announcement_del_1_plaintiff")
      ))
    )
    , SummaryArgs(table_name = "company_court_announcement_explode"
      , companyIdField = "litigant_info_id_explode"
      , distinctField = "rowkey,litigant_info_id_explode"
      , where = "litigant_info_id_explode is not null and length(litigant_info_id_explode) = 33"
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_court_announcement_del_0_litigant")
        , ("1", s"company_court_announcement_del_1_litigant")
      ))
    )

    , SummaryArgs(table_name = "company_court_register_explode"
      , companyIdField = "plaintiff_info_id_explode"
      , distinctField = "rowkey,plaintiff_info_id_explode"
      , where = "plaintiff_info_id_explode is not null and length(plaintiff_info_id_explode) = 33 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_court_register_del_0_plaintiff")
        , ("1", s"company_court_register_del_1_plaintiff")
      ))
    )

    , SummaryArgs(table_name = "company_court_register_explode"
      , companyIdField = "defendant_info_id_explode"
      , distinctField = "rowkey,defendant_info_id_explode"
      , where = "defendant_info_id_explode is not null and length(defendant_info_id_explode) = 33 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_court_register_del_0_defendant")
        , ("1", s"company_court_register_del_1_defendant")
      ))
    )
    , SummaryArgs(table_name = "company_send_announcement_explode"
      , companyIdField = "plaintiff_info_id_explode"
      , distinctField = "rowkey,plaintiff_info_id_explode"
      , where = "plaintiff_info_id_explode is not null and length(plaintiff_info_id_explode) = 33 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_send_announcement_del_0_plaintiff")
        , ("1", s"company_send_announcement_del_1_plaintiff")
      ))
    )
    , SummaryArgs(table_name = "company_send_announcement_explode"
      , companyIdField = "defendant_info_id_explode"
      , distinctField = "rowkey,defendant_info_id_explode"
      , where = "defendant_info_id_explode is not null and length(defendant_info_id_explode) = 33 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_send_announcement_del_0_defendant")
        , ("1", s"company_send_announcement_del_1_defendant")
      ))
    )


    , SummaryArgs(table_name = "company_equity_info_explode"
      , companyIdField = "pledgor_keyno_explode"
      , distinctField = "rowkey,pledgor_keyno_explode"
      , where = "pledgor_keyno_explode is not null and length(pledgor_keyno_explode) = 33 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_equity_info_del_0_pledgor")
        , ("1", s"company_equity_info_del_1_pledgor")
      ))
    )
    , SummaryArgs(table_name = "company_equity_info_explode"
      , companyIdField = "pledgee_keyno_explode"
      , distinctField = "rowkey,pledgee_keyno_explode"
      , where = "pledgee_keyno_explode is not null and length(pledgee_keyno_explode) = 33 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_equity_info_del_0_pledgee")
        , ("1", s"company_equity_info_del_1_pledgee")
      ))
    )
    , SummaryArgs(table_name = "company_equity_pledge_holder"
      , companyIdField = "holder_id"
      , distinctField = "rowkey,holder_id"
      , where = "holder_id is not null and length(holder_id) = 33 "
      , groupByInfo = GroupByInfoNg(field = "deleted", value_alias = Seq(
        ("0", s"company_equity_pledge_holder_del_0_holder")
        , ("1", s"company_equity_pledge_holder_del_1_holder")
      ))
    )

  )


  def main(args: Array[String]): Unit = {
    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_ng",
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "10000"
    )

    val spark = SparkUtils.InitEnv(getClass.getSimpleName, config)
    WinhcNgSummary_new(s = spark, project = "winhc_ng", target_tab = "winhc_ng.out_es_summary_person", args = start_args).calc()
    spark.stop()
  }
}
