package com.winhc.bigdata.spark.ng.utils

import com.winhc.bigdata.spark.ng.jobs.{args_company_job, general_handler}
import com.winhc.bigdata.spark.utils.{ElasticSearchIndexUtils, LoggingUtils, SparkUtils}
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @author: XuJiakai
 * @date: 2021/4/29 10:53
 */
case class ReimportSummary(s: SparkSession
                           , tabs: Seq[String]
                          ) extends LoggingUtils {
  @(transient@getter) val spark: SparkSession = s

  private lazy val es = ElasticSearchIndexUtils()


  private def re_explode(tn: String): Unit = {
    val ads_explode = s"winhc_ng.ads_${tn}_explode"
    val inc_ads_explode = s"winhc_ng.inc_ads_${tn}_explode"

    dropAllPartitions(ads_explode)
    dropAllPartitions(inc_ads_explode)


    general_handler(s = spark
      , project = "winhc_ng"
      , job_args = args_company_job.get_args_company_job(tn))
      .explode_calc()
  }

  def calc(ds: String): Unit = {
    for (e <- tabs) {
      re_explode(e)
    }

    dropAllPartitions("winhc_ng.out_es_summary")
    dropAllPartitions("winhc_ng.out_es_summary_person")

    create_index(ds)
  }


  def create_index(ds: String): Unit = {
    val template =
      s"""
         |{
         |  "mappings": {
         |    "_doc": {
         |      "properties": {
         |      }
         |    }
         |  },
         |  "settings": {
         |    "number_of_shards": 3,
         |    "number_of_replicas": 0,
         |    "index": {
         |      "mapping": {
         |        "nested_fields": {
         |          "limit": "300"
         |        }
         |      }
         |    }
         |  }
         |}
         |""".stripMargin

    es.put(s"out_es_summary_person_v$ds", template)
    es.put(s"out_es_summary_v$ds", template)
  }

}

object ReimportSummary {
  def main(args: Array[String]): Unit = {
    val Array(ds) = args
    val project = "winhc_ng"
    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> project,
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "100"
    )
    val spark = SparkUtils.InitEnv(this.getClass.getSimpleName + ":reimport", config)
    val tns = Seq(
      "company_court_open_announcement"
      ,"company_court_announcement"
      ,"company_send_announcement"
      ,"company_court_register"
    )
    ReimportSummary(spark, tns).calc(ds)
    spark.stop()
  }
}
