package com.winhc.bigdata.spark.ng.utils

import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, SparkUtils}
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @author: XuJiakai
 * @date: 2021/2/27 09:23
 */

case class StartAndEndDsArgs(inc: Boolean // true 写出到增量表或只读增量表
                             , target_ds: String
                             , inc_tab_gt_ds: String
                            )

case class StartAndEndDsUtils(s: SparkSession
                             ) extends LoggingUtils with BaseFunc{
  @(transient@getter) val spark: SparkSession = s


  /**
   * 两个表写入两个表中
   *
   * @param org_tab
   * @param inc_org_tab
   * @param target_tab
   * @param inc_target_tab
   */
  def get_start_and_end_args(org_tab: String, inc_org_tab: String, target_tab: String, inc_target_tab: String): StartAndEndDsArgs = {

    def inc(): StartAndEndDsArgs = {
      var gt_ds = ""
      var target_ds = ""


      val inc_org_tab_ds = getLastPartitionsOrElse(inc_org_tab, null)
      val inc_target_tab_ds = getLastPartitionsOrElse(inc_target_tab, null)
      val target_tab_ds = getLastPartitionsOrElse(target_tab, null)
      if (inc_org_tab_ds == null) {
        return all(org_tab, inc_org_tab)
      }

      target_ds = inc_org_tab_ds

      if (inc_target_tab_ds == null) {
        gt_ds = target_ds
      } else {
        gt_ds = inc_target_tab_ds
      }

      if (gt_ds.equals(target_ds)) {
        val inc_target_sec_ds = getSecondLastPartitionOrElse(inc_target_tab, null)
        if (inc_target_sec_ds == null) {
          gt_ds = target_tab_ds
        } else {
          gt_ds = inc_target_sec_ds
        }
      }
      StartAndEndDsArgs(true, target_ds = target_ds, gt_ds)
    }

    val ads_last_ds = getLastPartitionsOrElse(target_tab, null)
    if (ads_last_ds == null)
      all(org_tab, inc_org_tab)
    else
      inc()
  }


  private def all(org_tab: String, inc_org_tab: String): StartAndEndDsArgs = {
    val target_ds = getLastPartitionsOrElse(inc_org_tab, getLastPartitionsOrElse(org_tab, null))
    if (target_ds == null) {
      throw new RuntimeException("全量来源表为空 !")
    }
    StartAndEndDsArgs(false, target_ds = target_ds, null)
  }


  /**
   * 两个表写入到一个表中
   *
   * @param org_tab
   * @param inc_org_tab
   * @param target_tab
   */
  def get_start_and_end_args(org_tab: String, inc_org_tab: String, target_tab: String): StartAndEndDsArgs = {

    def inc(): StartAndEndDsArgs = {
      var gt_ds = ""
      var target_ds = ""


      val inc_org_tab_ds = getLastPartitionsOrElse(inc_org_tab, null)
      val target_tab_ds = getLastPartitionsOrElse(target_tab, null)
      if (inc_org_tab_ds == null) {
        return all(org_tab, inc_org_tab)
      }

      target_ds = inc_org_tab_ds
      gt_ds = target_tab_ds

      if (gt_ds.equals(target_ds)) {
        val target_second_ds = getSecondLastPartitionOrElse(target_tab, null)
        if (target_second_ds == null) {
          return all(org_tab, inc_org_tab)
        } else {
          gt_ds = target_second_ds
        }
      }
      StartAndEndDsArgs(true, target_ds = target_ds, gt_ds)
    }

    val targetLastDs = getLastPartitionsOrElse(target_tab, null)
    if (targetLastDs == null) {
      all(org_tab, inc_org_tab)
    } else {
      inc()
    }
  }


  def get_gt_ds(org_tab: String, target_tab: String): StartAndEndDsArgs = {
    val org_tab_ds = getLastPartitionsOrElse(org_tab, BaseUtil.getYesterday())

    val target_ds = getLastPartitionsOrElse(target_tab, null)
    if (target_ds == null) {
      return StartAndEndDsArgs(inc = false, target_ds = org_tab_ds, inc_tab_gt_ds = null)
    }

    if (target_ds.equals(org_tab_ds)) {
      val target_second_ds = getSecondLastPartitionOrElse(target_tab, null)
      if (target_second_ds == null) {
        StartAndEndDsArgs(inc = false, target_ds = org_tab_ds, inc_tab_gt_ds = null)
      } else {
        StartAndEndDsArgs(inc = true, target_ds = org_tab_ds, inc_tab_gt_ds = target_second_ds)
      }
    } else {
      StartAndEndDsArgs(inc = true, target_ds = org_tab_ds, inc_tab_gt_ds = target_ds)
    }
  }
}


object StartAndEndDsUtils {
  def main(args: Array[String]): Unit = {
    val project = "winhc_ng"
    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> project,
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "100"
    )
    val tn = "company_court_open_announcement"
    val spark = SparkUtils.InitEnv(this.getClass.getSimpleName + ":" + tn, config)


    val org_tab: String = s"winhc_ng.ads_$tn"
    val inc_org_tab: String = s"winhc_ng.inc_ads_$tn"
    val target_tab: String = s"winhc_ng.ads_${tn}_explode"
    val inc_target_tab: String = s"winhc_ng.inc_ads_${tn}_explode"


    val a = StartAndEndDsUtils(spark)

    /* val b = a.get_start_and_end_args(org_tab, inc_org_tab, target_tab)
     println(b)*/

   /* val c = a.get_start_and_end_args("winhc_ng.ads_company_dishonest_info", "winhc_ng.inc_ads_company_dishonest_info", "winhc_ng.bds_credit_punishment_data_extraction")
    println(c)*/


    val d = a.get_gt_ds("winhc_ng.bds_credit_punishment_data_extraction", "winhc_ng.bds_credit_punishment_case_info")
    println(d)
    spark.stop()
  }
}
