package com.winhc.bigdata.spark.ng.utils

import com.winhc.bigdata.spark.implicits.CaseClass2JsonHelper._
import com.winhc.bigdata.spark.utils.{Alias, ElasticSearchIndexUtils, LoggingUtils, SparkUtils}
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @author: XuJiakai
 * @date: 2021/4/29 16:31
 */

case class SwitchSummaryIndex(s: SparkSession
                             ) extends LoggingUtils {
  @(transient@getter) val spark: SparkSession = s

  private lazy val es = ElasticSearchIndexUtils()

  private val company_summary_alias = "company_summary"
  private val person_summary_alias = "company_summary_person"

  def switch(ds: String): Unit = {
    val company_summary = "out_es_summary_v" + ds
    val person_summary = "out_es_summary_person_v" + ds
    val allIndices = es.catIndices()

    val size = allIndices.count(a => a.equals(company_summary) || a.equals(person_summary))
    if (size != 2) {
      throw new RuntimeException("target indices is not found !")
    }

    val ali: Seq[Alias] = es.catAliases()
      .filter(a => a.alias.equals(company_summary_alias) || a.alias.equals(person_summary_alias))

    val add_alias = Seq(
      Alias(alias = company_summary_alias, index = company_summary)
      , Alias(alias = person_summary_alias, index = person_summary)
    )
    println(
      s"""
         |remove alias: ${ali.toJson()}
         |add alias: ${add_alias.toJson()}
         |""".stripMargin)

    es.updateAliases(add_alias, ali)

    deletedOtherIndices(allIndices, add_alias, ali)
  }

  def deletedOtherIndices(allIndices: Seq[String], add_alias: Seq[Alias], remove: Seq[Alias]): Unit = {
    val index = add_alias.map(_.index) ++ remove.map(_.index)
    val other = allIndices.filter(i => i.startsWith("out_es_summary_")).filterNot(i => index.contains(i))
    println(
      s"""
         |
         |deleted index: ${other.mkString(",")}
         |""".stripMargin)

    if(other.size==2){
      for (elem <- other) {
        es.delete(elem)
      }
    }else{
      throw new RuntimeException("删除的目标索引大于两个，请手动排查！！！")
    }
  }
}


object SwitchSummaryIndex {
  def main(args: Array[String]): Unit = {
    val Array(ds) = args
    val project = "winhc_ng"
    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> project,
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "100"
    )
    val spark = SparkUtils.InitEnv(this.getClass.getSimpleName, config)

    SwitchSummaryIndex(spark).switch(ds)

    spark.stop()
  }
}
