package com.winhc.bigdata.spark.ng.utils

import com.alibaba.fastjson.{JSON, JSONArray, JSONPath}
import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.BaseUtil.is_json_str
import com.winhc.bigdata.spark.utils.{LoggingUtils, SparkUtils}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @author: XuJiakai
 * @date: 2021/2/24 10:18
 *
 *        多表炸开
 */
case class explode_args(org_field: String, json_path: String, alias: String, is_filter_null: Boolean = false)

case class explode_tab(s: SparkSession,
                       org_table: String, //需要炸开的来源表
                       args: Seq[explode_args]
                      ) extends LoggingUtils with BaseFunc {
  @(transient@getter) val spark: SparkSession = s

  init()


  private def init(): Unit = {
    /**
     *
     * @param json_array
     * @param json_path "$.name"
     * @return
     */
    def json_2_array(json_array: String, json_path: String): Seq[String] = {
      try {
        if (StringUtils.isEmpty(json_array)) {
          return Seq.empty
        }
        if (!is_json_str(json_array)) {
          return Seq.empty
        }
        JSONPath.eval(JSON.parse(json_array), json_path).asInstanceOf[JSONArray].toArray[String](Array()).toSeq.distinct.diff(Seq(""))
      } catch {
        case e: Exception => {
          println(json_array)
          Seq.empty
        }
      }
    }

    spark.udf.register("json_2_array", json_2_array _)
  }


  def calc(tmpView: String): Unit = {
    val explode_args = args.map(f => {
      if (f.is_filter_null) {
        //过滤到null
        s"LATERAL VIEW explode(json_2_array(${f.org_field},'${f.json_path}')) t_${f.org_field} AS ${f.alias}"
      } else {
        //保留null值
        s"LATERAL VIEW OUTER explode(json_2_array(${f.org_field},'${f.json_path}')) t_${f.org_field} AS ${f.alias}"
      }
    }).mkString("\n")

    sql(
      s"""
         | SELECT  *
         | FROM    $org_table
         | $explode_args
         |""".stripMargin)
      .createTempView(tmpView)
  }
}

object explode_tab {
  def main(args: Array[String]): Unit = {
    val config = EsConfig.getEsConfigMap ++ mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_ng",
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "10"
    )
    val spark = SparkUtils.InitEnv(this.getClass.getName, config)

    val ags = Seq(
      explode_args("plaintiff_info", "$.litigant_id", "plaintiff_info_id_explode")
      , explode_args("defendant_info", "$.litigant_id", "defendant_info_id_explode")
    )

    explode_tab(spark, "winhc_ng.ads_company_court_open_announcement", ags).calc("abc")
    spark.sql("select * from abc").show()
    spark.stop()
  }
}
