package com.winhc.bigdata.spark.ng.utils

import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.{LoggingUtils, SparkUtils}
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @author: XuJiakai
 * @date: 2021/2/23 16:20
 *
 *        多表合并
 */
case class merge_table(s: SparkSession,
                       tables: Seq[String], //需要合并的表名
                       join_key: String
                      ) extends LoggingUtils with BaseFunc {
  @(transient@getter) val spark: SparkSession = s
  private val tmp_tab_prefix = "winhc_ng.tmp_"


  private def init(): Unit = {
    if (tables.length <= 1) {
      throw new RuntimeException(s"table.length = ${tables.length},Unable to merge the table !")
    }

    val tab_cols = tables.map(getColumns)
    var int_cols = tab_cols.head

    for (i <- 0 until tab_cols.length - 1) {
      val table_2 = tab_cols(i + 1)
      int_cols = int_cols.intersect(table_2)
    }

    if (int_cols.size != 1 || !int_cols.head.equals(join_key)) {
      throw new RuntimeException("Unable to merge the table !")
    }

    for (elem <- tables) {
      sql(
        s"""
           |DROP TABLE IF EXISTS $tmp_tab_prefix$elem
           |""".stripMargin)

      sql(
        s"""
           |CREATE TABLE IF NOT EXISTS $tmp_tab_prefix$elem
           |(
           |    company_id  STRING COMMENT 'company id'
           |    ,${getColumns(elem).diff(Seq("company_id")).map(s => s"$s bigint COMMENT 'v'").mkString(",")}
           |)
           |COMMENT '$elem'
           |""".stripMargin)

      sql(
        s"""
           |INSERT OVERWRITE TABLE $tmp_tab_prefix$elem
           |SELECT ${getColumns(s"$tmp_tab_prefix$elem").mkString(",")}
           |FROM
           |    $elem
           |""".stripMargin)

    }
  }

  def calc(tempView: String): Unit = {
    init()

    var df = sql(
      s"""
         |select * from $tmp_tab_prefix${tables.head}
         |""".stripMargin).cache()

    for (i <- 0 until tables.length - 1) {
      val df_2 = sql(
        s"""
           |select * from $tmp_tab_prefix${tables(i + 1)}
           |""".stripMargin)
      df = df.join(df_2, Seq(join_key), "full").cache()
    }
    df.createTempView(tempView)
  }


  def drop(): Unit = {
    for (elem <- tables) {
      sql(
        s"""
           |DROP TABLE IF EXISTS $tmp_tab_prefix$elem
           |""".stripMargin)
    }
  }
}

object merge_table {

  def main(args: Array[String]): Unit = {
    val config = EsConfig.getEsConfigMap ++ mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_ng",
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "100"
    )
    val spark = SparkUtils.InitEnv("inc_compaaaaaany_equity_info", config)

    spark.sql("select 2 as id,'v' as v").createTempView("a")
    spark.sql("select 1 as id,'v2' as v2").createTempView("b")
    spark.sql("select 1 as id,'v3' as v3").createTempView("c")
    merge_table(spark, Seq("a", "b", "c"), "id").calc("abc")
    spark.sql("select * from abc").show()
    spark.stop()
  }
}
