package com.winhc.bigdata.spark.test

import java.io.{File, PrintWriter}

import com.winhc.bigdata.spark.utils.SparkUtils

import scala.collection.mutable

/**
 * @Author: XuJiakai
 * @Date: 2020/6/30 09:55
 * @Description:
 */
object ExportSql {


  def getSql(tableName: String, cols: Seq[String]): String = {
    val cs = cols.map(f => {
      if ("deleted".equals(f)) {
        """
          |CASE _update_type    WHEN "DELETE" THEN 1
          |                             ELSE deleted
          |                     END as deleted
          |""".stripMargin
      } else {
        f
      }
    }).seq

    val temp =
      s"""
         |INSERT OVERWRITE TABLE winhc_eci_dev.${tableName} PARTITION(ds)
         |SELECT  ${cs.mkString(",")}
         |FROM    winhc_eci_dev.$tableName
         |WHERE   ds > 0
         |;
         |""".stripMargin
    temp
  }

  def getCols(catalog: org.apache.spark.sql.catalog.Catalog, tableName: String): Seq[String] = {
    catalog.listColumns(tableName).collect().map(_.name).seq
  }


  def main(args: Array[String]): Unit = {
    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_eci_dev",
      "spark.hadoop.odps.spark.local.partition.amt" -> "10"
    )
    val spark = SparkUtils.InitEnv("exportSql", config)

    val catalog = spark.catalog
    val tables = catalog.listTables.collect().map(_.name).filter(_.startsWith("inc_ods")).seq


    val allSql = tables.map(t => {
      getSql(t, getCols(catalog, t))
    }).seq.mkString("")

    val writer = new PrintWriter(new File("C:\\Users\\x\\Desktop\\公司\\all_sql.txt"))

    writer.write(allSql)
    writer.close()

    spark.stop()
  }
}
