package com.winhc.bigdata.spark.test

import com.winhc.bigdata.spark.config.HBaseConfig
import com.winhc.bigdata.spark.utils.{CompanyNameMappingUtil, SparkUtils}
import org.apache.hadoop.hbase.TableName
import org.apache.hadoop.hbase.spark.HBaseContext
import org.apache.hadoop.hbase.spark.HBaseRDDFunctions._
import org.apache.spark.sql.Row
import org.apache.spark.sql.types.{StringType, StructField, StructType}

/**
 * @Author: XuJiakai
 * @Date: 2020/6/10 17:09
 * @Description:
 */
object TestSpark4Hbase {
  def main(args: Array[String]): Unit = {
    val spark = SparkUtils.InitEnv("4hbase")
    import spark.implicits._
    val hbaseKVTable = "company_name_kv"
    val jobConf = HBaseConfig.HBaseOutputJobConf(hbaseKVTable)
    val hbaseContext = new HBaseContext(spark.sparkContext, jobConf)
    val rdd = spark.createDataset(Seq("3190263436", "3190295172", "3190295172")).rdd.hbaseMapPartitions(hbaseContext, (f, con) => {
      val table = con.getTable(TableName.valueOf(hbaseKVTable))
      val rdd_par = f.map(record => {
        try {
          val cid = record
          val (res_cid, res_name) = CompanyNameMappingUtil.getCurrentIdAndName(table, cid)
          Row(cid, null, null, res_cid, res_name)
        } catch {
          case e: Exception => {
            println(record.toString())
            println(e.getMessage, e)
          }
            null
        }
      })
      rdd_par
    }).filter(_ != null)
    val schema = StructType(Array(
      StructField("cid", StringType),
      StructField("cname", StringType),
      StructField("current_cid", StringType),
      StructField("new_cid", StringType),
      StructField("new_cname", StringType)))
    spark.createDataFrame(rdd, schema).show()
    spark.stop()

  }

}
