package com.winhc.bigdata.spark.udf

import com.winhc.bigdata.spark.utils.BaseUtil
import com.winhc.bigdata.spark.utils.BaseUtil.{json_array, list_json}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.Row
import org.apache.spark.sql.expressions.{MutableAggregationBuffer, UserDefinedAggregateFunction}
import org.apache.spark.sql.types._

import scala.collection.mutable

/**
 *
 * @Description:人名合并
 * @author π
 * @date 2020/10/26 15:15
 */

class AllNamePlus(max: Int) extends UserDefinedAggregateFunction {

  //val flags = Seq("0", "1", "2", "4", "8")
  val split = "\u0001"
  val empty_col = "[]"

  override def inputSchema: StructType = StructType(Array[StructField](
    StructField("all_name", DataTypes.StringType)
  ))

  override def bufferSchema: StructType = StructType(
    Array[StructField](
      StructField("t1", DataTypes.StringType)
    )
  )

  override def dataType: DataType = DataTypes.StringType

  override def deterministic: Boolean = true

  override def initialize(buffer: MutableAggregationBuffer): Unit = {
    buffer.update(0, "[]")
  }

  override def update(buffer: MutableAggregationBuffer, input: Row): Unit = {
    if (buffer.size >= max) {
      return
    }
    if (input.size != 1) return
    var all_name = input.getAs[String](0)
    all_name = BaseUtil.filter_json(all_name, "name")
    if (empty_col.equals(all_name)) {
      return
    }
    val buf_name: String = buffer.getAs[String](0)
    buffer(0) = list_json(json_array(buf_name) ++ json_array(all_name))

  }

  override def merge(buffer1: MutableAggregationBuffer, buffer2: Row): Unit = {
    if (buffer1.size >= max) {
      return
    }
    update(buffer1, buffer2)
  }

  override def evaluate(buffer: Row): Any = {
    var all_name = "[]"
    val tmp_name = buffer.getAs[String](0)

    if (StringUtils.isNotBlank(tmp_name)) all_name = tmp_name
    all_name
  }

}
