package com.winhc.bigdata.spark.udf

import com.winhc.bigdata.spark.utils.BaseUtil
import com.winhc.bigdata.spark.utils.BaseUtil.{json_array, list_json}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.Row
import org.apache.spark.sql.expressions.{MutableAggregationBuffer, UserDefinedAggregateFunction}
import org.apache.spark.sql.types._

/**
 *
 * @Description:人名合并
 * @author π
 * @date 2020/10/26 15:15
 */

class AllNamePlusV2(max: Int) extends UserDefinedAggregateFunction {

  //val flags = Seq("0", "1", "2", "4", "8")
  val split = "\u0001"
  val empty_col = "[]"

  override def inputSchema: StructType = StructType(Array[StructField](
    StructField("all_name", DataTypes.StringType)
  ))

  override def bufferSchema: StructType = StructType(
    Array[StructField](
      StructField("t1", DataTypes.createArrayType(DataTypes.StringType))
    )
  )

  override def dataType: DataType = DataTypes.StringType

  override def deterministic: Boolean = true

  override def initialize(buffer: MutableAggregationBuffer): Unit = {
    buffer.update(0, Seq.empty)
  }

  override def update(buffer: MutableAggregationBuffer, input: Row): Unit = {
    val buf_name = buffer.getSeq[String](0)
    if (buf_name.size >= max) {
      return
    }
    if (input.size != 1) return
    var all_name = input.getAs[String](0)
    all_name = BaseUtil.filter_json(all_name, "name")
    if (empty_col.equals(all_name)) {
      return
    }
    buffer(0) = buf_name ++ Seq(all_name)
  }

  override def merge(buffer1: MutableAggregationBuffer, buffer2: Row): Unit = {

    val buf_name = buffer1.getSeq[String](0)
    if (buf_name.size >= max) {
      return
    }
    val all_name = buffer2.getSeq[String](0)
    buffer1(0) = buf_name ++ all_name
  }

  override def evaluate(buffer: Row): Any = {
    val tmp_name = buffer.getSeq[String](0)
    if (tmp_name == null || tmp_name.isEmpty) return "[]"
    val list = tmp_name.map(x => {
      json_array(x)
    }).reduce(_ ++ _)
    list_json(list)
  }

}
