package com.winhc.bigdata.spark.udf

import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.Row
import org.apache.spark.sql.expressions.{MutableAggregationBuffer, UserDefinedAggregateFunction}
import org.apache.spark.sql.types._

import scala.collection.mutable

/**
 * 1、如果有被执行人，那么取被执行人的执行标的额
 * 2、如果有终本案件，取终本案件的执行标的
 * 3、如果有裁判文书取裁判文书的金额（取一审）
 *
 * @Description:案件金额聚合
 * @author π
 * @date 2020/10/26 15:15
 */

class CaseAmtAggs(max: Int) extends UserDefinedAggregateFunction {

  //val flags = Seq("0", "1", "2", "4", "8")
  val split = "\u0001"

  override def inputSchema: StructType = StructType(Array[StructField](
    StructField("case_amt", DataTypes.DoubleType)
    , StructField("bus_date", DataTypes.StringType)
    , StructField("flag", DataTypes.StringType)
  ))

  override def bufferSchema: StructType = StructType(
    Array[StructField](
      StructField("t1", DataTypes.createMapType(DataTypes.StringType, DataTypes.DoubleType))
    )
  )

  override def dataType: DataType = DataTypes.DoubleType

  override def deterministic: Boolean = true

  override def initialize(buffer: MutableAggregationBuffer): Unit = {
    buffer.update(0, Map[String, Double]())
  }

  override def update(buffer: MutableAggregationBuffer, input: Row): Unit = {
    if (buffer.size >= max) {
      return
    }
    //val case_amt: Double = input.getDouble(0)
    //val bus_date = input.getString(1)
    //val flag = input.getString(2)
    if(input.size != 3) return
    val case_amt: Double = input.getAs[Double](0)
    val bus_date = input.getAs[String](1)
    val flag = input.getAs[String](2)

    if (case_amt == 0 || bus_date == null || flag == null) {
      return
    }
    //    if (!flags.contains(flag)) {
    //      return
    //    }

    buffer(0) = mutable.Map(bus_date + split + flag -> case_amt) ++= scala.collection.mutable.Map[String, Double](buffer.getMap[String, Double](0).toMap.toSeq: _*)
  }

  override def merge(buffer1: MutableAggregationBuffer, buffer2: Row): Unit = {
    if (buffer1.size >= max) {
      return
    }
    buffer1(0) = buffer1.getAs[Map[String, Double]](0) ++ buffer2.getAs[Map[String, Double]](0)
  }

  override def evaluate(buffer: Row): Any = {
    var case_amt: Double = 0d
    val m0: Map[String, Double] = buffer.getAs[Map[String, Double]](0)
    if (m0.isEmpty) {
      return case_amt
    } else {
      val keys = m0.keySet.toSeq
      val tup: Seq[(String, String, Double)] = keys.map(x => {
        val arr = x.split(split, -1)
        (arr(1), arr(0), m0(x))
      })
      val flags = Seq[String]("7", "6", "0")
      val seq = flags.map(x => {
        get_value(tup, x)
      }).filter(_ > 0)
      if (seq.size > 0) {
        case_amt = seq.head
      }
    }
    case_amt
  }

  def get_value(tup: Seq[(String, String, Double)], flag: String): Double = {
    var case_amt: Double = 0d
    val t1 = tup.filter(x => {
      x._1.equals(flag) && x._3 > 0
    })
    if (t1.size > 0) {
      val t2 = t1.map(x => x._2 -> x._3).toMap
      val key = t2.keySet.toSeq.sorted.head
      case_amt = t2(key)
    }
    case_amt
  }
}
