package com.winhc.bigdata.spark.udf

import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.Row
import org.apache.spark.sql.expressions.{MutableAggregationBuffer, UserDefinedAggregateFunction}
import org.apache.spark.sql.types._

import scala.collection.mutable

/**
 * 1、如果有被执行人，那么取被执行人的执行标的额
 * 2、如果有终本案件，取终本案件的执行标的
 * 3、如果有裁判文书取裁判文书的金额（取一审）
 *
 * @Description:案件金额聚合
 * @author π
 * @date 2020/10/26 15:15
 */

class CaseAmtAggsPlus(max: Int) extends UserDefinedAggregateFunction {

  //val flags = Seq("0", "1", "2", "4", "8")
  val split = "\u0001"

  override def inputSchema: StructType = StructType(Array[StructField](
    StructField("case_amt", DataTypes.StringType)
    , StructField("bus_date", DataTypes.StringType)
    , StructField("flag", DataTypes.StringType)
  ))

  override def bufferSchema: StructType = StructType(
    Array[StructField](
      StructField("t1", DataTypes.createMapType(DataTypes.StringType, DataTypes.DoubleType))
    )
  )

  override def dataType: DataType = DataTypes.createMapType(DataTypes.StringType, DataTypes.StringType)

  override def deterministic: Boolean = true

  override def initialize(buffer: MutableAggregationBuffer): Unit = {
    buffer.update(0, Map[String, Double]())
  }

  override def update(buffer: MutableAggregationBuffer, input: Row): Unit = {
    val tmp_buff = buffer.getAs[Map[String, Double]](0)

    if (buffer.size >= max) {
      return
    }
    if (input.size != 3) return
    val case_amt0 = input.getAs[String](0)
    val bus_date = input.getAs[String](1)
    val flag = input.getAs[String](2)

    if (StringUtils.isBlank(case_amt0) || StringUtils.isBlank(bus_date) || StringUtils.isBlank(flag) || case_amt0 == "0") {
      return
    }
    val case_amt = case_amt0.toDouble
    buffer(0) = mutable.Map(bus_date + split + flag -> case_amt) ++= scala.collection.mutable.Map[String, Double](tmp_buff.toSeq: _*)
  }

  override def merge(buffer1: MutableAggregationBuffer, buffer2: Row): Unit = {
    val tmp_buff = buffer1.getAs[Map[String, Double]](0)
    if (tmp_buff.size >= max) {
      return
    }
    buffer1(0) = tmp_buff ++ buffer2.getAs[Map[String, Double]](0)
  }

  override def evaluate(buffer: Row): Any = {
    var bus_date = ""
    var flag = ""
    var case_amt = ""
    val m0: Map[String, Double] = buffer.getAs[Map[String, Double]](0)
    if (m0.isEmpty) {
      return Map("flag" -> flag, "date" -> bus_date, "case_amt" -> case_amt)
    } else {
      val keys = m0.keySet.toSeq
      val tup: Seq[(String, String, Double)] = keys.map(x => {
        val arr = x.split(split, -1)
        (arr(1), arr(0), m0(x))
      })
      val flags = Seq[String]("7", "6", "0")
      val seq = flags.map(x => {
        get_value(tup, x)
      }).filter(_._3 != "0")
      if (seq.nonEmpty) {
        flag = seq.head._1
        bus_date = seq.head._2
        case_amt = seq.head._3
      }
    }
    Map("flag" -> flag, "date" -> bus_date, "case_amt" -> case_amt)
  }

  def get_value(tup: Seq[(String, String, Double)], flag: String) = {
    var bus_date: String = ""
    var case_amt:String  = ""
    val t1 = tup.filter(x => {
      x._1.equals(flag) && x._3 > 0
    })
    if (t1.nonEmpty) {
      val t2 = t1.map(x => x._2 -> x._3).toMap
      val key = t2.keySet.toSeq.min
      case_amt = t2(key).toString
      bus_date = key
    }
    (flag, bus_date, case_amt)
  }
}
