package com.winhc.bigdata.spark.udf

import com.winhc.bigdata.spark.utils.BaseUtil._
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.Row
import org.apache.spark.sql.expressions.{MutableAggregationBuffer, UserDefinedAggregateFunction}
import org.apache.spark.sql.types._

import scala.collection.mutable

/**
 * 取被执行维度，根据当事人聚合[{'name':'被執行人','exec_money':'执行金额（单位取万元）'}]，如果出现一个司法案件，同一个当事人被执行过多次，那么根据case_create_time取最早的一个
 *
 * @Description:执行金额聚合
 * @author π
 * @date 2021/08/16 15:15
 */
class CaseAmtAggsPlusV2(max: Int) extends UserDefinedAggregateFunction {

  val flags = Seq("7")
  val split = "\u0001"
  val empty_col = "[]"

  override def inputSchema: StructType = StructType(Array[StructField](
    StructField("exec_info", DataTypes.StringType)
  ))

  override def bufferSchema: StructType = StructType(
    Array[StructField](
      StructField("t1", DataTypes.createArrayType(DataTypes.StringType))
    )
  )

  override def dataType: DataType = DataTypes.StringType

  override def deterministic: Boolean = true

  override def initialize(buffer: MutableAggregationBuffer): Unit = {
    buffer.update(0, Seq.empty)
  }

  override def update(buffer: MutableAggregationBuffer, input: Row): Unit = {
    val buf_info = buffer.getSeq[String](0)
    if (buf_info.size >= max) {
      return
    }
    if (input.size != 1) return
    val input_info = input.getAs[String](0)
    if (empty_col.equals(input_info)) {
      return
    }
    buffer(0) = buf_info ++ Seq(input_info)
  }

  override def merge(buffer1: MutableAggregationBuffer, buffer2: Row): Unit = {

    val buf_info = buffer1.getSeq[String](0)
    if (buf_info.size >= max) {
      return
    }
    val input_info = buffer2.getSeq[String](0)
    buffer1(0) = buf_info ++ input_info
  }

  override def evaluate(buffer: Row): Any = {
    val tmp_name: Seq[String] = buffer.getSeq[String](0)
    //println("tmp_name|  " + tmp_name)
    if (tmp_name == null || tmp_name.isEmpty) return "[]"
    val list = tmp_name.map(x => {
      json_array(x)
    }).reduce(_ ++ _)
    //println("list|  " + list)
    merge_exec_json(list)
  }

}
