package com.winhc.bigdata.spark.udf

import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.Row
import org.apache.spark.sql.expressions.{MutableAggregationBuffer, UserDefinedAggregateFunction}
import org.apache.spark.sql.types._
import scala.collection.mutable

/**
 * @Description:案由聚合
 * @author π
 * @date 2020/10/26 15:15
 */

class CaseReasonAggs(max: Int) extends UserDefinedAggregateFunction {

  //val flags = Seq("0", "1", "2", "4", "8")
  val split = "\u0001"

  override def inputSchema: StructType = StructType(Array[StructField](
    StructField("case_reason", DataTypes.StringType)
    , StructField("bus_date", DataTypes.StringType)
    , StructField("flag", DataTypes.StringType)
  ))

  override def bufferSchema: StructType = StructType(
    Array[StructField](
      StructField("t1", DataTypes.createMapType(DataTypes.StringType, DataTypes.StringType))
    )
  )

  override def dataType: DataType = DataTypes.StringType

  override def deterministic: Boolean = true

  override def initialize(buffer: MutableAggregationBuffer): Unit = {
    buffer.update(0, Map[String, String]())
  }

  override def update(buffer: MutableAggregationBuffer, input: Row): Unit = {
    if (buffer.size >= max) {
      return
    }
    if(input.size != 3) return
    val case_reason = input.getAs[String](0)
    val bus_date = input.getAs[String](1)
    val flag = input.getAs[String](2)

    if (StringUtils.isBlank(case_reason) || StringUtils.isBlank(bus_date)) {
      return
    }
//    if (!flags.contains(flag)) {
//      return
//    }

    buffer(0) = mutable.Map(bus_date -> s"$case_reason") ++= scala.collection.mutable.Map[String, String](buffer.getMap[String, String](0).toMap.toSeq: _*)
  }

  override def merge(buffer1: MutableAggregationBuffer, buffer2: Row): Unit = {
    if (buffer1.size >= max) {
      return
    }
    buffer1(0) = buffer1.getAs[Map[String, String]](0) ++ buffer2.getAs[Map[String, String]](0)
  }

  override def evaluate(buffer: Row): Any = {
    var case_reason = ""
    val m0: Map[String, String] = buffer.getAs[Map[String, String]](0)
    if (m0.isEmpty) {
      return case_reason
    } else {
      val key = m0.keySet.toSeq.sorted.head
      case_reason = m0(key)
    }
    case_reason
  }
}
