package com.winhc.bigdata.spark.udf

import com.winhc.bigdata.calc.DimScore
import org.apache.commons.lang3.StringUtils
import org.apache.spark.broadcast.Broadcast
import org.apache.spark.sql.SparkSession
import com.winhc.bigdata.spark.utils.BaseUtil.{json_array, _}
import com.winhc.bigdata.spark.utils.RowkeyRuleUtils._

trait CompanyMapping {

  def prepareFunctions(spark: SparkSession): Unit = {
    import spark._
    //清理特殊字符
    spark.udf.register("cleanup", (col: String) => {
      cleanup(col)
    })

    spark.udf.register("replace_char", (col: String) => {
      replaceChar(col)
    })

    spark.udf.register("case_no", (col: String) => {
      caseNo(col)
    })

    spark.udf.register("rowkey_trans", (col: String, tab: String) => {
      rowkey_trans(col, tab)
    })

    spark.udf.register("name_judge", (name: String, yg_name: String, bg_name: String) => {
      nameJudge(name, yg_name, bg_name)
    })

    spark.udf.register("case_type", (case_no: String) => {
      caseType(case_no)
    })

    spark.udf.register("case_stage", (case_no: String) => {
      caseStage(case_no)
    })

    spark.udf.register("last_stage", (stages: String) => {
      lastStage(stages)
    })
    spark.udf.register("case_label", (l: String) => {
      label(l)
    })

    spark.udf.register("sort", (s: String, split: String) => {
      sortString(s, split)
    })

    spark.udf.register("trim_black", (s: String) => {
      trimBlack(s)
    })

    spark.udf.register("title", (yg: String, bg: String, reason: String) => {
      title(yg, bg, reason)
    })

    spark.udf.register("compare_name", (yg: String, bg: String) => {
      compareName(yg, bg)
    })

    spark.udf.register("name_cleanup", (name: String) => {
      nameCleanup(name)
    })

    spark.udf.register("split_names", (json_array: String, json_path: String) => {
      split_names(json_array, json_path)
    })

    spark.udf.register("json_array", (json: String) => {
      json_array(json)
    })

    spark.udf.register("merge_json", (yg_name: String, bg_name: String, all_name: String) => {
      merge_json(yg_name, bg_name, all_name)
    })

    spark.udf.register("all_name", (name: String) => {
      all_name(name)
    })

    spark.udf.register("flag_tn", (name: String) => {
      flag_tn(name)
    })

    spark.udf.register("tn_flag", (name: String) => {
      tn_flag(name)
    })

    spark.udf.register("add_pre", (id: String) => {
      add_pre(id)
    })

    spark.udf.register("remove_pre", (id: String) => {
      remove_pre(id)
    })

    spark.udf.register("combine_id", (rw1: String, rw2: String) => {
      combine_id(rw1, rw2)
    })

    spark.udf.register("adjust_reason", (reason: String) => {
      adjust_reason(reason)
    })
    spark.udf.register("amt_div", (amt1: String, amt2: String) => {
      amt_div(amt1, amt2)
    })

    spark.udf.register("amt_merge", (info:String) => {
      amt_merge(info)
    })


  }

  def prepare(spark: SparkSession): Unit = {
    import spark._
    //注册资本
    spark.udf.register("reg_capital_score", (capital: String) => {
      DimScore.registEvaluate(capital)
    })

    //实缴资本金额转换
    spark.udf.register("actual_capital_score", (actual_capital: String, reg_capital: String) => {
      DimScore.actualEvaluate(actual_capital, reg_capital);
    })

    //所属行业
    val code2Name: Broadcast[Map[String, String]] = spark.sparkContext.broadcast(sql(
      """
        |select category_code,category_str_big
        |from const_company_category_code
      """.stripMargin).collect().map(r => {
      (r.getString(0), r.getString(1))
    }).toMap)

    spark.udf.register("industry_score", (code: String) => {
      DimScore.industryEvaluate(code2Name.value.getOrElse(code, null))
    })

    //企业类型
    spark.udf.register("company_type_score", (company_org_type: String) => {
      DimScore.companyTypeEvaluate(company_org_type);
    })

    //企业状态
    spark.udf.register("company_status_score", (reg_status: String) => {
      DimScore.companyStatusEvaluate(reg_status);
    })

    //成立日期
    spark.udf.register("start_date_score", (estiblish_time: String) => {
      DimScore.startDateEvaluate(estiblish_time);
    })

    //企业期限
    spark.udf.register("end_date_score", (from_time: String, to_time: String) => {
      DimScore.endDateEvaluate(from_time, to_time);
    })

    //注册地址
    spark.udf.register("reg_location_score", (reg_location: String) => {
      DimScore.addressEvaluate(reg_location);
    })

  }

}
