package com.winhc.bigdata.spark.udf

import org.apache.spark.sql.Row
import org.apache.spark.sql.expressions.{MutableAggregationBuffer, UserDefinedAggregateFunction}
import org.apache.spark.sql.types.{DataType, DataTypes, StructField, StructType}
import scala.collection.mutable

/**
 * Create by π
 * Desc: 聚合map
 */

class MapAggs(max: Int = 500) extends UserDefinedAggregateFunction {
  override def inputSchema: StructType = StructType(
    Array[StructField](StructField("t1", DataTypes.createMapType(DataTypes.StringType, DataTypes.StringType)))
  )

  override def bufferSchema: StructType = StructType(
    Array[StructField](StructField("t1", DataTypes.createMapType(DataTypes.StringType, DataTypes.StringType)))
  )

  override def dataType: DataType = DataTypes.createMapType(DataTypes.StringType, DataTypes.StringType)

  override def initialize(buffer: MutableAggregationBuffer): Unit = {
    buffer.update(0, Map[String, String]())
  }

  override def deterministic: Boolean = true

  override def update(buffer: MutableAggregationBuffer, input: Row): Unit = {
    val map1 = buffer.getMap[String, String](0).toMap
    if (map1.size >= max) {
      return
    }
    val map2 = input.getMap[String, String](0).toMap
    val map_new: mutable.Map[String, String] = (scala.collection.mutable.Map[String, String](map1.toSeq: _*)
      ++ scala.collection.mutable.Map[String, String](map2.toSeq: _*))
    buffer.update(0, map_new)
  }

  override def merge(buffer1: MutableAggregationBuffer, buffer2: Row): Unit = {
    update(buffer1, buffer2)
  }

  override def evaluate(buffer: Row): Any = {
    buffer.getAs[Map[String, String]](0)
  }
}
