package com.winhc.bigdata.spark.udf

import java.util

import com.alibaba.fastjson.JSON
import com.winhc.bigdata.spark.utils.BaseUtil
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.Row
import org.apache.spark.sql.expressions.{MutableAggregationBuffer, UserDefinedAggregateFunction}
import org.apache.spark.sql.types._

/**
 * @Description:原告,被告聚合v8版本
 * @author π
 * @date 2021/05/24 14:40
 */

class NameAggsPlus(max: Int) extends UserDefinedAggregateFunction {

  //val flags = Seq("0", "1", "2", "4", "8")
  val split = "\u0001"
  val empty_col = "[]"

  override def inputSchema: StructType = StructType(Array[StructField](
    StructField("yg_name", DataTypes.StringType)
    , StructField("bg_name", DataTypes.StringType)
    , StructField("flag", DataTypes.StringType)
    , StructField("bus_date", DataTypes.StringType)
  ))

  override def bufferSchema: StructType = StructType(
    Array[StructField](
      StructField("t1", DataTypes.createMapType(DataTypes.StringType, DataTypes.StringType))
      , StructField("t2", DataTypes.createMapType(DataTypes.StringType, DataTypes.StringType))
    )
  )

  override def dataType: DataType = DataTypes.createMapType(DataTypes.StringType, DataTypes.StringType)

  override def deterministic: Boolean = true

  override def initialize(buffer: MutableAggregationBuffer): Unit = {
    buffer.update(0, Map[String, String]())
    buffer.update(1, Map[String, String]())
  }

  override def update(buffer: MutableAggregationBuffer, input: Row): Unit = {

    if (buffer.size >= max) {
      return
    }
    if(input.size != 4) return
    val yg_name = input.getAs[String](0)
    val bg_name = input.getAs[String](1)
    val flag = input.getAs[String](2)
    val bus_date = input.getAs[String](3)

    if (StringUtils.isBlank(bus_date)) {
      return
    }
    if (StringUtils.isBlank(yg_name) && StringUtils.isBlank(bg_name)) {
      return
    }

    //原被告转换
    val yg_name1 = BaseUtil.filter_json(yg_name, "name")
    val bg_name1 = BaseUtil.filter_json(bg_name, "name")

    if (empty_col.equals(yg_name1) && empty_col.equals(bg_name1)) {
      return
    }

    val map0 = buffer.getMap[String, String](0).toMap
    val map1 = buffer.getMap[String, String](1).toMap
    var map_new0 = scala.collection.mutable.Map[String, String](map0.toSeq: _*)
    var map_new1 = scala.collection.mutable.Map[String, String](map1.toSeq: _*)
    if (!empty_col.equals(yg_name1) && !empty_col.equals(bg_name1)) {
      map_new0 ++= Map(bus_date -> s"$yg_name1$split$bg_name1")
    } else {
      map_new1 ++= Map(bus_date -> s"$yg_name1$split$bg_name1")
    }
    buffer.update(0, map_new0)
    buffer.update(1, map_new1)
  }

  override def merge(buffer1: MutableAggregationBuffer, buffer2: Row): Unit = {
    buffer1(0) = buffer1.getAs[Map[String, String]](0) ++ buffer2.getAs[Map[String, String]](0)
    buffer1(1) = buffer1.getAs[Map[String, String]](1) ++ buffer2.getAs[Map[String, String]](1)
  }

  override def evaluate(buffer: Row): Any = {
    var yg_name = empty_col
    var bg_name = empty_col
    val m0: Map[String, String] = buffer.getAs[Map[String, String]](0)
    val m1: Map[String, String] = buffer.getAs[Map[String, String]](1)
    if (m0.isEmpty && m1.isEmpty) {
      return Map("yg_name" -> yg_name, "bg_name" -> bg_name)
    } else if (m0.nonEmpty) {
      val key = m0.keySet.toSeq.sorted.head
      val arr = m0(key).split(s"$split", -1)
      yg_name = arr(0)
      bg_name = arr(1)
    } else if(m1.nonEmpty){
      val key = m1.keySet.toSeq.sorted.head
      val arr = m1(key).split(s"$split", -1)
      yg_name = arr(0)
      bg_name = arr(1)
    }
    Map("yg_name" -> yg_name, "bg_name" -> bg_name)
  }
}
