package com.winhc.bigdata.spark.udf

import com.winhc.bigdata.spark.utils.BaseUtil
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.Row
import org.apache.spark.sql.expressions.{MutableAggregationBuffer, UserDefinedAggregateFunction}
import org.apache.spark.sql.types._

import scala.collection.JavaConverters._
import scala.collection.immutable

/**
 * @Description:优先取文书数据
 * @author π
 * @date 2021/08/19 14:40
 */

class NameAggsPlusV2(max: Int) extends UserDefinedAggregateFunction {

  //val flags = Seq("0", "1", "2", "4", "8")
  val split = "\u0001"
  val empty_col = "[]"

  override def inputSchema: StructType = StructType(Array[StructField](
    StructField("yg_name", DataTypes.StringType)
    , StructField("bg_name", DataTypes.StringType)
    , StructField("flag", DataTypes.StringType)
    , StructField("bus_date", DataTypes.StringType)
  ))

  override def bufferSchema: StructType = StructType(
    Array[StructField](
      StructField("t1", DataTypes.createMapType(DataTypes.StringType, DataTypes.StringType))
    )
  )

  override def dataType: DataType = DataTypes.createMapType(DataTypes.StringType, DataTypes.StringType)

  override def deterministic: Boolean = true

  override def initialize(buffer: MutableAggregationBuffer): Unit = {
    buffer.update(0, Map[String, String]())
  }

  override def update(buffer: MutableAggregationBuffer, input: Row): Unit = {
    if (buffer.size >= max) {
      return
    }
    if (input.size != 4) return
    val yg_name = input.getAs[String](0)
    val bg_name = input.getAs[String](1)
    val flag = input.getAs[String](2)
    val bus_date = input.getAs[String](3)
    if (StringUtils.isBlank(bus_date)) {
      return
    }
    if (StringUtils.isBlank(yg_name) && StringUtils.isBlank(bg_name)) {
      return
    }
    //原被告转换
    val yg_name1 = BaseUtil.filter_json(yg_name, "name")
    val bg_name1 = BaseUtil.filter_json(bg_name, "name")

    if (empty_col.equals(yg_name1) && empty_col.equals(bg_name1)) {
      return
    }
    val map0 = buffer.getMap[String, String](0).toMap
    var map_new0 = scala.collection.mutable.Map[String, String](map0.toSeq: _*)
    map_new0 ++= Map(s"$bus_date$split$flag" -> s"$yg_name1$split$bg_name1$split$flag")
    buffer.update(0, map_new0)
  }

  override def merge(buffer1: MutableAggregationBuffer, buffer2: Row): Unit = {
    buffer1(0) = buffer1.getAs[Map[String, String]](0) ++ buffer2.getAs[Map[String, String]](0)
  }

  override def evaluate(buffer: Row): Any = {
    var yg_name = empty_col
    var bg_name = empty_col
    val m0: Map[String, String] = buffer.getAs[Map[String, String]](0)
    println(s"m0: $m0")
    var has = false
    if (m0.isEmpty) {
      return Map("yg_name" -> yg_name, "bg_name" -> bg_name)
    } else if (m0.nonEmpty) {
      val tmp: List[(String, String, String, String, String)] = m0.map(x => {
        val Array(yg_name1, bg_name1, flag) = x._2.split(s"$split", -1)
        if (flag.equals("0")) {
          has = true
        }
        var weight = "0"
        if (!empty_col.equals(yg_name1) && !empty_col.equals(bg_name1)) {
          weight = "1"
        }
        (x._1, yg_name1, bg_name1, flag, weight)
      }).
        filter(x => {
        if (has) {
          x._4.equals("0")
        } else {
          !has
        }
      }).
        toList.sortBy(x => (x._5, x._1))(Ordering.Tuple2(Ordering.String.reverse, Ordering.String))
      println(s"tmp: $tmp")
      val (_, yg_name2, bg_name2, _, _) = tmp.head
      yg_name = yg_name2
      bg_name = bg_name2
    }
    Map("yg_name" -> yg_name, "bg_name" -> bg_name)
  }
}
