package com.winhc.bigdata.spark.utils

import cn.hutool.core.util.StrUtil
import com.alibaba.fastjson.serializer.SerializeFilter
import com.alibaba.fastjson.{JSON, JSONArray, JSONPath}
import com.winhc.bigdata.spark.implicits.RegexUtils._
import org.apache.commons.lang3.StringUtils
import org.apache.commons.lang3.time.DateFormatUtils
import org.apache.spark.sql.SparkSession
import org.json4s.DefaultFormats
import org.json4s.jackson.Json

import java.security.MessageDigest
import java.text.SimpleDateFormat
import java.util
import java.util.regex.Pattern
import java.util.{Calendar, Date, Locale}
import scala.collection.{immutable, mutable}
import scala.collection.mutable.ListBuffer

/**
 * @Author: XuJiakai
 * @Date: 2020/6/3 18:49
 * @Description:
 */
object BaseUtil {
  //去其他符号 去空格
  private val pattern = "[^\\u4e00-\\u9fa50-9a-zA-Z]".r

  //去其他符号 去空格  补全null
  def cleanup(s: String): String = {
    if (StringUtils.isBlank(s))
      ""
    else
      pattern replaceAllIn(s, "")
  }

  def is_json_str(str: String): Boolean = {
    try {
      JSON.parse(str)
      true
    } catch {
      case exception: Exception => false
    }
  }

  def isWindows: Boolean = System.getProperty("os.name").contains("Windows")

  def getPartitions(t: String, @transient spark: SparkSession): Seq[String] = {
    import spark._
    val sql_s = s"show partitions " + t
    sql(sql_s).collect.toList.map(_.getString(0).split("=")(1)).seq
  }


  def getPartionsPlus(t: String, p: String, @transient spark: SparkSession): Seq[String] = {
    import spark._
    val sql_s = s"show partitions " + t
    val ps = sql(sql_s).collect.toList.map(r => {
      val Array(x, y) = r.getString(0).split("/")
      val Array(_, a2) = x.split("=")
      val Array(_, b2) = y.split("=")
      var r1 = ("", "")
      if (StringUtils.isNumeric(a2)) {
        r1 = (a2, b2)
      } else {
        r1 = (b2, a2)
      }
      r1
    }).filter(_._2.equals(p)).map(_._1).seq
    ps
  }

  def getPartion(t: String, p: String, @transient spark: SparkSession): String = {
    val ps = getPartionsPlus(t, p, spark)
    if (ps.size > 0) {
      ps.sorted.last
    } else {
      ""
    }
  }

  def getSecondPartion(t: String, p: String, @transient spark: SparkSession, default: String = ""): String = {
    val ps = getPartionsPlus(t, p, spark)
    if (ps.length >= 2) {
      ps(ps.length - 2)
    } else {
      default
    }
  }

  def getPartion(t: String, @transient spark: SparkSession) = {
    val ps = getPartitions(t, spark)
    if (ps.size > 0) {
      ps.last
    } else {
      ""
    }
  }

  def getFirstPartion(t: String, @transient spark: SparkSession) = {
    val ps = getPartitions(t, spark)
    if (ps.size > 0) {
      ps.head
    } else {
      ""
    }
  }

  def getLastPartion(t: String, @transient spark: SparkSession): Array[(String, String)] = {
    import spark._
    val sql_s = s"show partitions " + t
    val ps = sql(sql_s).collect.toList.map(r => {
      val tmp = r.getString(0).split("/")
      val Array(x, y) = {
        if (tmp.length == 2) {
          tmp
        }
        else {
          Array(tmp(0), "")
        }
      }
      val Array(_, a2) = x.split("=")
      val Array(_, b2) = if (y != "") y.split("=") else Array("", "")
      var r1 = ("", "")
      if (StringUtils.isNumeric(a2)) {
        r1 = (b2, a2) //维度名在前、日期在后
      } else {
        r1 = (a2, b2)
      }
      r1
    }).groupBy(_._1)
      .map(m => (m._1, m._2.maxBy(n => n._2)._2))
      .filter(_._2 != "")
    ps.toArray
  }

  def nowDate(pattern: String = "yyyy-MM-dd"): String = {
    new SimpleDateFormat(pattern).format(new Date)
  }

  def atMonthsBefore(n: Int, pattern: String = "yyyy-MM-dd"): String = {
    val c = Calendar.getInstance(Locale.CHINA)
    c.setTimeInMillis(new Date().getTime)
    c.add(Calendar.MONTH, -1 * n)
    DateFormatUtils.format(c.getTime.getTime, pattern)
  }

  def getYesterday(): String = {
    atDaysAfter(-1, nowDate("yyyyMMdd"))
  }

  def getYear(n: Int): String = {
    val cal = Calendar.getInstance(Locale.CHINA)
    cal.add(Calendar.YEAR, 1 * n)
    cal.get(Calendar.YEAR).toString
  }

  def atDaysAfter(n: Int, time: String, pattern: String = "yyyyMMdd"): String = {
    import java.text.SimpleDateFormat
    val newtime: Date = new SimpleDateFormat("yyyyMMdd").parse(time)
    val c = Calendar.getInstance(Locale.CHINA)
    c.setTimeInMillis(newtime.getTime)
    c.add(Calendar.DATE, 1 * n)
    DateFormatUtils.format(c.getTime.getTime, pattern)
  }

  private val case_no = Pattern.compile(".*([（\\(]\\d{4}[）\\)].*?号).*")

  def caseNo(s: String): String = {
    if (StringUtils.isBlank(s)) return ""
    val s1 = s.replaceAll("(\\r\\n|\\r|\\n|\\n\\r|\\s+)", "")
    val matcher = case_no.matcher(s1)
    if (matcher.find) {
      return matcher.replaceAll("$1").replaceAll("\\(", "（").replaceAll("\\)", "）")
    }
    ""
  }

  private val replace_char = "[^\\u4e00-\\u9fa5a-zA-Z\\(\\)（）]+".r

  def replaceChar(s: String) = {
    if (StringUtils.isNotBlank(s)) {
      val arr =
        s.replaceAll("、", ",")
          .replaceAll(";", ",")
          .replaceAll("，", ",")
          .replaceAll(" ", ",")
          .replaceAll("。", ",")
          .replaceAll("；", ",")
          .replaceAll("：", ",")
          .replaceAll("\\s+", ",").split(",")

      val list = arr.toSet.filter(_.length > 1).toList
      if (list.nonEmpty) {
        val sb = new StringBuilder
        for (a <- list) {
          sb.append(a).append(",")
        }
        sb.substring(0, sb.lastIndexOf(",")) toString()
      } else {
        ""
      }
    } else {
      ""
    }
  }

  def BKDRHash(str: String): Long = {
    val seed: Long = 1313131313 // 31 131 1313 13131 131313 etc..
    var hash: Long = 0
    for (i <- 0 to str.length - 1) {
      hash = hash * seed + str.charAt(i)
      hash = hash & 0x7FFFFFFFFFFFFFFFL
    }
    return hash
  }

  def MD5hash(content: String): String = {
    val md5 = MessageDigest.getInstance("MD5")
    val encoded = md5.digest(cleanup(content).getBytes)
    encoded.map("%02x".format(_)).mkString
  }

  def nameJudge(name: String, yg_name: String, bg_name: String): String = {
    if (StringUtils.isNotBlank(name)) {
      if (StringUtils.isNotBlank(yg_name) && yg_name.contains(name)) {
        return "y"
      } else if (StringUtils.isNotBlank(bg_name) && bg_name.contains(name)) {
        return "b"
      }
    }
    ""
  }

  def caseType(caseNO: String): String = {
    if (StringUtils.isNotBlank(caseNO)) {
      if (caseNO.contains("辖")) {
        return "管辖案件"
      } else if (caseNO.contains("民")) {
        return "民事案件"
      } else if (caseNO.contains("执")) {
        return "执行案件"
      } else if (caseNO.contains("行")) {
        return "行政案件"
      } else if (caseNO.contains("刑")) {
        return "刑事案件"
      } else if (caseNO.contains("赔")) {
        return "赔偿案件"
      }
    }
    "其它案件"
  }

  def caseStageOld(caseNo: String): String = {
    if (StringUtils.isNotBlank(caseNo)) {
      val casePre = caseType(caseNo).replaceAll("案件", "")
      if (StrUtil.containsAny(caseNo, "破申", "商申") || casePre.contains("其它")) {
        return "其它阶段"
      }
      if (StrUtil.containsAny(caseNo, "监", "抗", "再", "申", "提", "再")) {
        return s"${casePre}再审"
      }
      if (StrUtil.containsAny(caseNo, "初")) {
        return s"${casePre}一审"
      } else if (StrUtil.containsAny(caseNo, "终")) {
        return s"${casePre}二审"
      } else if (StrUtil.containsAny(caseNo, "执")) {
        return "首次执行"
      }
    }
    "其它阶段"
  }

  def caseStage(caseNo: String): String = {
    if (StringUtils.isNotBlank(caseNo)) {
      if (StrUtil.containsAny(caseNo, "民") && StrUtil.containsAny(caseNo, "初")) {//特殊
        return "民事一审"
      } else if (StrUtil.containsAny(caseNo, "执恢")) {
        return "恢复执行"
      } else if (StrUtil.containsAny(caseNo, "执异")) {
        return "执行异议"
      } else if (StrUtil.containsAny(caseNo, "执复")) {
        return "执行复议"
      } else if (StrUtil.containsAny(caseNo, "司救执")) {
        return "执行司法救助"
      } else if (StrUtil.containsAny(caseNo, "司救民")) {
        return "民事司法救助"
      } else if (StrUtil.containsAny(caseNo, "执监")) {
        return "执行监督"
      } else if (StrUtil.containsAny(caseNo, "执他")) {
        return "其他执行"
      } else if (StrUtil.containsAny(caseNo, "执")) {
        return "首次执行"
      } else if (StrUtil.containsAny(caseNo, "刑初")) {
        return "刑事一审"
      } else if (StrUtil.containsAny(caseNo, "民终")) {
        return "民事二审"
      } else if (StrUtil.containsAny(caseNo, "特")) {
        return "特别程序"
      } else if (StrUtil.containsAny(caseNo, "民申")) {
        return "民事申请再审审查"
      } else if (StrUtil.containsAny(caseNo, "刑终")) {
        return "刑事二审"
      } else if (StrUtil.containsAny(caseNo, "行终")) {
        return "行政二审"
      } else if (StrUtil.containsAny(caseNo, "财保")) {
        return "财产保全"
      } else if (StrUtil.containsAny(caseNo, "行初")) {
        return "行政一审"
      } else if (StrUtil.containsAny(caseNo, "民辖终")) {
        return "民事管辖上诉"
      } else if (StrUtil.containsAny(caseNo, "民辖")) {
        return "民事管辖"
      } else if (StrUtil.containsAny(caseNo, "民再")) {
        return "民事再审"
      } else if (StrUtil.containsAny(caseNo, "诉前调")) {
        return "诉前调解"
      } else if (StrUtil.containsAny(caseNo, "刑更")) {
        return "刑罚与执行变更"
      } else if (StrUtil.containsAny(caseNo, "行审复")) {
        return "非诉行政行为申请执行审查复议"
      } else if (StrUtil.containsAny(caseNo, "行审")) {
        return "非诉行政行为申请执行审查"
      } else if (StrUtil.containsAny(caseNo, "民督")) {
        return "申请支付令审查"
      } else if (StrUtil.containsAny(caseNo, "行申")) {
        return "行政申请再审审查"
      } else if (StrUtil.containsAny(caseNo, "刑再")) {
        return "刑事审判监督"
      } else if (StrUtil.containsAny(caseNo, "行赔初")) {
        return "行政赔偿一审"
      } else if (StrUtil.containsAny(caseNo, "行赔终")) {
        return "行政赔偿二审"
      } else if (StrUtil.containsAny(caseNo, "行赔申")) {
        return "行政赔偿申请再审审查"
      } else if (StrUtil.containsAny(caseNo, "刑辖")) {
        return "刑事管辖"
      } else if (StrUtil.containsAny(caseNo, "司惩")) {
        return "司法制裁审查"
      } else if (StrUtil.containsAny(caseNo, "民监")) {
        return "民事依职权再审审查"
      } else if (StrUtil.containsAny(caseNo, "刑他")) {
        return "其他刑事"
      } else if (StrUtil.containsAny(caseNo, "民抗")) {
        return "民事抗诉再审审查"
      } else if (StrUtil.containsAny(caseNo, "民催")) {
        return "催告"
      } else if (StrUtil.containsAny(caseNo, "民他")) {
        return "其他民事"
      } else if (StrUtil.containsAny(caseNo, "行辖")) {
        return "行政管辖"
      } else if (StrUtil.containsAny(caseNo, "民撤")) {
        return "第三人撤销之诉"
      } else if (StrUtil.containsAny(caseNo, "行再")) {
        return "行政再审"
      } else if (StrUtil.containsAny(caseNo, "委赔监")) {
        return "司法赔偿监督审查"
      } else if (StrUtil.containsAny(caseNo, "委赔")) {
        return "赔偿委员会审理赔偿"
      } else if (StrUtil.containsAny(caseNo, "刑医")) {
        return "强制医疗"
      } else if (StrUtil.containsAny(caseNo, "法赔")) {
        return "法院作为赔偿义务机关自赔"
      } else {
        return "其它阶段"
      }
    }
    "其它阶段"
  }

  def lastStage(s: String): String = {
    if (StringUtils.isNotBlank(s)) {
      val arr = s.split(" ")
      if (StrUtil.containsAny(s, "执")) {
        return arr.filter(StrUtil.containsAny(_, "执")).head
      }
      if (StrUtil.containsAny(s, "再")) {
        return arr.filter(StrUtil.containsAny(_, "再")).head
      }
      if (StrUtil.containsAny(s, "二")) {
        return arr.filter(StrUtil.containsAny(_, "二")).head
      }
      if (StrUtil.containsAny(s, "一")) {
        return arr.filter(StrUtil.containsAny(_, "一")).head
      }
      return arr.head
    }
    "其它"
  }

  def title(ygname: String, bgname: String, reason: String): String = {
    Seq(replaceChar(ygname), replaceChar(bgname), reason).filter(s => StringUtils.isNotBlank(s)).mkString(",")
  }

  def compareName(ygname: String, bgname: String): Boolean = {
    if (StringUtils.isBlank(ygname) || StringUtils.isBlank(bgname)) {
      return false
    }
    val a1 = ygname.split(",", -1).filter(StringUtils.isNotBlank(_)).toSet
    val a2 = bgname.split(",", -1).filter(StringUtils.isNotBlank(_)).toSet
    if (a1.size == 0 || a2.size == 0) {
      return false
    }
    (a1 & a2).size == 0
  }

  def trimBlack(s: String): String = {
    var r = ""
    if (StringUtils.isNotBlank(s)) {
      r = s.split(",").filter(StringUtils.isNotBlank(_)).toSet.mkString(",")
    }
    r
  }

  def label(s: String): String = {
    var r = ""
    if (StringUtils.isNotBlank(s)) {
      r = s match {
        case "0" => "裁判文书" //企业
        case "1" => "开庭公告" //企业
        case "2" => "法院公告" //企业
        case "3" => "失信人" //企业
        case "4" => "送达公告" //企业
        case "5" => "限制高消费" //企业
        case "6" => "终本案件" //企业
        case "7" => "被执行人" //企业
        case "8" => "立案信息" //企业
        case "9" => "失信人" //人
        case "10" => "被执行人" //人
        case "11" => "限制高消费" //人
        case "12" => "终本案件" //人
        case _ => ""
      }
    }
    r
  }


  /**
   * 全角字符转半角字符
   *
   * @param str
   * @return
   */
  def toDBC(str: String): String = {
    if(str==null){
      return null
    }
    val c = str.toCharArray
    for (i <- 0 until c.length) {
      if (c(i) == 12288) {
        c(i) = 32.toChar
      }
      if (c(i) > 65280 && c(i) < 65375) c(i) = (c(i) - 65248).toChar
    }
    new String(c)
  }

  @deprecated
  private val case_pat =
    ".*([（\\(]\\d{4}[）\\)][^号]*号?).*".r
  @deprecated
  private val case_pat2 =
    "(((20\\d{2}|19\\d{2})|\\((20\\d{2}|19\\d{2})\\))[^号]{3,}号?).*".r
  private val case_pat3 =
    ".*([（\\(]\\d{4}[）\\)][^号]*号.*?)".r
  @deprecated
  private val year_pat =
    "(\\d{4}?)年".r
  private val year_pat_2 =
    "\\(?(\\d{4}?)\\)?年".r

  /**
   * 案号格式规整
   *
   * @param str
   * @return
   */
  @deprecated
  def case_no_trim(str: String): String = {
    if (StringUtils.isBlank(str)) return null
    var case_no = toDBC(str)
    case_no = case_no.replace(" ", "");
    if (case_no.length < 8) return null
    case_no = year_pat.replaceAllIn(case_no, "\\($1\\)")
    case_no = case_no.replace("（", "(")
      .replace("）", ")")
    if (case_pat2 matches case_no) {
      case_pat2.replaceAllIn(case_no, "$1")
    } else null
  }


  /**
   * 案号格式规整
   *
   *   1.全角符号替换为半角。
   *   2.去除空格
   *   3.长度大于等于8
   *   4.中英文括号统一为英文
   *   5.将手误的字母o转为数字0
   *   6.去除连续的号字。如“号号” -> “号”
   *   7.将“yyyy年”转化为 “(yyyy)”
   *   8.从“(yyyy)”开始必须包函“号”字
   *
   * @param str
   * @return
   */
  def case_no_trim_v2(str: String): String = {
    if (StringUtils.isBlank(str)) return null
    var case_no = toDBC(str)
    case_no = case_no.replace(" ", "");
    if (case_no.length < 8) return null
    case_no = case_no.replace("（", "(")
      .replace("）", ")")
      .replaceAll("O", "0")
      .replaceAll("号{2,}", "号")
    case_no = year_pat_2.replaceAllIn(case_no, "\\($1\\)")
    if (case_pat3 matches case_no) {
      case_pat3.replaceAllIn(case_no, "$1")
    } else null
  }

  def sortString(s: String, split: String = "\\001"): String = {
    var r = ""
    if (StringUtils.isNotBlank(s)) {
      r = s.split(split).toSet.toList.sorted.mkString(split)
    }
    r
  }

  /**
   * 身份证号格式统一
   *
   * @param str
   * @return
   */
  def id_card_trim(str: String): String = {
    if (StringUtils.isNotBlank(str)) {
      return s"${str.substring(0, 10)}****${str.substring(str.length - 4)}".toUpperCase
    }
    null
  }

  private val id_card_pattern =
    "^[1-9]\\d{5}(19|20)\\d{2}((0[1-9])|(1[0-2])|([0-1]\\*{1,2})|\\*{2})(([0-2][1-9])|10|20|30|31|\\*{2})\\d{3}[0-9Xx]$".r

  private lazy val maxYear =
    BaseUtil.nowDate(pattern = "yyyy").toInt

  def is_id_card(str: String): Boolean = {
    if (StringUtils.isEmpty(str))
      return false

    if (id_card_pattern matches str) {
      val d = str.substring(6, 10).toInt
      d <= maxYear
    } else
      false
  }

  val pat = "(?<province>[^省]+省|.+自治区)?(?<city>[^自治州]+自治州|[^市]+市|[^盟]+盟|[^地区]+地区|.+区划)?(?<district>[^市]+市|[^县]+县|[^旗]+旗|.+区)?".r

  /**
   * 提取省市区
   *
   * @param addr
   * @return
   */
  def parseAddress(addr: String): (String, String, String) = {
    val matcher = pat.pattern.matcher(addr)
    if (!matcher.find()) {
      return null
    }
    val province = matcher.group("province")
    val city = matcher.group("city")
    val district = matcher.group("district")
    (province, city, district)
  }

  private val LONGITUDE =
    "^[\\-\\+]?(0?\\d{1,2}\\.\\d{1,15}|1[0-7]?\\d{1}\\.\\d{1,15}|180\\.0{1,15})$"
  private val LATITUDE =
    "^[\\-\\+]?([0-8]?\\d{1}\\.\\d{1,15}|90\\.0{1,15})$"

  /**
   * 经纬度转换
   */
  def verify(longitude: String, latitude: String): Unit = {
    val s = new StringBuilder()
    if (StringUtils.isNotBlank(longitude) && StringUtils.isNotBlank(latitude)) {
      if (Pattern.matches(LONGITUDE, longitude) && Pattern.matches(LATITUDE, latitude)) {
        s.append(longitude).append(",").append(latitude)
      }
      else if (Pattern.matches(LONGITUDE, latitude) && Pattern.matches(LATITUDE, longitude)) {
        s.append(latitude).append(",").append(longitude)
      }
    } else {
      s.append(",")
    }
    s.toString()
  }

  private val name_pat =
    Pattern.compile("[^\\u4e00-\\u9fa50-9a-zA-Z（）() ·,]")

  def nameCleanup(name: String): String = {
    var re = ""
    if (StringUtils.isNotBlank(name)) {
      re = name_pat.matcher(name.replaceAll("[、，；]", ",")).replaceAll("").replaceAll(" +", " ")
    }
    re
  }

  def split_names(json_array: String, json_path: String): Seq[String] = {
    try {
      if (StringUtils.isEmpty(json_array)) {
        return Seq.empty
      }

      val set = mutable.Set[String]()
      val jsonArr = json_array.split("&", -1)
      val pathArr = json_path.split("&", -1)
      var path = pathArr.head
      for (i <- 0 until jsonArr.length) {
        if (pathArr.length > 1) {
          path = pathArr(i)
        }
        parse(jsonArr(i), json_path, set)
      }

      set.toSeq
    } catch {
      case e: Exception => {
        println(json_array)
        Seq.empty
      }
    }
  }

  def parse(json: String, json_path: String, set: mutable.Set[String]): Unit = {
    val jsonArray = JSONPath.eval(JSON.parse(json), json_path).asInstanceOf[JSONArray].toArray[String](Array())
    for (o <- jsonArray) {
      set.add(o)
    }
  }

  def filter_json(json: String, col: String): String = {
    if (StringUtils.isBlank(json) || StringUtils.isBlank(col)) return "[]"
    import scala.collection.JavaConverters._
    try {
      val list1 = JSON.parseArray(json, classOf[util.Map[String, String]]).asScala
      val list2 = list1.filter(x => x.containsKey(col) && StringUtils.isNotBlank(x.get(col))).asJava
      JSON.toJSONString(list2, new Array[SerializeFilter](0))
    } catch {
      case e: Exception => {
        "[]"
      }
    }
  }


  def regJson(json: Option[Any]) = json match {
    case Some(map: Map[String, Any]) => map
    case Some(map: List[Map[String, Any]]) => map
    case _ => List[Map[String, String]]()
  }

  def json_array(json: String, col: String = "name"): List[Map[String, String]] = {
    try {
      if (StringUtils.isBlank(json)) return List[Map[String, String]]()
      regJson(scala.util.parsing.json.JSON.parseFull(filter_json(json, col))).asInstanceOf[List[Map[String, String]]]
    } catch {
      case e: Exception => {
        //println("error--json "+json)
        List[Map[String, String]]()
        // List.empty
      }
    }
  }

  def list_json(list: List[Map[String, String]]): String = {
    import org.json4s.DefaultFormats
    var m = mutable.Map[String, mutable.Map[String, String]]()
    try {
      list.foreach(x => {
        val n = collection.mutable.Map(x.toSeq: _*)
        val name = x.getOrElse("name", "")
        val litigant_id = x.getOrElse("litigant_id", "")
        if (name.length > 0 && (!m.contains(name) || litigant_id.length > 0)) {
          m ++= mutable.Map(name -> n)
        }
      })
      if (m.isEmpty) return "[]"
      Json(DefaultFormats).write(m.values)
    } catch {
      case e: Exception => {
        return "[]"
      }
    }
  }

  def merge_exec_json(list: List[Map[String, String]]): String = {
    import org.json4s.DefaultFormats
    //    try {
    val list1: Map[String, List[(String, String, String, Double)]] = list.map(x => {
      val name = x.getOrElse("name", "")
      val litigant_id = x.getOrElse("litigant_id", "")
      val date = x.getOrElse("date", "")
      val exec_money = x.getOrElse("exec_money", 0).asInstanceOf[Double]
      (name, litigant_id, date, exec_money)
    }).filter(x => {
      x._4 > 0 && StringUtils.isNotBlank(x._1) && StringUtils.isNotBlank(x._3)
    }).groupBy(_._1)
    //println("list1|  " + list)
    val list2: Iterable[Map[String, Any]] = list1.mapValues(l => {
      var name: String = l.head._1
      var litigant_id: String = l.head._2
      var date: String = l.head._3
      var exec_money: Double = l.head._4
      l.foreach(x => {
        if (StringUtils.isNotBlank(x._1)) name = x._1
        if (StringUtils.isNotBlank(x._2)) litigant_id = x._2
        if (date.compare(x._3) > 0) {
          date = x._3
          exec_money = x._4
        }
      })
      Map("name" -> name, "litigant_id" -> litigant_id, "date" -> date, "exec_money" -> exec_money)
    }).values
    if (list2.isEmpty) return "[]"
    Json(DefaultFormats).write(list2)
    //    } catch {
    //      case e: Exception => {
    //        return "[]"
    //      }
    //    }
  }

  def merge_json(yg_name: String, bg_name: String, all_name: String): String = {
    list_json(json_array(yg_name) ++ json_array(bg_name) ++ json_array(all_name))
  }

  def all_name(s: String): String = {
    val r = "[]"
    if (StringUtils.isNotBlank(s)) {
      val names = s.split("\\u0001", -1)
      val list = ListBuffer[Map[String, String]]()
      for (n <- names) {
        list ++= json_array(n)
      }
      return list_json(list.toList)
    }
    r
  }

  val tn_mapping = Map[String, String](
    "company_lawsuit" -> "0"
    , "company_court_open_announcement" -> "1"
    , "company_court_announcement" -> "2"
    , "company_dishonest_info" -> "3"
    , "company_send_announcement" -> "4"
    , "company_zxr_restrict" -> "5"
    , "company_zxr_final_case" -> "6"
    , "company_zxr" -> "7"
    , "company_court_register" -> "8"
  )

  def flag_tn(flag: String): String = {
    if (StringUtils.isBlank(flag)) return "-1"
    val m = for ((a, b) <- tn_mapping) yield (b, a)
    if (m.contains(flag)) return m(flag)
    "-1"
  }

  def tn_flag(tn: String): String = {
    if (StringUtils.isBlank(tn)) return "-1"
    if (tn_mapping.contains(tn)) return tn_mapping(tn)
    "-1"
  }

  def add_pre(id: String): String = {
    import scala.util.Random
    val r = new Random().nextInt(100)
    s"${r}_${id}"
  }

  def remove_pre(id: String): String = {
    if (StringUtils.isBlank(id)) return ""
    id.split("_", -1)(1)
  }

  def combine_id(rw1: String, rw2: String): String = {
    val listBuffer = new ListBuffer[String]()
    if (StringUtils.isNotBlank(rw1)) {
      listBuffer.append(rw1)
    }
    if (StringUtils.isNotBlank(rw2)) {
      listBuffer.append(rw2)
    }
    listBuffer.toList.sorted.mkString("_")
  }

  def adjust_reason(reason: String): String = {
    if (StringUtils.isBlank(reason)) return ""
    if (reason.equals("其他")) return "其它"
    reason
  }

  def amt_div(amt1: String, amt2: String): Double = {
    var r1 = 0d
    if (StringUtils.isBlank(amt1) || StringUtils.isBlank(amt2)) r1 = 0d
    try {
      r1 = BigDecimalUtil.div(amt1, amt2, 6)
    } catch {
      case e: Exception => r1 = 0d
    }
    r1
  }

  def amt_merge(amt_info: String): String = {
    if (StringUtils.isBlank(amt_info)) return "[]"
    val a1 = amt_info.split("&", -1)
    val m: immutable.Seq[Map[String, Any]] = a1.map(x => {
      val Array(case_stage, amt) = x.split("\\|", -1)
      (case_stage, amt.toDouble)
    }).filter(_._2 > 0).groupBy(_._1).mapValues(x => {
      x.map(_._2).max
    }).toList.map(x => {
      Map("case_stage" -> x._1, "amt" -> x._2)
    })
    Json(DefaultFormats).write(m)
  }

  def main(args: Array[String]): Unit = {
    println(amt_merge("一审|20.3&二审|30.3&再审|40.3&再审|30.3&一审|30.3"))
    println(amt_div("1998301", "10000"))
    println(adjust_reason("其他"))
    println(adjust_reason("其它"))
    println(combine_id(null, ""))
    println(add_pre("123"))
    println(remove_pre("1_123"))
    println(flag_tn("10"))
    println(tn_flag("company_zxr1"))
    val s = "[{\"name\":\"史某某\",\"litigant_id\":\"xx\"},{\"name\":\"伍新贵\",\"litigant_id\":\"\"}]\u0001[{\"name\":\"伍新贵\",\"litigant_id\":\"\"}]"
    println(all_name(s))
    val s1 = "[{\"name\":\"xxx\",\"litigant_id\":\"\",\"date\":\"2021\"},{\"name\":\"伍新贵\",\"litigant_id\":\"\",\"date\":\"2022\"}]"
    val s2 = "[{\"name\":\"\",\"litigant_id\":\"xx\"},{\"name\":\"何安平\",\"litigant_id\":\"222\"}]"
    val s3 = "[{\"name\":\"xxx\",\"litigant_id\":\"\",\"date\":\"2021\"},{\"name\":\"伍新贵\",\"litigant_id\":\"333\"},{\"name\":\"何安平\",\"litigant_id\":\"\"}]"

    //    val s1 = null
    //    val s2 = ""
    //    val s3 = ""

    println(merge_json(s1, s2, s3))
    //val str = ""
    //    val json_array = "[{\"name\": \"史某某\", \"litigant_id\": \"\"}, {\"name\": \"丁某某\", \"litigant_id\": \"\"}, {\"name\": \"杨某\", \"litigant_id\": \"\"}, {\"name\": \"陈某某\", \"litigant_id\": \"\"}]&[{\"name\": \"丁某某\", \"litigant_id\": \"\"}, {\"name\": \"史某某\", \"litigant_id\": \"\"}]&[{\"name\": \"杨某\", \"litigant_id\": \"\"}, {\"name\": \"陈某某\", \"litigant_id\": \"\"}]"
    //    val json_path = "$.name"
    //    println(split_names(json_array, json_path))
    //    println(nameCleanup("小米科技.；有,@限公  司  雷军"))
    //    println(title("xx", null, "reason"))
    //    println(parseAddress("大石桥市人民法院"))
    println(case_no_trim_v2("(2020)豫1728执541号之一"))
    //    val seq = Seq("1", "3", "2", "7").mkString("\001")
    //    println(sortString(seq))
    println(caseStage("(2019)鄂民申7号"))
    /*val yg_name = ",,"
    val bg_name = "张三,,小米,；"
    println(compareName(yg_name, bg_name))*/
  }

}
