package com.winhc.bigdata.spark.utils

import java.util.Date

import com.winhc.bigdata.spark.udf.CompanyMapping
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter

/**
 * π
 * cid转换
 */

case class CompanyForCidUtils(s: SparkSession, space: String, sourceTable: String, cols: Seq[String]) extends LoggingUtils with CompanyMapping {
  @(transient@getter) val spark: SparkSession = s

  val rowKeyMapping =
    Map("company_double_random_check_result_info" -> "new_cid,main_id" //双随机抽查-结果公示
    )

  def calc(): Unit = {
    println(s"${this.getClass.getSimpleName} calc start! " + new Date().toString)
    prepareFunctions(spark)
    val odsTable = s"${space}.ods_$sourceTable"
    val adsTable = s"${space}.ads_${sourceTable}"
    val companyMapping = s"${space}.company_name_mapping_pro_v2"
    val ds = BaseUtil.getPartion(odsTable, spark)
    //table字段
    val columns: Seq[String] = spark.table(odsTable).schema.map(_.name).filter(!_.equals("ds"))
    val disCol = cols

    val cols_md5 = disCol.filter(!_.equals("new_cid"))

    //rowkey前缀匹配
    val rowKeyPre = rowKeyMapping.getOrElse(sourceTable,"new_cid")

    val ddl = spark.table(odsTable).schema.filter(s => {
      !"ds".equals(s.name)
    }).map(s => {
      val name = s.name
      val dataType = s.dataType
      s"$name ${DataTypeUtils.getDataType(dataType)} COMMENT '${s.getComment().getOrElse("")}'\n"
    }).mkString(",")


    println(
      s"""
         |CREATE TABLE IF NOT EXISTS ${adsTable}
         |(
         |    rowkey  STRING COMMENT 'FIELD'
         |    ,new_cid STRING COMMENT 'FIELD'
         |    ,$ddl
         |)
         |COMMENT 'TABLE COMMENT'
         |PARTITIONED BY (ds STRING COMMENT '分区')
         |""".stripMargin)
    if (!spark.catalog.tableExists(adsTable)) {
      return
    }


    //替换字段
    sql(
      s"""
         |INSERT OVERWRITE TABLE ${adsTable} PARTITION(ds=${ds})
         |SELECT  rowkey,new_cid,${columns.mkString(",")}
         |FROM    (
         |        SELECT
         |                *
         |                ,ROW_NUMBER() OVER (PARTITION BY ${disCol.mkString(",")} ORDER BY id DESC ) num
         |                ,CONCAT_WS('_',$rowKeyPre,md5(cleanup(CONCAT_WS('',${cols_md5.mkString(",")})))) AS rowkey
         |                ,cleanup(CONCAT_WS('',${cols_md5.mkString(",")})) AS cols
         |        FROM    (
         |                SELECT
         |                        a.*
         |                        ,coalesce(b.new_cid,a.cid) AS new_cid
         |                FROM    $odsTable a
         |                LEFT JOIN $companyMapping b
         |                ON      a.cid = b.cid
         |                WHERE   a.ds = $ds AND a.cid IS NOT NULL
         |                ) c
         |        ) d
         |WHERE   num =1  AND cols is not null AND trim(cols) <> ''
         |""".stripMargin)
    //      .createOrReplaceTempView(s"t2")

    //    sql(s"select rowkey,new_cid,${columns.mkString(",")} from t2").show(10)

    //写表
    //    sql(s"insert into table ${adsTable} partition (ds=${ds}) select rowkey,new_cid,${columns.mkString(",")} from t2")

    println(s"${this.getClass.getSimpleName} calc end! " + new Date().toString)
  }
}