package com.winhc.bigdata.spark.utils

import java.util.Date

import com.winhc.bigdata.spark.udf.CompanyMapping
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter

/**
 * π
 * 拆平cids,落表
 */

case class CompanyForCidsUtils(s: SparkSession, space: String, sourceTable: String, cols: Seq[String]) extends LoggingUtils  with CompanyMapping{
  @(transient@getter) val spark: SparkSession = s

  import spark.implicits._
  import spark._
  import org.apache.spark.sql.functions._

  val tabMapping =
    Map("company_court_open_announcement" -> ("litigant_cids", ";") //开庭公告
    )

  def calc(): Unit = {
    println(s"${this.getClass.getSimpleName} calc start! " + new Date().toString)
    val odsTable = s"${space}.ods_$sourceTable"
    val adsListTable = s"${space}.ads_${sourceTable}_list"
    val adsTable = s"${space}.ads_$sourceTable"
    val companyMapping = s"${space}.company_name_mapping_pro_v2"
    prepareFunctions(spark)

    val ds = BaseUtil.getPartion(odsTable, spark)

    val (split_cols,delimiter) = tabMapping.getOrElse(sourceTable,("cids",";"))

    //table字段
    val columns: Seq[String] = spark.table(odsTable).schema.map(_.name).filter(!_.equals("ds"))
    val list_columns: Seq[String] = spark.table(adsListTable).schema.map(_.name).filter(!_.equals("ds"))
    val disCol = cols
    val cols_md5 = disCol.filter(!_.equals("new_cid"))

    //拆平新表
    sql(
      s"""
         |insert overwrite table ${adsListTable} partition (ds=${ds})
         |SELECT
         |        ${list_columns.mkString(",")}
         |FROM    (
         |        SELECT
         |                *
         |                ,ROW_NUMBER() OVER (PARTITION BY ${disCol.mkString(",")} ORDER BY - ABS(CAST(new_cid AS BIGINT )- CAST(cid AS BIGINT )) DESC ) num
         |                ,CONCAT_WS('_',new_cid,md5(cleanup(CONCAT_WS('',${cols_md5.mkString(",")})))) AS rowkey
         |                ,cleanup(CONCAT_WS('',${cols_md5.mkString(",")})) AS cols
         |        FROM    (
         |                SELECT
         |                        c.*
         |                        ,coalesce(d.new_cid,c.cid) AS new_cid
         |                FROM    (
         |                        SELECT
         |                                *
         |                        FROM    $odsTable a
         |                        LATERAL VIEW explode(split($split_cols,'$delimiter')) b AS cid
         |                        WHERE   a.ds = $ds
         |                        AND $split_cols is not null
         |                        AND trim($split_cols) <> ''
         |                        ) c
         |                LEFT JOIN $companyMapping d
         |                ON      c.cid = d.cid
         |                ) e
         |        ) f
         |WHERE   num =1  AND cols is not null AND trim(cols) <> ''
         |""".stripMargin)
      .createOrReplaceTempView(s"t2")

    //聚合新cids
    sql(
      s"""
         |insert overwrite table ${adsTable} partition (ds=${ds})
         |SELECT
         |md5(cleanup(CONCAT_WS('',${cols_md5.mkString(",")}))) AS rowkey
         |,x.new_cids
         |,${columns.mkString(",")}
         |FROM    $odsTable y
         |LEFT JOIN (
         |              SELECT  id as new_id
         |                      ,concat_ws(';',collect_set(new_cid)) new_cids
         |              FROM    $adsListTable t
         |              WHERE   ds = $ds
         |              GROUP BY id
         |          ) x
         |ON      y.id = x.new_id
         |WHERE   y.ds = 20200604
         |AND     $split_cols IS NOT NULL
         |AND     trim($split_cols) <> ''
         |""".stripMargin).createOrReplaceTempView("t3")

    println(s"${this.getClass.getSimpleName} calc end! " + new Date().toString)
  }
}