package com.winhc.bigdata.spark.utils

import java.util.Date

import com.winhc.bigdata.spark.udf.CompanyMapping
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter

/**
 * π
 * 增量cid转换
 */
case class CompanyIncrForCidUtils(s: SparkSession,
                                  project: String, //表所在工程名
                                  tableName: String, //表名（不加前后辍）
                                  dupliCols: Seq[String] // 去重列
                                 ) extends LoggingUtils with CompanyMapping {
  @(transient@getter) val spark: SparkSession = s
 //主键字段
  val rowKeyMapping =
    Map("company_double_random_check_result_info" -> "new_cid,main_id" //双随机抽查-结果公示
    )
//排序时间
  val updateTimeMapping =
    Map("wenshu_detail_combine" -> "update_date" //文书排序时间
    )

  def calc(): Unit = {
    println(s"${this.getClass.getSimpleName} calc start! " + new Date().toString)

    prepareFunctions(spark)

    val inc_ods_company = s"${project}.inc_ods_company" //每日公司基本信息增量
    val ads_company_tb = s"${project}.ads_${tableName}" //存量ads表
    val inc_ods_company_tb = s"${project}.inc_ods_$tableName" //增量ods表
    val inc_ads_company_tb = s"${project}.inc_ads_$tableName" //增量ads表



    //table字段
    val columns: Seq[String] = spark.table(ads_company_tb).schema.map(_.name).filter(s => {
      !s.equals("ds") && !s.equals("cid") && !s.equals("new_cid") && !s.equals("rowkey")
    })

    val colsSet = columns.toSet

    val ddl = spark.table(inc_ods_company_tb).schema.filter(s => colsSet.contains(s.name)).map(s => {
      val name = s.name
      val dataType = s.dataType
      s"$name ${DataTypeUtils.getDataType(dataType)} COMMENT '${s.getComment().getOrElse("")}'\n"
    }).mkString(",")

    println(
      s"""
         |CREATE TABLE IF NOT EXISTS ${inc_ads_company_tb}
         |(
         |    rowkey  STRING COMMENT 'FIELD'
         |    ,flag STRING COMMENT 'FIELD'
         |    ,new_cid STRING COMMENT 'FIELD'
         |    ,cid STRING COMMENT 'FIELD'
         |    ,$ddl
         |)
         |COMMENT 'TABLE COMMENT'
         |PARTITIONED BY (ds STRING COMMENT '分区')
         |""".stripMargin)

    if (!spark.catalog.tableExists(inc_ads_company_tb)) {
      return
    }

    //存量表ads最新分区
    val remainDs = BaseUtil.getPartion(ads_company_tb, spark)

    //增量ads最后一个分区
    val lastDsIncAds = BaseUtil.getPartion(inc_ads_company_tb, spark)

    val list = sql(s"show partitions $inc_ods_company_tb").collect.toList.map(_.getString(0).split("=")(1))
    //增量ods第一个分区
    val firstDsIncOds = list.head
    //增量ods最后一个分区//落表分区
    val lastDsIncOds = list.last
    //执行分区
    var runDs = ""
    //第一次run
    if (StringUtils.isBlank(lastDsIncAds)) {
      runDs = firstDsIncOds
    } else { //非第一次分区时间 + 1天
      runDs = BaseUtil.atDaysAfter(1, lastDsIncAds)
    }

    val cols_md5 = dupliCols.filter(!_.equals("new_cid"))

    //增量ods和增量ads最后一个分区相等,跳出
    if (lastDsIncOds.equals(lastDsIncAds)) {
      println("inc_ods equals inc_ads ds ,please delete last ds !!!")
      //runDs = lastDsIncOds
      val l1 = sql(s"show partitions $inc_ads_company_tb").collect.toList.map(_.getString(0).split("=")(1)).sorted
      if (l1.size > 1) {
        runDs = BaseUtil.atDaysAfter(1, l1(l1.size - 2))
      }else{
        runDs = firstDsIncOds
      }
      //sys.exit(-1)
    }

    println(
      s"""
         |cols_md5:$cols_md5
         |remainDs:$remainDs
         |lastDsIncOds:$lastDsIncOds
         |lastDsIncAds:$lastDsIncAds
         |runDs:$runDs
         |firstDsIncOds:$firstDsIncOds
         |""".stripMargin)

    //rowkey前缀匹配
    val rowKeyPre = rowKeyMapping.getOrElse(tableName,"new_cid")

    //排序时间
    val update_time = updateTimeMapping.getOrElse(tableName,"update_time")

    sql(
      s"""
         |SELECT  cid,current_cid as new_cid
         |FROM    ${inc_ods_company}
         |WHERE   ds >= ${runDs}
         |AND     cid IS NOT NULL
         |AND     current_cid IS NOT NULL
         |GROUP BY cid,current_cid
         |""".stripMargin).createOrReplaceTempView("mapping")


    sql(
      s"""
         |INSERT OVERWRITE TABLE ${inc_ads_company_tb} PARTITION(ds=$lastDsIncOds)
         |SELECT  rowkey
         |        ,flag
         |        ,new_cid
         |        ,cid
         |        ,${columns.mkString(",")}
         |FROM    (
         |            SELECT  CONCAT_WS('_',$rowKeyPre,md5(cleanup(CONCAT_WS('',${cols_md5.mkString(",")})))) AS rowkey
         |                    ,flag
         |                    ,new_cid
         |                    ,cid
         |                    ,${columns.mkString(",")}
         |                    ,ROW_NUMBER() OVER (PARTITION BY cleanup(CONCAT_WS('',${dupliCols.mkString(",")})) ORDER BY $update_time DESC ) num
         |            FROM    (
         |                        SELECT  "0" AS flag
         |                                ,a.new_cid
         |                                ,b.cid
         |                                ,${columns.mkString(",")}
         |                        FROM    mapping a
         |                        JOIN    (
         |                                    SELECT  new_cid AS cid
         |                                            ,${columns.mkString(",")}
         |                                    FROM    ${ads_company_tb}
         |                                    WHERE   ds >= ${remainDs}
         |                                    UNION ALL
         |                                    SELECT  new_cid AS cid
         |                                            ,${columns.mkString(",")}
         |                                    FROM    ${inc_ads_company_tb}
         |                                    WHERE   ds > ${remainDs} AND  ds <  ${runDs}
         |                                ) b
         |                        ON      a.cid = b.cid
         |                        UNION ALL
         |                        SELECT  "1" AS flag
         |                                ,coalesce(b.new_cid,a.cid) new_cid
         |                                ,CAST(a.cid AS STRING) AS cid
         |                                ,${columns.mkString(",")}
         |                        FROM    ${inc_ods_company_tb} a
         |                        LEFT JOIN mapping b
         |                        ON      a.cid = b.cid
         |                        WHERE   a.ds >= ${runDs}
         |                        AND     a.cid IS NOT NULL
         |                    ) d
         |        ) e
         |WHERE   num = 1
         |AND     cleanup(CONCAT_WS('',${cols_md5.mkString(",")})) IS NOT NULL
         |AND     trim(cleanup(CONCAT_WS('',${cols_md5.mkString(",")}))) <> ''
         |""".stripMargin)

    val colsTotal = columns ++ Seq("new_cid")

    MaxComputer2Phoenix(
      spark,
      colsTotal,
      inc_ads_company_tb,
      tableName,
      lastDsIncOds,
      s"CONCAT_WS('_',$rowKeyPre,md5(cleanup(CONCAT_WS('',${cols_md5.mkString(",")}))))"
    ).syn()

    println(s"${this.getClass.getSimpleName} calc end! " + new Date().toString)
  }


}