package com.winhc.bigdata.spark.utils

import com.winhc.bigdata.spark.const.EnvConst
import org.apache.http.HttpHost
import org.apache.http.auth.{AuthScope, UsernamePasswordCredentials}
import org.apache.http.entity.ContentType
import org.apache.http.impl.client.BasicCredentialsProvider
import org.apache.http.impl.nio.client.HttpAsyncClientBuilder
import org.apache.http.nio.entity.NStringEntity
import org.apache.http.util.EntityUtils
import org.apache.spark.internal.Logging
import org.elasticsearch.client.{RestClient, RestClientBuilder}

import java.util.Collections

/**
 * @author: XuJiakai
 * @date: 2021/4/28 17:46
 */
case class Alias(alias: String, index: String)

case class ElasticSearchIndexUtils() extends Logging {

  private lazy val client = getClient("es.eci.nodes")

  private def getClient(nodes: String): RestClient = {
    val credentialsProvider = new BasicCredentialsProvider();
    credentialsProvider.setCredentials(AuthScope.ANY,
      new UsernamePasswordCredentials("elastic", "elastic_168"))
    val restClient = RestClient.builder(new HttpHost(EnvConst.getEnv().getValue(nodes), 9200))
      .setHttpClientConfigCallback(new RestClientBuilder.HttpClientConfigCallback() {
        override def customizeHttpClient(httpClientBuilder: HttpAsyncClientBuilder): HttpAsyncClientBuilder = httpClientBuilder.setDefaultCredentialsProvider(credentialsProvider)
      }).build()
    restClient
  }


  def catIndices(): Seq[String] = {
    val res = get("_cat/indices?h=index", null)
    res.split("\n").toSeq
  }

  def catAliases(): Seq[Alias] = {
    val res = get("_cat/aliases?h=alias,index", null)
    res.split("\n").map(r => {
      val s = r.split(" +")
      Alias(s(0), s(1))
    }).toSeq
  }


  def updateAliases(add: Seq[Alias], remove: Seq[Alias]): Boolean = {
    val all = _aliases("add", add) ++ _aliases("remove", remove)
    val body = all.mkString("""{"actions":[""", ",", """]}""")
    try {
      post("/_aliases", body)
      true
    } catch {
      case e: Exception => logError(e.getMessage, e)
        false
    }
  }

  def delete(index: String): Unit = action("DELETE", index, null)


  def get(url: String, body: String): String = action("GET", url, body)

  def post(url: String, body: String): String = action("POST", url, body)

  def put(url: String, body: String): String = action("PUT", url, body)

  private def action(method: String, url: String, body: String): String = {
    val entity = body == null match {
      case true => null
      case false => new NStringEntity(body, ContentType.APPLICATION_JSON)
    }
    logInfo(
      s"""
         |- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         |ElasticSearch action:
         |
         |$method  $url
         |$body
         |
         |- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         |
         |""".stripMargin)

    val res = client.performRequest(
      method,
      url,
      Collections.emptyMap[String, String](),
      entity
    )
    return EntityUtils.toString(res.getEntity)
  }

  private def _aliases(`type`: String, list: Seq[Alias]): Seq[String] = if (list == null) Seq.empty else list.map(s => s"""{ "${`type`}": { "index": "${s.index}", "alias": "${s.alias}" }}""")
}

object ElasticSearchIndexUtils {

  def main(args: Array[String]): Unit = {
    val es = ElasticSearchIndexUtils()
  }
}
