package com.winhc.bigdata.spark.utils

import java.util.Collections

import com.alibaba.fastjson.JSON
import com.winhc.bigdata.spark.const.EnvConst
import org.apache.http.HttpHost
import org.apache.http.auth.{AuthScope, UsernamePasswordCredentials}
import org.apache.http.entity.ContentType
import org.apache.http.impl.client.BasicCredentialsProvider
import org.apache.http.impl.nio.client.HttpAsyncClientBuilder
import org.apache.http.nio.entity.NStringEntity
import org.apache.http.util.EntityUtils
import org.apache.spark.internal.Logging
import org.elasticsearch.client.{RestClient, RestClientBuilder}

/**
 * @Description:
 * @author π
 * @date 2020/12/5 11:14
 */
object EsQueryUtils extends Logging {
  def getClient(): RestClient = {
    val credentialsProvider = new BasicCredentialsProvider();
    credentialsProvider.setCredentials(AuthScope.ANY,
      new UsernamePasswordCredentials("elastic", "elastic_168"))
    // 单击所创建的Elasticsearch实例ID，在基本信息页面获取公网地址，即为HOST。
    val restClient = RestClient.builder(new HttpHost(EnvConst.getEnv().getValue("es.nodes"), 9200))
      .setHttpClientConfigCallback(new RestClientBuilder.HttpClientConfigCallback() {
        override def customizeHttpClient(httpClientBuilder: HttpAsyncClientBuilder): HttpAsyncClientBuilder = httpClientBuilder.setDefaultCredentialsProvider(credentialsProvider)
      }).build()
    restClient
  }

  def createTable(index: String, query: String, client: RestClient): Unit = {
    if (!tableExists(index, client)) {
      logInfo(query)
      try {
        val entity = new NStringEntity(query, ContentType.APPLICATION_JSON)
        val indexResponse = client.performRequest(
          "PUT",
          index,
          Collections.emptyMap[String, String](),
          entity)
        val res = EntityUtils.toString(indexResponse.getEntity)
        println(s"create table $index success .")
      } catch {
        case e: Exception => {
          println(s"create table $index error .")
          logError(e.getMessage, e)
        }
      }
    }
  }

  def tableExists(index: String, client: RestClient): Boolean = {
    try {
      val entity = new NStringEntity("{\"size\":1}", ContentType.APPLICATION_JSON)
      val indexResponse = client.performRequest(
        "GET",
        index,
        Collections.emptyMap[String, String](),
        entity)
      val res = EntityUtils.toString(indexResponse.getEntity)
      println(res)
      println(s"tableExists $index true .")
      true
    } catch {
      case e: Exception => {
        println(s"tableExists $index false .")
        //logError(e.getMessage, e)
        false
      }
    }
  }

  def getAliases(index: String, client: RestClient) = {
    try {
      import scala.collection.JavaConverters._
      val indexResponse1 = client.performRequest("GET", s"/$index/_alias/*")
      val list = JSON.parseObject(EntityUtils.toString(indexResponse1.getEntity)).keySet().asScala.toList.sorted
      println(s"getAliases $index success .")
      list
    } catch {
      case e: Exception => {
        println(s"getAliases $index error .")
        logError(e.getMessage, e)
        List()
      }
    }
  }

  def getIndexs(index: String, client: RestClient) = {
    try {
      val indexResponse1 = client.performRequest("GET", s"_cat/indices/$index*?h=index")
      val res = EntityUtils.toString(indexResponse1.getEntity)
      val list = res.split("\\n", -1).filter(_.length > 0).toList.sorted
      println(s"getIndexs $index success .")
      list
    } catch {
      case e: Exception => {
        println(s"getIndexs $index error .")
        logError(e.getMessage, e)
        List()
      }
    }
  }

  def deleteIndex(index: String, client: RestClient) = {
    try {
      val indexResponse1 = client.performRequest("DELETE", index)
      println(EntityUtils.toString(indexResponse1.getEntity))
      println(s"deleteIndex $index success .")
    } catch {
      case e: Exception => {
        println(s"deleteIndex $index error .")
        logError(e.getMessage, e)
      }
    }
  }

  def updateAliases(index: String, aliases: String, aliasesType: String, client: RestClient) = {
    val query =
      s"""
         |{
         |  "actions": [
         |    {
         |      "$aliasesType": {
         |        "index": "$index",
         |        "alias": "$aliases"
         |      }
         |    }
         |  ]
         |}
         |""".stripMargin

    try {
      val entity = new NStringEntity(query, ContentType.APPLICATION_JSON)
      val indexResponse = client.performRequest(
        "POST",
        "/_aliases/",
        Collections.emptyMap[String, String](),
        entity)
      println(query)
      println(s"updateAliases $index success .")
    } catch {
      case e: Exception => {
        println(s"updateAliases $index error .")
        logError(e.getMessage, e)
      }
    }
  }

  lazy val judicial_case_detail: String =
    """
      |{
      |  "settings": {
      |    "number_of_shards": 20,
      |    "number_of_replicas": 0,
      |    "index": {
      |      "mapping": {
      |        "nested_fields": {
      |          "limit": "300"
      |        }
      |      },
      |      "analysis": {
      |        "analyzer": {
      |          "ik_analyzer": {
      |            "tokenizer": "ik_max_tokenizer"
      |          }
      |        },
      |        "tokenizer": {
      |          "ik_max_tokenizer": {
      |            "type": "ik_max_word"
      |          }
      |        }
      |      }
      |    }
      |  },
      |  "mappings": {
      |    "judicial": {
      |      "properties": {
      |         "judicase_id": {
      |          "type": "keyword"
      |        },
      |        "title": {
      |          "type": "text",
      |          "fields": {
      |            "keyword": {
      |              "type": "keyword",
      |              "ignore_above": 256
      |            }
      |          },"analyzer": "ik_analyzer"
      |        },
      |        "case_type": {
      |          "type": "keyword"
      |        },
      |        "case_reason": {
      |          "type": "keyword"
      |        },
      |        "case_no": {
      |          "type": "keyword"
      |        },
      |        "court_name": {
      |          "type": "keyword"
      |        },
      |        "case_stage": {
      |          "type": "keyword"
      |        },
      |        "lable": {
      |          "type": "keyword"
      |        },
      |        "detail": {
      |          "properties": {
      |            "flag": {
      |              "type": "keyword"
      |            },
      |            "detail_id": {
      |              "type": "keyword"
      |            },
      |            "date": {
      |              "type": "date",
      |              "format": "yyyy-MM-dd HH:mm:ss||yyyy-MM-dd"
      |            },
      |            "name": {
      |              "type": "keyword"
      |            }
      |          }
      |        },
      |         "yg_name": {
      |          "type": "text",
      |          "fields": {
      |            "keyword": {
      |              "type": "keyword",
      |              "ignore_above": 256
      |            }
      |          },"analyzer": "ik_analyzer"
      |        },
      |        "bg_name": {
      |          "type": "text",
      |          "fields": {
      |            "keyword": {
      |              "type": "keyword",
      |              "ignore_above": 256
      |            }
      |          },"analyzer": "ik_analyzer"
      |        },
      |        "last_date": {
      |          "type": "date",
      |          "format": "yyyy-MM-dd HH:mm:ss||yyyy-MM-dd"
      |        },
      |        "deleted": {
      |          "type": "integer"
      |        }
      |      }
      |    }
      |  }
      |}
      |""".stripMargin

  lazy val judicial_case: String =
    """
      |{
      |  "settings": {
      |    "number_of_shards": 20,
      |    "number_of_replicas": 0,
      |    "index": {
      |      "mapping": {
      |        "nested_fields": {
      |          "limit": "300"
      |        }
      |      },
      |      "analysis": {
      |        "analyzer": {
      |          "ik_analyzer": {
      |            "tokenizer": "ik_max_tokenizer"
      |          }
      |        },
      |        "tokenizer": {
      |          "ik_max_tokenizer": {
      |            "type": "ik_max_word"
      |          }
      |        }
      |      }
      |    }
      |  },
      |  "mappings": {
      |    "judicial": {
      |      "properties": {
      |        "title": {
      |          "type": "text",
      |          "fields": {
      |            "keyword": {
      |              "type": "keyword",
      |              "ignore_above": 256
      |            }
      |          },
      |          "analyzer": "ik_analyzer"
      |        },
      |        "case_type": {
      |          "type": "keyword"
      |        },
      |        "case_reason": {
      |          "type": "keyword"
      |        },
      |        "case_no": {
      |          "type": "text",
      |          "fields": {
      |            "keyword": {
      |              "type": "keyword",
      |              "ignore_above": 256
      |            }
      |          },
      |          "analyzer": "ik_analyzer"
      |        },
      |        "court_name": {
      |          "type": "keyword"
      |        },
      |        "case_stage": {
      |          "type": "keyword"
      |        },
      |        "lable": {
      |          "type": "keyword"
      |        },
      |        "yg_name": {
      |          "type": "text",
      |          "fields": {
      |            "keyword": {
      |              "type": "keyword",
      |              "ignore_above": 256
      |            }
      |          },
      |          "analyzer": "ik_analyzer"
      |        },
      |        "bg_name": {
      |          "type": "text",
      |          "fields": {
      |            "keyword": {
      |              "type": "keyword",
      |              "ignore_above": 256
      |            }
      |          },
      |          "analyzer": "ik_analyzer"
      |        },
      |        "case_amt": {
      |          "type": "double"
      |        },
      |        "date": {
      |          "type": "date",
      |          "format": "yyyy-MM-dd HH:mm:ss||yyyy-MM-dd"
      |        },
      |        "court_level": {
      |          "type": "keyword"
      |        },
      |        "deleted": {
      |          "type": "integer"
      |        },
      |        "cids": {
      |          "type": "keyword"
      |        }
      |      }
      |    }
      |  }
      |}
      |""".stripMargin

  def main(args: Array[String]): Unit = {
    val client = EsQueryUtils.getClient()
    deleteIndex("xf_test02", client)
    //    updateAliases("xf_test02", "xf_test", "add", client)
    //    println(getAliases("xf_test", client))
    //EsQueryUtils.createTable("test05", EsQueryUtils.judicial_case_detail, client)
  }
}
