package com.winhc.bigdata.spark.utils

import java.util.Date

import org.apache.spark.sql.SparkSession

import scala.collection.mutable

/**
 * @Description: MaxComputer2Phoenix
 * @author π
 * @date 2020/6/2815:44
 */
case class MaxComputer2Phoenix(spark: SparkSession,
                               phoenixCols: Seq[String], //phoenix 列
                               adsTable: String, //odps表
                               htable: String, //hbase表
                               ds: String, //分区
                               rowkey: String //rowkey字段
                              ) extends LoggingUtils {
  //同步max到phoenix
  def syn() = {

    println(s"${htable} phoenix syn start! " + new Date().toString)

//    val resTable = s"TEST_${htable}"
    val resTable = s"${htable}"

    println("------------" + resTable + "---------")


    val key = s"$rowkey AS rowkey"
    var res = Seq[String]()
    if ("auction_tracking_list".equalsIgnoreCase(htable)){
      res = phoenixCols.map(s => {
        if ("NEW_CID".equals(s.toUpperCase())) {
          s"cast ($s as string) as CID"
        } else {
          s"cast ($s as string) as ${s.toUpperCase}"
        }
      }) ++ Seq(key)

    }else{
      res = phoenixCols.filter(!_.equalsIgnoreCase("id")).map(s => {
        if ("NEW_CID".equals(s.toUpperCase())) {
          s"cast ($s as string) as CID"
        } else {
          s"cast ($s as string) as ${s.toUpperCase}"
        }
      }) ++ Seq(key)
    }



    val df = sql(
      s"""
         |select
         |${res.mkString(", ")}
         |from
         |${adsTable}
         |where ds = $ds
         |""".stripMargin).repartition(100)

    import com.winhc.bigdata.spark.implicits.PhoenixHelper._
    df.save2PhoenixByJDBC(s"${resTable.toUpperCase}")

    println(s"${htable} phoenix syn end! " + new Date().toString)
  }

}

/**
 * max 同步到 phoenix
 */
object MaxComputer2Phoenix {
  def main(args: Array[String]): Unit = {

    val map = mutable.Map[String, String](
      "spark.hadoop.odps.spark.local.partition.amt" -> "10",
      "spark.hadoop.odps.project.name" -> "winhc_eci_dev"
    )
    val sparkSession = SparkUtils.InitEnv("scala spark on Phoenix5.x test", map)

    val Array(project,table) = args

    val odpsTable = "winhc_eci_dev.inc_ads_company_bid_list"
    val phoenixTable = "company_bid_list"
    val ds = "20200621"
    val sublistTableFieldName = sparkSession.table(odpsTable).columns.filter(s => {
      !s.equals("ds") && !s.equals("new_cid") && !s.equals("rowkey") && !s.equals("cids") && !s.equals("new_cids") && !s.equals("cid")
    }).seq

    val colsList = sublistTableFieldName ++ Seq("new_cid")

    MaxComputer2Phoenix(
      sparkSession,
      colsList,
      odpsTable,
      phoenixTable,
      "20200621",
      "CONCAT_WS('_',new_cid,id)"
    ).syn()

    sparkSession.stop()
  }


}
