package com.winhc.bigdata.spark.utils

import java.util.Date

import com.winhc.bigdata.spark.config.HBaseConfig
import org.apache.hadoop.hbase.client.Put
import org.apache.hadoop.hbase.io.ImmutableBytesWritable
import org.apache.hadoop.hbase.util.Bytes
import org.apache.spark.internal.Logging
import org.apache.spark.sql.{Column, DataFrame}

import scala.collection.mutable

/**
 * @Description: df导出到hbase
 * @author π
 * @date 2020/6/2911:18
 */
case class Maxcomputer2Hbase(dataFrame: DataFrame
                             , hbaseTable: String) extends Logging {
  lazy val f_bytes: Array[Byte] = Bytes.toBytes("F")

  def syn(): Unit = {
    println(s"${hbaseTable} HBASE syn start! " + new Date().toString)
    val jobConf = HBaseConfig.HBaseOutputJobConf(hbaseTable.toUpperCase)
    import org.apache.spark.sql.functions.col
    //df字段转化string
    val columns: Array[String] = dataFrame.columns
    val df2: DataFrame = dataFrame.select(columns.map(column => col(column).cast("string")): _*)
    //df2.printSchema()
    df2.rdd.map(row => {
      try {
        val rowkey = row.getAs[String]("rowkey")
        val put = new Put(Bytes.toBytes(rowkey))
        columns.filter(_ != "rowkey") map (c => {
          val value = row.getAs[String](c)
          put.addColumn(f_bytes, Bytes.toBytes(c), Bytes.toBytes(value))
        })
        (new ImmutableBytesWritable, put)
      } catch {
        case e: Exception => {
          logWarning(row.toString())
          logError(e.getMessage, e)
          null
        }
      }
    }).filter(_ != null).saveAsHadoopDataset(jobConf)
    println(s"${hbaseTable} HBASE syn start! " + new Date().toString)
  }

}

object Maxcomputer2Hbase {
  def main(args: Array[String]): Unit = {

    val map = mutable.Map[String, String]()
    val spark = SparkUtils.InitEnv("Maxcomputer2Hbase", map)
    import spark._
    val df1 = spark.createDataFrame(Seq(("1", "xx", 5), ("2", "yy", 3))).toDF("rowkey", "name", "age")
    df1.printSchema()
    df1.show(10)
    Maxcomputer2Hbase(df1, "TEST_HBASE").syn()
    spark.stop()
  }
}
