package com.winhc.bigdata.spark.utils

import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter

/**
 * @Author: XuJiakai
 * @Date: 2020/6/22 16:33
 * @Description:
 */
case class Odps2PhoenixUtils(s: SparkSession,
                             orgTableName: String,
                             phoenixTableName: String,
                             partitions: String,
                             ignoreCols: Seq[String],
                             renameCols: Map[String, String]
                            ) extends LoggingUtils {
  @(transient@getter) val spark: SparkSession = s

  def save(): Unit = {
    println(
      s"""
         |tableName : $orgTableName
         |phoenix tableName : $phoenixTableName
         |partitions : $partitions
         |ignore cols: $ignoreCols
         |renameCols : $renameCols
         |""".stripMargin)


    val fields = spark.table(orgTableName).schema.filter(!ignoreCols.contains(_)).map(schemaType => {
      val name = schemaType.name
      s"CAST($name as string) as ${renameCols.getOrElse(name, name)}"
    }).filter(_ != null).seq

    import com.winhc.bigdata.spark.implicits.PhoenixHelper._

    sql(
      s"""
         |select ${fields.mkString(",")}
         |form $orgTableName
         |where ds = '$partitions'
         |""".stripMargin).save2PhoenixByJDBC(phoenixTableName)
  }
}
