package com.winhc.bigdata.spark.utils

import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter

/**
 * @Author: XuJiakai
 * @Date: 2020/9/29 11:21
 * @Description: 跟据某个表，获取另一个表的ddl
 */
trait TableInitUtil {
  @(transient@getter) protected val spark: SparkSession

  def getDDL(org_tableName: String
             , target_tableName: String
             , addOtherCols: Seq[(String, String)] = Seq.empty
             , removeCols: Seq[String] = Seq.empty
             , addPartitions: Seq[(String, String)] = Seq(("ds", "STRING"))
            ): String = {
    val colsSet = addOtherCols.map(_._1).toSet
    val rmSet = removeCols.toSet

    val cols = addOtherCols.map(f => {
      s"${f._1} ${f._2} \n"
    }) ++
      spark.table(s"$org_tableName").schema
        .fields
        .filter(f => {
          !f.name.equals("ds")
        })
        .filter(f => {
          !colsSet.contains(f.name)
        })
        .filter(f => {
          !rmSet.contains(f.name)
        })
        .map(f => {
          val name = f.name
          val dataType = f.dataType
          s"$name ${DataTypeUtils.getDataType(dataType)} COMMENT '${f.getComment().getOrElse("")}'\n"
        })

    val str = addPartitions.map(f => {
      s"${f._1} ${f._2} COMMENT '分区'"
    }).mkString(",")

    s"""
       |CREATE TABLE IF NOT EXISTS $target_tableName
       |(
       |    ${cols.mkString(",")}
       |)
       |COMMENT 'TABLE COMMENT'
       |PARTITIONED BY ($str)
       |""".stripMargin
  }
}
