package com.winhc.bigdata.spark.utils

import java.util.concurrent.TimeUnit

import org.apache.commons.lang3.time.StopWatch

/**
 * @author π
 */
trait Watching extends LoggingUtils {

  def asyncWatch[T](name: String, f: () => Unit): Unit = {
    val t = new Thread(new Runnable {
      override def run(): Unit = {
        watch(name, f)
      }
    })
    t.setName(name)
    t.start()
  }

  def watch[T](name: String, fun: () => T): T = {
    val stopWatch = new StopWatch
    stopWatch.start()
    println(
      s"""
        |
        |----------------------- submit [action=$name] begin... -----------------------
        |""".stripMargin)
    logInfo(
      s"""
         |
         |----------------------- submit [action=$name] begin... -----------------------
       """.stripMargin
    )

    val r = fun()
    stopWatch.stop()
    val totalSeconds = stopWatch.getTime(TimeUnit.SECONDS)

    println(
      s"""
        |-------------- submit [action=$name] end, cost ${toTimeStr(totalSeconds)} -------------
        |
        |""".stripMargin)
    logInfo(
      s"""
         |-------------- submit [action=$name] end, cost ${toTimeStr(totalSeconds)} -------------
         |
       """.stripMargin
    )
    r
  }

  private def toTimeStr(totalSeconds: Long): String = {
    val hours = TimeUnit.SECONDS.toHours(totalSeconds)
    val minutes = TimeUnit.SECONDS.toMinutes(totalSeconds - hours * 3600)
    val seconds = totalSeconds - hours * 3600 - minutes * 60
    val timeStr = if (hours > 0) {
      s"${hours}h${minutes}m${seconds}s"
    } else if (minutes > 0) {
      s"${minutes}m${seconds}s"
    } else {
      s"${seconds}s"
    }
    timeStr
  }
}
